@@ -16,6 +16,7 @@
 import argparse
 import os.path
 
+import jinja2
 import libvirt
 
 templatedir = os.path.join(os.path.dirname(os.path.dirname(__file__)),
@@ -70,18 +71,27 @@ def main():
                         help="What boot device to use (hd/network).")
     parser.add_argument('--libvirt-nic-driver', default='virtio',
                         help='The libvirt network driver to use')
-    parser.add_argument('--bridge', default="br-seed",
-                        help='The linux bridge name to use for seeding \
-                                the baremetal pseudo-node\'s OS image')
+    parser.add_argument('--interface-count', default=1, type=int,
+                        help='The number of interfaces to add to VM.'),
+    parser.add_argument('--mac', default=None,
+                        help='The mac for the first interface on the vm')
     parser.add_argument('--console-log',
                         help='File to log console')
     parser.add_argument('--emulator', default=None,
                         help='Path to emulator bin for vm template')
     parser.add_argument('--disk-format', default='qcow2',
                         help='Disk format to use.')
+    parser.add_argument('--uefi-loader', default='',
+                        help='The absolute path of the UEFI firmware blob.')
+    parser.add_argument('--uefi-nvram', default='',
+                        help=('The absolute path of the non-volatile memory '
+                              'to store the UEFI variables. Should be used '
+                              'only when --uefi-loader is also specified.'))
     args = parser.parse_args()
-    with open(templatedir + '/vm.xml', 'rb') as f:
-        source_template = f.read()
+
+    env = jinja2.Environment(loader=jinja2.FileSystemLoader(templatedir))
+    template = env.get_template('vm.xml')
+
     params = {
         'name': args.name,
         'imagefile': args.image,
@@ -90,29 +100,38 @@ def main():
         'memory': args.memory,
         'cpus': args.cpus,
         'bootdev': args.bootdev,
-        'bridge': args.bridge,
+        'interface_count': args.interface_count,
+        'mac': args.mac,
         'nicdriver': args.libvirt_nic_driver,
         'emulator': args.emulator,
-        'disk_format': args.disk_format
+        'disk_format': args.disk_format,
+        'uefi_loader': args.uefi_loader,
+        'uefi_nvram': args.uefi_nvram,
     }
 
     if args.emulator:
         params['emulator'] = args.emulator
     else:
-        if os.path.exists("/usr/bin/kvm"):  # Debian
-            params['emulator'] = "/usr/bin/kvm"
-        elif os.path.exists("/usr/bin/qemu-kvm"):  # Redhat
-            params['emulator'] = "/usr/bin/qemu-kvm"
+        qemu_kvm_locations = ['/usr/bin/kvm',
+                              '/usr/bin/qemu-kvm',
+                              '/usr/libexec/qemu-kvm']
+        for location in qemu_kvm_locations:
+            if os.path.exists(location):
+                params['emulator'] = location
+                break
+        else:
+            raise RuntimeError("Unable to find location of kvm executable")
 
     if args.console_log:
         params['console'] = CONSOLE_LOG % {'console_log': args.console_log}
     else:
         params['console'] = CONSOLE_PTY
-    libvirt_template = source_template % params
+    libvirt_template = template.render(**params)
     conn = libvirt.open("qemu:///system")
 
     a = conn.defineXML(libvirt_template)
     print("Created machine %s with UUID %s" % (args.name, a.UUIDString()))
 
+
 if __name__ == '__main__':
     main()
