@@ -16,6 +16,7 @@
 import datetime
 
 from ironic_lib import metrics_utils
+from oslo_utils import uuidutils
 import pecan
 from pecan import rest
 from six.moves import http_client
@@ -26,6 +27,7 @@ from ironic.api.controllers import base
 from ironic.api.controllers import link
 from ironic.api.controllers.v1 import collection
 from ironic.api.controllers.v1 import node
+from ironic.api.controllers.v1 import notification_utils as notify
 from ironic.api.controllers.v1 import types
 from ironic.api.controllers.v1 import utils as api_utils
 from ironic.api import expose
@@ -172,7 +174,7 @@ class ChassisController(rest.RestController):
     invalid_sort_key_list = ['extra']
 
     def _get_chassis_collection(self, marker, limit, sort_key, sort_dir,
-                                resource_url=None, fields=None):
+                                resource_url=None, fields=None, detail=None):
         limit = api_utils.validate_limit(limit)
         sort_dir = api_utils.validate_sort_dir(sort_dir)
         marker_obj = None
@@ -188,17 +190,22 @@ class ChassisController(rest.RestController):
         chassis = objects.Chassis.list(pecan.request.context, limit,
                                        marker_obj, sort_key=sort_key,
                                        sort_dir=sort_dir)
+        parameters = {}
+        if detail is not None:
+            parameters['detail'] = detail
+
         return ChassisCollection.convert_with_links(chassis, limit,
                                                     url=resource_url,
                                                     fields=fields,
                                                     sort_key=sort_key,
-                                                    sort_dir=sort_dir)
+                                                    sort_dir=sort_dir,
+                                                    **parameters)
 
     @METRICS.timer('ChassisController.get_all')
     @expose.expose(ChassisCollection, types.uuid, int,
-                   wtypes.text, wtypes.text, types.listtype)
+                   wtypes.text, wtypes.text, types.listtype, types.boolean)
     def get_all(self, marker=None, limit=None, sort_key='id', sort_dir='asc',
-                fields=None):
+                fields=None, detail=None):
         """Retrieve a list of chassis.
 
         :param marker: pagination marker for large data sets.
@@ -211,14 +218,16 @@ class ChassisController(rest.RestController):
         :param fields: Optional, a list with a specified set of fields
             of the resource to be returned.
         """
-        cdict = pecan.request.context.to_dict()
+        cdict = pecan.request.context.to_policy_values()
         policy.authorize('baremetal:chassis:get', cdict, cdict)
 
         api_utils.check_allow_specify_fields(fields)
-        if fields is None:
-            fields = _DEFAULT_RETURN_FIELDS
+
+        fields = api_utils.get_request_return_fields(fields, detail,
+                                                     _DEFAULT_RETURN_FIELDS)
+
         return self._get_chassis_collection(marker, limit, sort_key, sort_dir,
-                                            fields=fields)
+                                            fields=fields, detail=detail)
 
     @METRICS.timer('ChassisController.detail')
     @expose.expose(ChassisCollection, types.uuid, int,
@@ -234,7 +243,7 @@ class ChassisController(rest.RestController):
         :param sort_key: column to sort results by. Default: id.
         :param sort_dir: direction to sort. "asc" or "desc". Default: asc.
         """
-        cdict = pecan.request.context.to_dict()
+        cdict = pecan.request.context.to_policy_values()
         policy.authorize('baremetal:chassis:get', cdict, cdict)
 
         # /detail should only work against collections
@@ -255,7 +264,7 @@ class ChassisController(rest.RestController):
         :param fields: Optional, a list with a specified set of fields
             of the resource to be returned.
         """
-        cdict = pecan.request.context.to_dict()
+        cdict = pecan.request.context.to_policy_values()
         policy.authorize('baremetal:chassis:get', cdict, cdict)
 
         api_utils.check_allow_specify_fields(fields)
@@ -270,12 +279,19 @@ class ChassisController(rest.RestController):
 
         :param chassis: a chassis within the request body.
         """
-        cdict = pecan.request.context.to_dict()
+        context = pecan.request.context
+        cdict = context.to_policy_values()
         policy.authorize('baremetal:chassis:create', cdict, cdict)
 
-        new_chassis = objects.Chassis(pecan.request.context,
-                                      **chassis.as_dict())
-        new_chassis.create()
+        # NOTE(yuriyz): UUID is mandatory for notifications payload
+        if not chassis.uuid:
+            chassis.uuid = uuidutils.generate_uuid()
+
+        new_chassis = objects.Chassis(context, **chassis.as_dict())
+        notify.emit_start_notification(context, new_chassis, 'create')
+        with notify.handle_error_notification(context, new_chassis, 'create'):
+            new_chassis.create()
+        notify.emit_end_notification(context, new_chassis, 'create')
         # Set the HTTP Location Header
         pecan.response.location = link.build_url('chassis', new_chassis.uuid)
         return Chassis.convert_with_links(new_chassis)
@@ -289,11 +305,11 @@ class ChassisController(rest.RestController):
         :param chassis_uuid: UUID of a chassis.
         :param patch: a json PATCH document to apply to this chassis.
         """
-        cdict = pecan.request.context.to_dict()
+        context = pecan.request.context
+        cdict = context.to_policy_values()
         policy.authorize('baremetal:chassis:update', cdict, cdict)
 
-        rpc_chassis = objects.Chassis.get_by_uuid(pecan.request.context,
-                                                  chassis_uuid)
+        rpc_chassis = objects.Chassis.get_by_uuid(context, chassis_uuid)
         try:
             chassis = Chassis(
                 **api_utils.apply_jsonpatch(rpc_chassis.as_dict(), patch))
@@ -313,7 +329,10 @@ class ChassisController(rest.RestController):
             if rpc_chassis[field] != patch_val:
                 rpc_chassis[field] = patch_val
 
-        rpc_chassis.save()
+        notify.emit_start_notification(context, rpc_chassis, 'update')
+        with notify.handle_error_notification(context, rpc_chassis, 'update'):
+            rpc_chassis.save()
+        notify.emit_end_notification(context, rpc_chassis, 'update')
         return Chassis.convert_with_links(rpc_chassis)
 
     @METRICS.timer('ChassisController.delete')
@@ -323,9 +342,12 @@ class ChassisController(rest.RestController):
 
         :param chassis_uuid: UUID of a chassis.
         """
-        cdict = pecan.request.context.to_dict()
+        context = pecan.request.context
+        cdict = context.to_policy_values()
         policy.authorize('baremetal:chassis:delete', cdict, cdict)
 
-        rpc_chassis = objects.Chassis.get_by_uuid(pecan.request.context,
-                                                  chassis_uuid)
-        rpc_chassis.destroy()
+        rpc_chassis = objects.Chassis.get_by_uuid(context, chassis_uuid)
+        notify.emit_start_notification(context, rpc_chassis, 'delete')
+        with notify.handle_error_notification(context, rpc_chassis, 'delete'):
+            rpc_chassis.destroy()
+        notify.emit_end_notification(context, rpc_chassis, 'delete')
