@@ -48,6 +48,8 @@ VERBS = {
     'abort': 'abort',
     'clean': 'clean',
     'adopt': 'adopt',
+    'rescue': 'rescue',
+    'unrescue': 'unrescue',
 }
 """ Mapping of state-changing events that are PUT to the REST API
 
@@ -162,13 +164,20 @@ INSPECTING = 'inspecting'
 """ Node is under inspection.
 
 This is the provision state used when inspection is started. A successfully
-inspected node shall transition to MANAGEABLE state.
+inspected node shall transition to MANAGEABLE state. For asynchronous
+inspection, node shall transition to INSPECTWAIT state.
 """
 
-
 INSPECTFAIL = 'inspect failed'
 """ Node inspection failed. """
 
+INSPECTWAIT = 'inspect wait'
+""" Node is under inspection.
+
+This is the provision state used when an asynchronous inspection is in
+progress. A successfully inspected node shall transition to MANAGEABLE state.
+"""
+
 ADOPTING = 'adopting'
 """ Node is being adopted.
 
@@ -185,16 +194,54 @@ potentially due to invalid or incompatible information being defined for the
 node.
 """
 
-UPDATE_ALLOWED_STATES = (DEPLOYFAIL, INSPECTING, INSPECTFAIL, CLEANFAIL, ERROR,
-                         VERIFYING, ADOPTFAIL)
+RESCUE = 'rescue'
+""" Node is in rescue mode. """
+
+RESCUEFAIL = 'rescue failed'
+""" Node rescue failed. """
+
+RESCUEWAIT = 'rescue wait'
+""" Node is waiting on an external callback.
+
+This will be the node `provision_state` while the node is waiting for
+the driver to finish rescuing the node.
+"""
+
+RESCUING = 'rescuing'
+""" Node is in process of being rescued. """
+
+UNRESCUEFAIL = 'unrescue failed'
+""" Node unrescue failed. """
+
+UNRESCUING = 'unrescuing'
+""" Node is being restored from rescue mode (to active state). """
+
+# NOTE(kaifeng): INSPECTING is allowed to keep backwards compatibility,
+# starting from API 1.39 node update is disallowed in this state.
+UPDATE_ALLOWED_STATES = (DEPLOYFAIL, INSPECTING, INSPECTFAIL, INSPECTWAIT,
+                         CLEANFAIL, ERROR, VERIFYING, ADOPTFAIL, RESCUEFAIL,
+                         UNRESCUEFAIL)
 """Transitional states in which we allow updating a node."""
 
 DELETE_ALLOWED_STATES = (AVAILABLE, MANAGEABLE, ENROLL, ADOPTFAIL)
 """States in which node deletion is allowed."""
 
-STABLE_STATES = (ENROLL, MANAGEABLE, AVAILABLE, ACTIVE, ERROR)
+STABLE_STATES = (ENROLL, MANAGEABLE, AVAILABLE, ACTIVE, ERROR, RESCUE)
 """States that will not transition unless receiving a request."""
 
+UNSTABLE_STATES = (DEPLOYING, DEPLOYWAIT, CLEANING, CLEANWAIT, VERIFYING,
+                   DELETING, INSPECTING, INSPECTWAIT, ADOPTING, RESCUING,
+                   RESCUEWAIT, UNRESCUING)
+"""States that can be changed without external request."""
+
+STUCK_STATES_TREATED_AS_FAIL = (DEPLOYING, CLEANING, VERIFYING, INSPECTING,
+                                ADOPTING, RESCUING, UNRESCUING)
+"""States that cannot be resumed once a conductor dies.
+
+If a node gets stuck with one of these states for some reason
+(eg. conductor goes down when executing task), node will be moved
+to fail state.
+"""
 
 ##############
 # Power states
@@ -209,6 +256,12 @@ POWER_OFF = 'power off'
 REBOOT = 'rebooting'
 """ Node is rebooting. """
 
+SOFT_REBOOT = 'soft rebooting'
+""" Node is rebooting gracefully. """
+
+SOFT_POWER_OFF = 'soft power off'
+""" Node is in the process of soft power off. """
+
 
 #####################
 # State machine model
@@ -224,6 +277,7 @@ def on_enter(new_state, event):
     LOG.debug("Entering new state '%s' in response to event '%s'",
               new_state, event)
 
+
 watchers = {}
 watchers['on_exit'] = on_exit
 watchers['on_enter'] = on_enter
@@ -258,11 +312,19 @@ machine.add_transition(AVAILABLE, DEPLOYING, 'deploy')
 # Add inspect* states.
 machine.add_state(INSPECTING, target=MANAGEABLE, **watchers)
 machine.add_state(INSPECTFAIL, target=MANAGEABLE, **watchers)
+machine.add_state(INSPECTWAIT, target=MANAGEABLE, **watchers)
 
 # Add adopt* states
 machine.add_state(ADOPTING, target=ACTIVE, **watchers)
 machine.add_state(ADOPTFAIL, target=ACTIVE, **watchers)
 
+# rescue states
+machine.add_state(RESCUING, target=RESCUE, **watchers)
+machine.add_state(RESCUEWAIT, target=RESCUE, **watchers)
+machine.add_state(RESCUEFAIL, target=RESCUE, **watchers)
+machine.add_state(UNRESCUING, target=ACTIVE, **watchers)
+machine.add_state(UNRESCUEFAIL, target=ACTIVE, **watchers)
+
 # A deployment may fail
 machine.add_transition(DEPLOYING, DEPLOYFAIL, 'fail')
 
@@ -351,6 +413,18 @@ machine.add_transition(INSPECTING, MANAGEABLE, 'done')
 # Inspection may fail.
 machine.add_transition(INSPECTING, INSPECTFAIL, 'fail')
 
+# Transition for asynchronous inspection
+machine.add_transition(INSPECTING, INSPECTWAIT, 'wait')
+
+# Inspection is done
+machine.add_transition(INSPECTWAIT, MANAGEABLE, 'done')
+
+# Inspection failed.
+machine.add_transition(INSPECTWAIT, INSPECTFAIL, 'fail')
+
+# Inspection is aborted.
+machine.add_transition(INSPECTWAIT, INSPECTFAIL, 'abort')
+
 # Move the node to manageable state for any other
 # action.
 machine.add_transition(INSPECTFAIL, MANAGEABLE, 'manage')
@@ -358,6 +432,59 @@ machine.add_transition(INSPECTFAIL, MANAGEABLE, 'manage')
 # Reinitiate the inspect after inspectfail.
 machine.add_transition(INSPECTFAIL, INSPECTING, 'inspect')
 
+# A provisioned node may have a rescue initiated.
+machine.add_transition(ACTIVE, RESCUING, 'rescue')
+
+# A rescue may succeed.
+machine.add_transition(RESCUING, RESCUE, 'done')
+
+# A rescue may also wait on external callbacks
+machine.add_transition(RESCUING, RESCUEWAIT, 'wait')
+machine.add_transition(RESCUEWAIT, RESCUING, 'resume')
+
+# A rescued node may be re-rescued.
+machine.add_transition(RESCUE, RESCUING, 'rescue')
+
+# A rescued node may be deleted.
+machine.add_transition(RESCUE, DELETING, 'delete')
+
+# A rescue may fail.
+machine.add_transition(RESCUEWAIT, RESCUEFAIL, 'fail')
+machine.add_transition(RESCUING, RESCUEFAIL, 'fail')
+
+# While waiting for a rescue step to be finished, rescuing may be aborted
+machine.add_transition(RESCUEWAIT, RESCUEFAIL, 'abort')
+
+# A failed rescue may be re-rescued.
+machine.add_transition(RESCUEFAIL, RESCUING, 'rescue')
+
+# A failed rescue may be unrescued.
+machine.add_transition(RESCUEFAIL, UNRESCUING, 'unrescue')
+
+# A failed rescue may be deleted.
+machine.add_transition(RESCUEFAIL, DELETING, 'delete')
+
+# A rescuewait node may be deleted.
+machine.add_transition(RESCUEWAIT, DELETING, 'delete')
+
+# A rescued node may be unrescued.
+machine.add_transition(RESCUE, UNRESCUING, 'unrescue')
+
+# An unrescuing node may succeed
+machine.add_transition(UNRESCUING, ACTIVE, 'done')
+
+# An unrescuing node may fail
+machine.add_transition(UNRESCUING, UNRESCUEFAIL, 'fail')
+
+# A failed unrescue may be re-rescued
+machine.add_transition(UNRESCUEFAIL, RESCUING, 'rescue')
+
+# A failed unrescue may be re-unrescued
+machine.add_transition(UNRESCUEFAIL, UNRESCUING, 'unrescue')
+
+# A failed unrescue may be deleted.
+machine.add_transition(UNRESCUEFAIL, DELETING, 'delete')
+
 # Start power credentials verification
 machine.add_transition(ENROLL, VERIFYING, 'manage')
 
