@@ -23,25 +23,27 @@ import datetime
 import errno
 import hashlib
 import os
-import random
 import re
 import shutil
 import tempfile
 
-import netaddr
+import jinja2
 from oslo_concurrency import processutils
 from oslo_log import log as logging
+from oslo_serialization import jsonutils
+from oslo_utils import netutils
 from oslo_utils import timeutils
-import paramiko
 import pytz
 import six
 
 from ironic.common import exception
-from ironic.common.i18n import _, _LE, _LW
+from ironic.common.i18n import _
 from ironic.conf import CONF
 
 LOG = logging.getLogger(__name__)
 
+warn_deprecated_extra_vif_port_id = False
+
 
 def _get_root_helper():
     # NOTE(jlvillal): This function has been moved to ironic-lib. And is
@@ -72,102 +74,11 @@ def execute(*cmd, **kwargs):
     result = processutils.execute(*cmd, **kwargs)
     LOG.debug('Execution completed, command line is "%s"',
               ' '.join(map(str, cmd)))
-    LOG.debug('Command stdout is: "%s"' % result[0])
-    LOG.debug('Command stderr is: "%s"' % result[1])
+    LOG.debug('Command stdout is: "%s"', result[0])
+    LOG.debug('Command stderr is: "%s"', result[1])
     return result
 
 
-def trycmd(*args, **kwargs):
-    """Convenience wrapper around oslo's trycmd() method."""
-    if kwargs.get('run_as_root') and 'root_helper' not in kwargs:
-        kwargs['root_helper'] = _get_root_helper()
-    return processutils.trycmd(*args, **kwargs)
-
-
-def ssh_connect(connection):
-    """Method to connect to a remote system using ssh protocol.
-
-    :param connection: a dict of connection parameters.
-    :returns: paramiko.SSHClient -- an active ssh connection.
-    :raises: SSHConnectFailed
-
-    """
-    try:
-        ssh = paramiko.SSHClient()
-        ssh.set_missing_host_key_policy(paramiko.AutoAddPolicy())
-        key_contents = connection.get('key_contents')
-        if key_contents:
-            data = six.moves.StringIO(key_contents)
-            if "BEGIN RSA PRIVATE" in key_contents:
-                pkey = paramiko.RSAKey.from_private_key(data)
-            elif "BEGIN DSA PRIVATE" in key_contents:
-                pkey = paramiko.DSSKey.from_private_key(data)
-            else:
-                # Can't include the key contents - secure material.
-                raise ValueError(_("Invalid private key"))
-        else:
-            pkey = None
-        ssh.connect(connection.get('host'),
-                    username=connection.get('username'),
-                    password=connection.get('password'),
-                    port=connection.get('port', 22),
-                    pkey=pkey,
-                    key_filename=connection.get('key_filename'),
-                    timeout=connection.get('timeout', 10))
-
-        # send TCP keepalive packets every 20 seconds
-        ssh.get_transport().set_keepalive(20)
-    except Exception as e:
-        LOG.debug("SSH connect failed: %s" % e)
-        raise exception.SSHConnectFailed(host=connection.get('host'))
-
-    return ssh
-
-
-def generate_uid(topic, size=8):
-    characters = '01234567890abcdefghijklmnopqrstuvwxyz'
-    choices = [random.choice(characters) for _x in range(size)]
-    return '%s-%s' % (topic, ''.join(choices))
-
-
-def random_alnum(size=32):
-    characters = '0123456789ABCDEFGHIJKLMNOPQRSTUVWXYZ'
-    return ''.join(random.choice(characters) for _ in range(size))
-
-
-def delete_if_exists(pathname):
-    """delete a file, but ignore file not found error."""
-
-    try:
-        os.unlink(pathname)
-    except OSError as e:
-        if e.errno == errno.ENOENT:
-            return
-        else:
-            raise
-
-
-def is_valid_boolstr(val):
-    """Check if the provided string is a valid bool string or not."""
-    boolstrs = ('true', 'false', 'yes', 'no', 'y', 'n', '1', '0')
-    return str(val).lower() in boolstrs
-
-
-def is_valid_mac(address):
-    """Verify the format of a MAC address.
-
-    Check if a MAC address is valid and contains six octets. Accepts
-    colon-separated format only.
-
-    :param address: MAC address to be validated.
-    :returns: True if valid. False if not.
-
-    """
-    m = "[0-9a-f]{2}(:[0-9a-f]{2}){5}$"
-    return (isinstance(address, six.string_types) and
-            re.match(m, address.lower()))
-
-
 def is_valid_datapath_id(datapath_id):
     """Verify the format of an OpenFlow datapath_id.
 
@@ -180,8 +91,8 @@ def is_valid_datapath_id(datapath_id):
 
     """
     m = "^[0-9a-f]{16}$"
-    return (isinstance(datapath_id, six.string_types) and
-            re.match(m, datapath_id.lower()))
+    return (isinstance(datapath_id, six.string_types)
+            and re.match(m, datapath_id.lower()))
 
 
 _is_valid_logical_name_re = re.compile(r'^[A-Z0-9-._~]+$', re.I)
@@ -243,7 +154,7 @@ def is_valid_no_proxy(no_proxy):
     """
     if not isinstance(no_proxy, six.string_types):
         return False
-    hostname_re = re.compile('(?!-)[A-Z\d-]{1,63}(?<!-)$', re.IGNORECASE)
+    hostname_re = re.compile('(?!-)[A-Z\\d-]{1,63}(?<!-)$', re.IGNORECASE)
     for hostname in no_proxy.split(','):
         hostname = hostname.strip().split(':')[0]
         if not hostname:
@@ -278,7 +189,7 @@ def validate_and_normalize_mac(address):
     :raises: InvalidMAC If the MAC address is not valid.
 
     """
-    if not is_valid_mac(address):
+    if not netutils.is_valid_mac(address):
         raise exception.InvalidMAC(mac=address)
     return address.lower()
 
@@ -300,78 +211,6 @@ def validate_and_normalize_datapath_id(datapath_id):
     return datapath_id.lower()
 
 
-def is_valid_ipv6_cidr(address):
-    try:
-        str(netaddr.IPNetwork(address, version=6).cidr)
-        return True
-    except Exception:
-        return False
-
-
-def get_shortened_ipv6(address):
-    addr = netaddr.IPAddress(address, version=6)
-    return str(addr.ipv6())
-
-
-def get_shortened_ipv6_cidr(address):
-    net = netaddr.IPNetwork(address, version=6)
-    return str(net.cidr)
-
-
-def get_ip_version(network):
-    """Returns the IP version of a network (IPv4 or IPv6).
-
-    :raises: AddrFormatError if invalid network.
-    """
-    if netaddr.IPNetwork(network).version == 6:
-        return "IPv6"
-    elif netaddr.IPNetwork(network).version == 4:
-        return "IPv4"
-
-
-def convert_to_list_dict(lst, label):
-    """Convert a value or list into a list of dicts."""
-    if not lst:
-        return None
-    if not isinstance(lst, list):
-        lst = [lst]
-    return [{label: x} for x in lst]
-
-
-def sanitize_hostname(hostname):
-    """Return a hostname which conforms to RFC-952 and RFC-1123 specs."""
-    if isinstance(hostname, six.text_type):
-        hostname = hostname.encode('latin-1', 'ignore')
-
-    hostname = re.sub(b'[ _]', b'-', hostname)
-    hostname = re.sub(b'[^\w.-]+', b'', hostname)
-    hostname = hostname.lower()
-    hostname = hostname.strip(b'.-')
-
-    return hostname
-
-
-def read_cached_file(filename, cache_info, reload_func=None):
-    """Read from a file if it has been modified.
-
-    :param cache_info: dictionary to hold opaque cache.
-    :param reload_func: optional function to be called with data when
-                        file is reloaded due to a modification.
-
-    :returns: data from file
-
-    """
-    mtime = os.path.getmtime(filename)
-    if not cache_info or mtime != cache_info.get('mtime'):
-        LOG.debug("Reloading cached file %s" % filename)
-        with open(filename) as fap:
-            cache_info['data'] = fap.read()
-        cache_info['mtime'] = mtime
-        if reload_func:
-            reload_func(cache_info['data'])
-    return cache_info['data']
-
-
 def _get_hash_object(hash_algo_name):
     """Create a hash object based on given algorithm.
 
@@ -402,60 +241,32 @@ def hash_file(file_like_object, hash_algo='md5'):
     :returns: a condensed digest of the bytes of contents.
     """
     checksum = _get_hash_object(hash_algo)
-    for chunk in iter(lambda: file_like_object.read(32768), b''):
-        checksum.update(chunk)
+    while True:
+        chunk = file_like_object.read(32768)
+        if not chunk:
+            break
+        encoded_chunk = (chunk.encode(encoding='utf-8')
+                         if isinstance(chunk, six.string_types) else chunk)
+        checksum.update(encoded_chunk)
     return checksum.hexdigest()
 
 
-@contextlib.contextmanager
-def temporary_mutation(obj, **kwargs):
-    """Temporarily change object attribute.
+def file_has_content(path, content, hash_algo='md5'):
+    """Checks that content of the file is the same as provided reference.
 
-    Temporarily set the attr on a particular object to a given value then
-    revert when finished.
-
-    One use of this is to temporarily set the read_deleted flag on a context
-    object:
-
-        with temporary_mutation(context, read_deleted="yes"):
-            do_something_that_needed_deleted_objects()
+    :param path: path to file
+    :param content: reference content to check against
+    :param hash_algo: hashing algo from hashlib to use, default is 'md5'
+    :returns: True if the hash of reference content is the same as
+        the hash of file's content, False otherwise
     """
-    def is_dict_like(thing):
-        return hasattr(thing, 'has_key')
-
-    def get(thing, attr, default):
-        if is_dict_like(thing):
-            return thing.get(attr, default)
-        else:
-            return getattr(thing, attr, default)
-
-    def set_value(thing, attr, val):
-        if is_dict_like(thing):
-            thing[attr] = val
-        else:
-            setattr(thing, attr, val)
-
-    def delete(thing, attr):
-        if is_dict_like(thing):
-            del thing[attr]
-        else:
-            delattr(thing, attr)
-
-    NOT_PRESENT = object()
-
-    old_values = {}
-    for attr, new_value in kwargs.items():
-        old_values[attr] = get(obj, attr, NOT_PRESENT)
-        set_value(obj, attr, new_value)
-
-    try:
-        yield
-    finally:
-        for attr, old_value in old_values.items():
-            if old_value is NOT_PRESENT:
-                delete(obj, attr)
-            else:
-                set_value(obj, attr, old_value)
+    with open(path, 'rb') as existing:
+        file_hash_hex = hash_file(existing, hash_algo=hash_algo)
+    ref_hash = _get_hash_object(hash_algo)
+    encoded_content = (content.encode(encoding='utf-8')
+                       if isinstance(content, six.string_types) else content)
+    ref_hash.update(encoded_content)
+    return file_hash_hex == ref_hash.hexdigest()
 
 
 @contextlib.contextmanager
@@ -468,7 +279,7 @@ def tempdir(**kwargs):
         try:
             shutil.rmtree(tmpdir)
         except OSError as e:
-            LOG.error(_LE('Could not remove tmpdir: %s'), e)
+            LOG.error('Could not remove tmpdir: %s', e)
 
 
 def rmtree_without_raise(path):
@@ -476,7 +287,7 @@ def rmtree_without_raise(path):
         if os.path.isdir(path):
             shutil.rmtree(path)
     except OSError as e:
-        LOG.warning(_LW("Failed to remove dir %(path)s, error: %(e)s"),
+        LOG.warning("Failed to remove dir %(path)s, error: %(e)s",
                     {'path': path, 'e': e})
 
 
@@ -492,10 +303,9 @@ def create_link_without_raise(source, link):
         if e.errno == errno.EEXIST:
             return
         else:
-            LOG.warning(
-                _LW("Failed to create symlink from %(source)s to %(link)s"
-                    ", error: %(e)s"),
-                {'source': source, 'link': link, 'e': e})
+            LOG.warning("Failed to create symlink from "
+                        "%(source)s to %(link)s, error: %(e)s",
+                        {'source': source, 'link': link, 'e': e})
 
 
 def safe_rstrip(value, chars=None):
@@ -507,9 +317,9 @@ def safe_rstrip(value, chars=None):
 
     """
     if not isinstance(value, six.string_types):
-        LOG.warning(_LW("Failed to remove trailing character. Returning "
-                        "original object. Supplied object is not a string: "
-                        "%s,"), value)
+        LOG.warning("Failed to remove trailing character. Returning "
+                    "original object. Supplied object is not a string: "
+                    "%s,", value)
         return value
 
     return value.rstrip(chars) or value
@@ -533,7 +343,7 @@ def umount(loc, *args):
     """Umounts a mounted location.
 
     :param loc: the path to be unmounted.
-    :param args: a tuple containing the argumnets to be
+    :param args: a tuple containing the arguments to be
         passed to the umount command.
     :raises: processutils.ProcessExecutionError if it failed
         to run the process.
@@ -663,15 +473,84 @@ def validate_network_port(port, port_name="Port"):
     :returns: An integer port number.
     :raises: InvalidParameterValue, if the port is invalid.
     """
-    try:
-        port = int(port)
-    except ValueError:
-        raise exception.InvalidParameterValue(_(
-            '%(port_name)s "%(port)s" is not a valid integer.') %
-            {'port_name': port_name, 'port': port})
-    if port < 1 or port > 65535:
-        raise exception.InvalidParameterValue(_(
-            '%(port_name)s "%(port)s" is out of range. Valid port '
-            'numbers must be between 1 and 65535.') %
-            {'port_name': port_name, 'port': port})
-    return port
+
+    if netutils.is_valid_port(port):
+        return int(port)
+
+    raise exception.InvalidParameterValue(_(
+        '%(port_name)s "%(port)s" is not a valid port.') %
+        {'port_name': port_name, 'port': port})
+
+
+def render_template(template, params, is_file=True):
+    """Renders Jinja2 template file with given parameters.
+
+    :param template: full path to the Jinja2 template file
+    :param params: dictionary with parameters to use when rendering
+    :param is_file: whether template is file or string with template itself
+    :returns: the rendered template as a string
+    """
+    if is_file:
+        tmpl_path, tmpl_name = os.path.split(template)
+        loader = jinja2.FileSystemLoader(tmpl_path)
+    else:
+        tmpl_name = 'template'
+        loader = jinja2.DictLoader({tmpl_name: template})
+    env = jinja2.Environment(loader=loader)
+    tmpl = env.get_template(tmpl_name)
+    return tmpl.render(params, enumerate=enumerate)
+
+
+def warn_about_deprecated_extra_vif_port_id():
+    global warn_deprecated_extra_vif_port_id
+    if not warn_deprecated_extra_vif_port_id:
+        warn_deprecated_extra_vif_port_id = True
+        LOG.warning("Starting with API version 1.28, attaching/detaching VIF "
+                    "to/from a port or port group via extra['vif_port_id'] is "
+                    "deprecated and will not be supported starting in the "
+                    "Stein release. API endpoint v1/nodes/<node>/vifs should "
+                    "be used instead.")
+
+
+def parse_instance_info_capabilities(node):
+    """Parse the instance_info capabilities.
+
+    One way of having these capabilities set is via Nova, where the
+    capabilities are defined in the Flavor extra_spec and passed to
+    Ironic by the Nova Ironic driver.
+
+    NOTE: Although our API fully supports JSON fields, to maintain the
+    backward compatibility with Juno the Nova Ironic driver is sending
+    it as a string.
+
+    :param node: a single Node.
+    :raises: InvalidParameterValue if the capabilities string is not a
+             dictionary or is malformed.
+    :returns: A dictionary with the capabilities if found, otherwise an
+              empty dictionary.
+    """
+
+    def parse_error():
+        error_msg = (_('Error parsing capabilities from Node %s instance_info '
+                       'field. A dictionary or a "jsonified" dictionary is '
+                       'expected.') % node.uuid)
+        raise exception.InvalidParameterValue(error_msg)
+
+    capabilities = node.instance_info.get('capabilities', {})
+    if isinstance(capabilities, six.string_types):
+        try:
+            capabilities = jsonutils.loads(capabilities)
+        except (ValueError, TypeError):
+            parse_error()
+
+    if not isinstance(capabilities, dict):
+        parse_error()
+
+    return capabilities
+
+
+def validate_conductor_group(conductor_group):
+    if not isinstance(conductor_group, six.string_types):
+        raise exception.InvalidConductorGroup(group=conductor_group)
+    if not re.match(r'^[a-zA-Z0-9_\-\.]*$', conductor_group):
+        raise exception.InvalidConductorGroup(group=conductor_group)
