@@ -22,23 +22,50 @@ from futurist import rejection
 from oslo_db import exception as db_exception
 from oslo_log import log
 from oslo_utils import excutils
+from oslo_utils import versionutils
+import six
 
 from ironic.common import context as ironic_context
 from ironic.common import driver_factory
 from ironic.common import exception
-from ironic.common import hash_ring as hash
-from ironic.common.i18n import _, _LC, _LE, _LI, _LW
+from ironic.common import hash_ring
+from ironic.common.i18n import _
+from ironic.common import release_mappings as versions
 from ironic.common import rpc
 from ironic.common import states
+from ironic.conductor import notification_utils as notify_utils
 from ironic.conductor import task_manager
 from ironic.conf import CONF
 from ironic.db import api as dbapi
+from ironic.drivers import base as driver_base
 from ironic import objects
+from ironic.objects import fields as obj_fields
 
 
 LOG = log.getLogger(__name__)
 
 
+def _check_enabled_interfaces():
+    """Sanity-check enabled_*_interfaces configs.
+
+    We do this before we even bother to try to load up drivers. If we have any
+    dynamic drivers enabled, then we need interfaces enabled as well.
+
+    :raises: ConfigInvalid if an enabled interfaces config option is empty.
+    """
+    empty_confs = []
+    iface_types = ['enabled_%s_interfaces' % i
+                   for i in driver_base.ALL_INTERFACES]
+    for iface_type in iface_types:
+        conf_value = getattr(CONF, iface_type)
+        if not conf_value:
+            empty_confs.append(iface_type)
+    if empty_confs:
+        msg = (_('Configuration options %s cannot be an empty list.') %
+               ', '.join(empty_confs))
+        raise exception.ConfigInvalid(error_msg=msg)
+
+
 class BaseConductorManager(object):
 
     def __init__(self, host, topic):
@@ -49,6 +76,7 @@ class BaseConductorManager(object):
         self.topic = topic
         self.sensors_notifier = rpc.get_sensors_notifier()
         self._started = False
+        self._shutdown = None
 
     def init_host(self, admin_context=None):
         """Initialize the conductor host.
@@ -58,10 +86,13 @@ class BaseConductorManager(object):
         :raises: NoDriversLoaded when no drivers are enabled on the conductor.
         :raises: DriverNotFound if a driver is enabled that does not exist.
         :raises: DriverLoadError if an enabled driver cannot be loaded.
+        :raises: DriverNameConflict if a classic driver and a dynamic driver
+                 are both enabled and have the same name.
         """
         if self._started:
             raise RuntimeError(_('Attempt to start an already running '
                                  'conductor manager'))
+        self._shutdown = False
 
         self.dbapi = dbapi.get_instance()
 
@@ -76,56 +107,40 @@ class BaseConductorManager(object):
             check_and_reject=rejection_func)
         """Executor for performing tasks async."""
 
-        self.ring_manager = hash.HashRingManager()
+        # TODO(jroll) delete the use_groups argument and use the default
+        # in Stein.
+        self.ring_manager = hash_ring.HashRingManager(
+            use_groups=self._use_groups())
         """Consistent hash ring which maps drivers to conductors."""
 
+        # TODO(dtantsur): remove in Stein
+        if CONF.enabled_drivers:
+            raise RuntimeError("The enabled_drivers configuration option "
+                               "no longer has any effect and must be empty")
+
+        _check_enabled_interfaces()
+
         # NOTE(deva): these calls may raise DriverLoadError or DriverNotFound
-        # NOTE(vdrok): instantiate network interface factory on startup so that
-        # all the network interfaces are loaded at the very beginning, and
-        # failures prevent the conductor from starting.
-        drivers = driver_factory.drivers()
+        # NOTE(vdrok): Instantiate network and storage interface factory on
+        # startup so that all the interfaces are loaded at the very
+        # beginning, and failures prevent the conductor from starting.
+        hardware_types = driver_factory.hardware_types()
         driver_factory.NetworkInterfaceFactory()
-        if not drivers:
-            msg = _LE("Conductor %s cannot be started because no drivers "
-                      "were loaded.  This could be because no drivers were "
-                      "specified in 'enabled_drivers' config option.")
-            LOG.error(msg, self.host)
-            raise exception.NoDriversLoaded(conductor=self.host)
+        driver_factory.StorageInterfaceFactory()
 
         # NOTE(jroll) this is passed to the dbapi, which requires a list, not
         # a generator (which keys() returns in py3)
-        driver_names = list(drivers)
+        hardware_type_names = list(hardware_types)
 
-        # Collect driver-specific periodic tasks.
-        # Conductor periodic tasks accept context argument, driver periodic
-        # tasks accept this manager and context. We have to ensure that the
-        # same driver interface class is not traversed twice, otherwise
-        # we'll have several instances of the same task.
-        LOG.debug('Collecting periodic tasks')
-        self._periodic_task_callables = []
-        periodic_task_classes = set()
-        self._collect_periodic_tasks(self, (admin_context,))
-        for driver_obj in drivers.values():
-            # TODO(dtantsur): collecting tasks from driver objects is
-            # deprecated and should be removed in Ocata.
-            self._collect_periodic_tasks(driver_obj, (self, admin_context))
-            for iface_name in driver_obj.all_interfaces:
-                iface = getattr(driver_obj, iface_name, None)
-                if iface and iface.__class__ not in periodic_task_classes:
-                    self._collect_periodic_tasks(iface, (self, admin_context))
-                    periodic_task_classes.add(iface.__class__)
+        # check that at least one driver is loaded, whether classic or dynamic
+        if not hardware_type_names:
+            msg = ("Conductor %s cannot be started because no hardware types "
+                   "were specified in the 'enabled_hardware_types' config "
+                   "option.")
+            LOG.error(msg, self.host)
+            raise exception.NoDriversLoaded(conductor=self.host)
 
-        if (len(self._periodic_task_callables) >
-                CONF.conductor.workers_pool_size):
-            LOG.warning(_LW('This conductor has %(tasks)d periodic tasks '
-                            'enabled, but only %(workers)d task workers '
-                            'allowed by [conductor]workers_pool_size option'),
-                        {'tasks': len(self._periodic_task_callables),
-                         'workers': CONF.conductor.workers_pool_size})
-
-        self._periodic_tasks = periodics.PeriodicWorker(
-            self._periodic_task_callables,
-            executor_factory=periodics.ExistingExecutor(self._executor))
+        self._collect_periodic_tasks(admin_context)
 
         # clear all target_power_state with locks by this conductor
         self.dbapi.clear_node_target_power_state(self.host)
@@ -134,16 +149,29 @@ class BaseConductorManager(object):
         try:
             # Register this conductor with the cluster
             self.conductor = objects.Conductor.register(
-                admin_context, self.host, driver_names)
+                admin_context, self.host, hardware_type_names,
+                CONF.conductor.conductor_group)
         except exception.ConductorAlreadyRegistered:
             # This conductor was already registered and did not shut down
             # properly, so log a warning and update the record.
-            LOG.warning(
-                _LW("A conductor with hostname %(hostname)s "
-                    "was previously registered. Updating registration"),
-                {'hostname': self.host})
+            LOG.warning("A conductor with hostname %(hostname)s was "
+                        "previously registered. Updating registration",
+                        {'hostname': self.host})
             self.conductor = objects.Conductor.register(
-                admin_context, self.host, driver_names, update_existing=True)
+                admin_context, self.host, hardware_type_names,
+                CONF.conductor.conductor_group, update_existing=True)
+
+        # register hardware types and interfaces supported by this conductor
+        # and validate them against other conductors
+        try:
+            self._register_and_validate_hardware_interfaces(hardware_types)
+        except (exception.DriverLoadError, exception.DriverNotFound,
+                exception.ConductorHardwareInterfacesAlreadyRegistered,
+                exception.InterfaceNotFoundInEntrypoint,
+                exception.NoValidDefaultForInterface) as e:
+            with excutils.save_and_reraise_exception():
+                LOG.error('Failed to register hardware types. %s', e)
+                self.del_host()
 
         # Start periodic tasks
         self._periodic_tasks_worker = self._executor.submit(
@@ -151,47 +179,114 @@ class BaseConductorManager(object):
         self._periodic_tasks_worker.add_done_callback(
             self._on_periodic_tasks_stop)
 
-        # NOTE(lucasagomes): If the conductor server dies abruptly
-        # mid deployment (OMM Killer, power outage, etc...) we
-        # can not resume the deployment even if the conductor
-        # comes back online. Cleaning the reservation of the nodes
-        # (dbapi.clear_node_reservations_for_conductor) is not enough to
-        # unstick it, so let's gracefully fail the deployment so the node
-        # can go through the steps (deleting & cleaning) to make itself
-        # available again.
-        filters = {'reserved': False,
-                   'provision_state': states.DEPLOYING}
-        last_error = (_("The deployment can't be resumed by conductor "
-                        "%s. Moving to fail state.") % self.host)
-        self._fail_if_in_state(ironic_context.get_admin_context(), filters,
-                               states.DEPLOYING, 'provision_updated_at',
-                               last_error=last_error)
+        for state in states.STUCK_STATES_TREATED_AS_FAIL:
+            self._fail_transient_state(
+                state,
+                _("The %(state)s state can't be resumed by conductor "
+                  "%(host)s. Moving to fail state.") %
+                {'state': state, 'host': self.host})
 
         # Start consoles if it set enabled in a greenthread.
         try:
             self._spawn_worker(self._start_consoles,
                                ironic_context.get_admin_context())
         except exception.NoFreeConductorWorker:
-            LOG.warning(_LW('Failed to start worker for restarting consoles.'))
+            LOG.warning('Failed to start worker for restarting consoles.')
 
         # Spawn a dedicated greenthread for the keepalive
         try:
             self._spawn_worker(self._conductor_service_record_keepalive)
-            LOG.info(_LI('Successfully started conductor with hostname '
-                         '%(hostname)s.'),
+            LOG.info('Successfully started conductor with hostname '
+                     '%(hostname)s.',
                      {'hostname': self.host})
         except exception.NoFreeConductorWorker:
             with excutils.save_and_reraise_exception():
-                LOG.critical(_LC('Failed to start keepalive'))
+                LOG.critical('Failed to start keepalive')
                 self.del_host()
 
         self._started = True
 
+    def _use_groups(self):
+        release_ver = versions.RELEASE_MAPPING.get(CONF.pin_release_version)
+        # NOTE(jroll) self.RPC_API_VERSION is actually defined in a subclass,
+        # but we only use this class from there.
+        version_cap = (release_ver['rpc'] if release_ver
+                       else self.RPC_API_VERSION)
+        return versionutils.is_compatible('1.47', version_cap)
+
+    def _fail_transient_state(self, state, last_error):
+        """Apply "fail" transition to nodes in a transient state.
+
+        If the conductor server dies abruptly mid deployment or cleaning
+        (OMM Killer, power outage, etc...) we can not resume the process even
+        if the conductor comes back online. Cleaning the reservation of
+        the nodes (dbapi.clear_node_reservations_for_conductor) is not enough
+        to unstick it, so let's gracefully fail the process.
+        """
+        filters = {'reserved': False, 'provision_state': state}
+        self._fail_if_in_state(ironic_context.get_admin_context(), filters,
+                               state, 'provision_updated_at',
+                               last_error=last_error)
+
+    def _collect_periodic_tasks(self, admin_context):
+        """Collect driver-specific periodic tasks.
+
+        Conductor periodic tasks accept context argument, driver periodic
+        tasks accept this manager and context. We have to ensure that the
+        same driver interface class is not traversed twice, otherwise
+        we'll have several instances of the same task.
+
+        :param admin_context: Administrator context to pass to tasks.
+        """
+        LOG.debug('Collecting periodic tasks')
+        # collected callables
+        periodic_task_callables = []
+        # list of visited classes to avoid adding the same tasks twice
+        periodic_task_classes = set()
+
+        def _collect_from(obj, args):
+            """Collect tasks from the given object.
+
+            :param obj: the object to collect tasks from.
+            :param args: a tuple of arguments to pass to tasks.
+            """
+            if obj and obj.__class__ not in periodic_task_classes:
+                for name, member in inspect.getmembers(obj):
+                    if periodics.is_periodic(member):
+                        LOG.debug('Found periodic task %(owner)s.%(member)s',
+                                  {'owner': obj.__class__.__name__,
+                                   'member': name})
+                        periodic_task_callables.append((member, args, {}))
+                periodic_task_classes.add(obj.__class__)
+
+        # First, collect tasks from the conductor itself
+        _collect_from(self, (admin_context,))
+
+        # Second, collect tasks from hardware interfaces
+        for ifaces in driver_factory.all_interfaces().values():
+            for iface in ifaces.values():
+                _collect_from(iface, args=(self, admin_context))
+        # TODO(dtantsur): allow periodics on hardware types themselves?
+
+        if len(periodic_task_callables) > CONF.conductor.workers_pool_size:
+            LOG.warning('This conductor has %(tasks)d periodic tasks '
+                        'enabled, but only %(workers)d task workers '
+                        'allowed by [conductor]workers_pool_size option',
+                        {'tasks': len(periodic_task_callables),
+                         'workers': CONF.conductor.workers_pool_size})
+
+        self._periodic_tasks = periodics.PeriodicWorker(
+            periodic_task_callables,
+            executor_factory=periodics.ExistingExecutor(self._executor))
+        # This is only used in tests currently. Delete it?
+        self._periodic_task_callables = periodic_task_callables
+
     def del_host(self, deregister=True):
         # Conductor deregistration fails if called on non-initialized
         # conductor (e.g. when rpc server is unreachable).
         if not hasattr(self, 'conductor'):
             return
+        self._shutdown = True
         self._keepalive_evt.set()
         if deregister:
             try:
@@ -199,14 +294,13 @@ class BaseConductorManager(object):
                 # Note that rebalancing will not occur immediately, but when
                 # the periodic sync takes place.
                 self.conductor.unregister()
-                LOG.info(_LI('Successfully stopped conductor with hostname '
-                             '%(hostname)s.'),
+                LOG.info('Successfully stopped conductor with hostname '
+                         '%(hostname)s.',
                          {'hostname': self.host})
             except exception.ConductorNotFound:
                 pass
         else:
-            LOG.info(_LI('Not deregistering conductor with hostname '
-                         '%(hostname)s.'),
+            LOG.info('Not deregistering conductor with hostname %(hostname)s.',
                      {'hostname': self.host})
         # Waiting here to give workers the chance to finish. This has the
         # benefit of releasing locks workers placed on nodes, as well as
@@ -216,29 +310,48 @@ class BaseConductorManager(object):
         self._executor.shutdown(wait=True)
         self._started = False
 
-    def _collect_periodic_tasks(self, obj, args):
-        """Collect periodic tasks from a given object.
+    def _register_and_validate_hardware_interfaces(self, hardware_types):
+        """Register and validate hardware interfaces for this conductor.
 
-        Populates self._periodic_task_callables with tuples
-        (callable, args, kwargs).
+        Registers a row in the database for each combination of
+        (hardware type, interface type, interface) that is supported and
+        enabled.
 
-        :param obj: object containing periodic tasks as methods
-        :param args: tuple with arguments to pass to every task
+        TODO: Validates against other conductors to check if the
+        set of registered hardware interfaces for a given hardware type is the
+        same, and warns if not (we can't error out, otherwise all conductors
+        must be restarted at once to change configuration).
+
+        :param hardware_types: Dictionary mapping hardware type name to
+                               hardware type object.
+        :raises: ConductorHardwareInterfacesAlreadyRegistered
+        :raises: InterfaceNotFoundInEntrypoint
+        :raises: NoValidDefaultForInterface if the default value cannot be
+                 calculated and is not provided in the configuration
         """
-        for name, member in inspect.getmembers(obj):
-            if periodics.is_periodic(member):
-                LOG.debug('Found periodic task %(owner)s.%(member)s',
-                          {'owner': obj.__class__.__name__,
-                           'member': name})
-                self._periodic_task_callables.append((member, args, {}))
+        # first unregister, in case we have cruft laying around
+        self.conductor.unregister_all_hardware_interfaces()
+
+        for ht_name, ht in hardware_types.items():
+            interface_map = driver_factory.enabled_supported_interfaces(ht)
+            for interface_type, interface_names in interface_map.items():
+                default_interface = driver_factory.default_interface(
+                    ht, interface_type, driver_name=ht_name)
+                self.conductor.register_hardware_interfaces(ht_name,
+                                                            interface_type,
+                                                            interface_names,
+                                                            default_interface)
+
+        # TODO(jroll) validate against other conductor, warn if different
+        # how do we do this performantly? :|
 
     def _on_periodic_tasks_stop(self, fut):
         try:
             fut.result()
         except Exception as exc:
-            LOG.critical(_LC('Periodic tasks worker has failed: %s'), exc)
+            LOG.critical('Periodic tasks worker has failed: %s', exc)
         else:
-            LOG.info(_LI('Successfully shut down periodic tasks'))
+            LOG.info('Successfully shut down periodic tasks')
 
     def iter_nodes(self, fields=None, **kwargs):
         """Iterate over nodes mapped to this conductor.
@@ -246,19 +359,23 @@ class BaseConductorManager(object):
         Requests node set from and filters out nodes that are not
         mapped to this conductor.
 
-        Yields tuples (node_uuid, driver, ...) where ... is derived from
-        fields argument, e.g.: fields=None means yielding ('uuid', 'driver'),
-        fields=['foo'] means yielding ('uuid', 'driver', 'foo').
+        Yields tuples (node_uuid, driver, conductor_group, ...) where ... is
+        derived from fields argument, e.g.: fields=None means yielding ('uuid',
+        'driver', 'conductor_group'), fields=['foo'] means yielding ('uuid',
+        'driver', 'conductor_group', 'foo').
 
-        :param fields: list of fields to fetch in addition to uuid and driver
+        :param fields: list of fields to fetch in addition to uuid, driver,
+                       and conductor_group
         :param kwargs: additional arguments to pass to dbapi when looking for
                        nodes
         :return: generator yielding tuples of requested fields
         """
-        columns = ['uuid', 'driver'] + list(fields or ())
+        columns = ['uuid', 'driver', 'conductor_group'] + list(fields or ())
         node_list = self.dbapi.get_nodeinfo_list(columns=columns, **kwargs)
         for result in node_list:
-            if self._mapped_to_this_conductor(*result[:2]):
+            if self._shutdown:
+                break
+            if self._mapped_to_this_conductor(*result[:3]):
                 yield result
 
     def _spawn_worker(self, func, *args, **kwargs):
@@ -282,11 +399,14 @@ class BaseConductorManager(object):
             try:
                 self.conductor.touch()
             except db_exception.DBConnectionError:
-                LOG.warning(_LW('Conductor could not connect to database '
-                                'while heartbeating.'))
+                LOG.warning('Conductor could not connect to database '
+                            'while heartbeating.')
+            except Exception as e:
+                LOG.exception('Error while heartbeating. Error: %(err)s',
+                              {'err': e})
             self._keepalive_evt.wait(CONF.conductor.heartbeat_interval)
 
-    def _mapped_to_this_conductor(self, node_uuid, driver):
+    def _mapped_to_this_conductor(self, node_uuid, driver, conductor_group):
         """Check that node is mapped to this conductor.
 
         Note that because mappings are eventually consistent, it is possible
@@ -295,11 +415,13 @@ class BaseConductorManager(object):
         take out a lock.
         """
         try:
-            ring = self.ring_manager[driver]
+            ring = self.ring_manager.get_ring(driver, conductor_group)
         except exception.DriverNotFound:
             return False
 
-        return self.host in ring.get_hosts(node_uuid)
+        return self.host in ring.get_nodes(
+            node_uuid.encode('utf-8'),
+            replicas=CONF.hash_distribution_replicas)
 
     def _fail_if_in_state(self, context, filters, provision_state,
                           sort_key, callback_method=None,
@@ -319,7 +441,8 @@ class BaseConductorManager(object):
                          this would process nodes whose provision_updated_at
                          field value was 60 or more seconds before 'now'.
         :param: provision_state: provision_state that the node is in,
-                                 for the provisioning activity to have failed.
+                                 for the provisioning activity to have failed,
+                                 either one string or a set.
         :param: sort_key: the nodes are sorted based on this key.
         :param: callback_method: the callback method to be invoked in a
                                  spawned thread, for a failed node. This
@@ -336,17 +459,21 @@ class BaseConductorManager(object):
                                    fsm.
 
         """
+        if isinstance(provision_state, six.string_types):
+            provision_state = {provision_state}
+
         node_iter = self.iter_nodes(filters=filters,
                                     sort_key=sort_key,
                                     sort_dir='asc')
 
         workers_count = 0
-        for node_uuid, driver in node_iter:
+        for node_uuid, driver, conductor_group in node_iter:
             try:
                 with task_manager.acquire(context, node_uuid,
                                           purpose='node state check') as task:
-                    if (task.node.maintenance or
-                            task.node.provision_state != provision_state):
+                    if (task.node.maintenance
+                            or task.node.provision_state
+                            not in provision_state):
                         continue
 
                     target_state = (None if not keep_target_state else
@@ -379,16 +506,22 @@ class BaseConductorManager(object):
 
         node_iter = self.iter_nodes(filters=filters)
 
-        for node_uuid, driver in node_iter:
+        for node_uuid, driver, conductor_group in node_iter:
             try:
                 with task_manager.acquire(context, node_uuid, shared=False,
                                           purpose='start console') as task:
+                    notify_utils.emit_console_notification(
+                        task, 'console_restore',
+                        obj_fields.NotificationStatus.START)
                     try:
                         LOG.debug('Trying to start console of node %(node)s',
                                   {'node': node_uuid})
                         task.driver.console.start_console(task)
-                        LOG.info(_LI('Successfully started console of node '
-                                     '%(node)s'), {'node': node_uuid})
+                        LOG.info('Successfully started console of node '
+                                 '%(node)s', {'node': node_uuid})
+                        notify_utils.emit_console_notification(
+                            task, 'console_restore',
+                            obj_fields.NotificationStatus.END)
                     except Exception as err:
                         msg = (_('Failed to start console of node %(node)s '
                                  'while starting the conductor, so changing '
@@ -401,14 +534,17 @@ class BaseConductorManager(object):
                         task.node.last_error = msg
                         task.node.console_enabled = False
                         task.node.save()
+                        notify_utils.emit_console_notification(
+                            task, 'console_restore',
+                            obj_fields.NotificationStatus.ERROR)
             except exception.NodeLocked:
-                LOG.warning(_LW('Node %(node)s is locked while trying to '
-                                'start console on conductor startup'),
+                LOG.warning('Node %(node)s is locked while trying to '
+                            'start console on conductor startup',
                             {'node': node_uuid})
                 continue
             except exception.NodeNotFound:
-                LOG.warning(_LW("During starting console on conductor "
-                                "startup, node %(node)s was not found"),
+                LOG.warning("During starting console on conductor "
+                            "startup, node %(node)s was not found",
                             {'node': node_uuid})
                 continue
             finally:
