@@ -25,8 +25,10 @@ import oslo_messaging as messaging
 from ironic.common import exception
 from ironic.common import hash_ring
 from ironic.common.i18n import _
+from ironic.common import release_mappings as versions
 from ironic.common import rpc
 from ironic.conductor import manager
+from ironic.conf import CONF
 from ironic.objects import base as objects_base
 
 
@@ -81,11 +83,26 @@ class ConductorAPI(object):
     |    1.32 - Add do_node_clean
     |    1.33 - Added update and destroy portgroup.
     |    1.34 - Added heartbeat
+    |    1.35 - Added destroy_volume_connector and update_volume_connector
+    |    1.36 - Added create_node
+    |    1.37 - Added destroy_volume_target and update_volume_target
+    |    1.38 - Added vif_attach, vif_detach, vif_list
+    |    1.39 - Added timeout optional parameter to change_node_power_state
+    |    1.40 - Added inject_nmi
+    |    1.41 - Added create_port
+    |    1.42 - Added optional agent_version to heartbeat
+    |    1.43 - Added do_node_rescue, do_node_unrescue and can_send_rescue
+    |    1.44 - Added add_node_traits and remove_node_traits.
+    |    1.45 - Added continue_node_deploy
+    |    1.46 - Added reset_interfaces to update_node
+    |    1.47 - Added support for conductor groups
 
     """
 
     # NOTE(rloo): This must be in sync with manager.ConductorManager's.
-    RPC_API_VERSION = '1.34'
+    # NOTE(pas-ha): This also must be in sync with
+    #               ironic.common.release_mappings.RELEASE_MAPPING['master']
+    RPC_API_VERSION = '1.47'
 
     def __init__(self, topic=None):
         super(ConductorAPI, self).__init__()
@@ -96,11 +113,15 @@ class ConductorAPI(object):
         target = messaging.Target(topic=self.topic,
                                   version='1.0')
         serializer = objects_base.IronicObjectSerializer()
-        self.client = rpc.get_client(target,
-                                     version_cap=self.RPC_API_VERSION,
+        release_ver = versions.RELEASE_MAPPING.get(CONF.pin_release_version)
+        version_cap = (release_ver['rpc'] if release_ver
+                       else self.RPC_API_VERSION)
+        self.client = rpc.get_client(target, version_cap=version_cap,
                                      serializer=serializer)
+
+        use_groups = self.client.can_send_version('1.47')
         # NOTE(deva): this is going to be buggy
-        self.ring_manager = hash_ring.HashRingManager()
+        self.ring_manager = hash_ring.HashRingManager(use_groups=use_groups)
 
     def get_topic_for(self, node):
         """Get the RPC topic for the conductor service the node is mapped to.
@@ -112,13 +133,20 @@ class ConductorAPI(object):
         """
         self.ring_manager.reset()
 
+        # There are no conductors, temporary failure - 503 Service Unavailable
+        if not self.ring_manager.ring:
+            raise exception.TemporaryFailure()
+
         try:
-            ring = self.ring_manager[node.driver]
-            dest = ring.get_hosts(node.uuid)
-            return self.topic + "." + dest[0]
+            ring = self.ring_manager.get_ring(node.driver,
+                                              node.conductor_group)
+            dest = ring.get_nodes(node.uuid.encode('utf-8'),
+                                  replicas=CONF.hash_distribution_replicas)
+            return '%s.%s' % (self.topic, dest.pop())
         except exception.DriverNotFound:
             reason = (_('No conductor service registered which supports '
-                        'driver %s.') % node.driver)
+                        'driver %(driver)s for conductor group "%(group)s".') %
+                      {'driver': node.driver, 'group': node.conductor_group})
             raise exception.NoValidHost(reason=reason)
 
     def get_topic_for_driver(self, driver_name):
@@ -133,13 +161,42 @@ class ConductorAPI(object):
         :raises: DriverNotFound
 
         """
-        self.ring_manager.reset()
-
-        hash_ring = self.ring_manager[driver_name]
-        host = random.choice(list(hash_ring.hosts))
+        # NOTE(jroll) we want to be able to route this to any conductor,
+        # regardless of groupings. We use a fresh, uncached hash ring that
+        # does not take groups into account.
+        local_ring_manager = hash_ring.HashRingManager(use_groups=False,
+                                                       cache=False)
+        ring = local_ring_manager.get_ring(driver_name, '')
+        host = random.choice(list(ring.nodes))
         return self.topic + "." + host
 
-    def update_node(self, context, node_obj, topic=None):
+    def can_send_create_port(self):
+        """Return whether the RPCAPI supports the create_port method."""
+        return self.client.can_send_version("1.41")
+
+    def can_send_rescue(self):
+        """Return whether the RPCAPI supports node rescue methods."""
+        return self.client.can_send_version("1.43")
+
+    def create_node(self, context, node_obj, topic=None):
+        """Synchronously, have a conductor validate and create a node.
+
+        Create the node's information in the database and return a node object.
+
+        :param context: request context.
+        :param node_obj: a created (but not saved) node object.
+        :param topic: RPC topic. Defaults to self.topic.
+        :returns: created node object.
+        :raises: InterfaceNotFoundInEntrypoint if validation fails for any
+                 dynamic interfaces (e.g. network_interface).
+        :raises: NoValidDefaultForInterface if no default can be calculated
+                 for some interfaces, and explicit values must be provided.
+        """
+        cctxt = self.client.prepare(topic=topic or self.topic, version='1.36')
+        return cctxt.call(context, 'create_node', node_obj=node_obj)
+
+    def update_node(self, context, node_obj, topic=None,
+                    reset_interfaces=False):
         """Synchronously, have a conductor update the node's information.
 
         Update the node's information in the database and return a node object.
@@ -154,13 +211,19 @@ class ConductorAPI(object):
         :param context: request context.
         :param node_obj: a changed (but not saved) node object.
         :param topic: RPC topic. Defaults to self.topic.
+        :param reset_interfaces: whether to reset hardware interfaces to their
+                                 defaults.
         :returns: updated node object, including all fields.
+        :raises: NoValidDefaultForInterface if no default can be calculated
+                 for some interfaces, and explicit values must be provided.
 
         """
         cctxt = self.client.prepare(topic=topic or self.topic, version='1.1')
-        return cctxt.call(context, 'update_node', node_obj=node_obj)
+        return cctxt.call(context, 'update_node', node_obj=node_obj,
+                          reset_interfaces=reset_interfaces)
 
-    def change_node_power_state(self, context, node_id, new_state, topic=None):
+    def change_node_power_state(self, context, node_id, new_state,
+                                topic=None, timeout=None):
         """Change a node's power state.
 
         Synchronously, acquire lock and start the conductor background task
@@ -169,14 +232,16 @@ class ConductorAPI(object):
         :param context: request context.
         :param node_id: node id or uuid.
         :param new_state: one of ironic.common.states power state values
+        :param timeout: timeout (in seconds) positive integer (> 0) for any
+           power state. ``None`` indicates to use default timeout.
         :param topic: RPC topic. Defaults to self.topic.
         :raises: NoFreeConductorWorker when there is no free worker to start
                  async task.
 
         """
-        cctxt = self.client.prepare(topic=topic or self.topic, version='1.6')
+        cctxt = self.client.prepare(topic=topic or self.topic, version='1.39')
         return cctxt.call(context, 'change_node_power_state', node_id=node_id,
-                          new_state=new_state)
+                          new_state=new_state, timeout=timeout)
 
     def vendor_passthru(self, context, node_id, driver_method, http_method,
                         info, topic=None):
@@ -240,6 +305,11 @@ class ConductorAPI(object):
         :raises: DriverNotFound if the supplied driver is not loaded.
         :raises: NoFreeConductorWorker when there is no free worker to start
                  async task.
+        :raises: InterfaceNotFoundInEntrypoint if the default interface for a
+                 hardware type is invalid.
+        :raises: NoValidDefaultForInterface if no default interface
+                 implementation can be found for this driver's vendor
+                 interface.
         :returns: A dictionary containing:
 
             :return: The response of the invoked vendor method
@@ -278,6 +348,14 @@ class ConductorAPI(object):
         :param context: an admin context.
         :param driver_name: name of the driver.
         :param topic: RPC topic. Defaults to self.topic.
+        :raises: UnsupportedDriverExtension if current driver does not have
+                 vendor interface.
+        :raises: DriverNotFound if the supplied driver is not loaded.
+        :raises: InterfaceNotFoundInEntrypoint if the default interface for a
+                 hardware type is invalid.
+        :raises: NoValidDefaultForInterface if no default interface
+                 implementation can be found for this driver's vendor
+                 interface.
         :returns: dictionary of <method name>:<method metadata> entries.
 
         """
@@ -360,6 +438,20 @@ class ConductorAPI(object):
         return cctxt.cast(context, 'continue_node_clean',
                           node_id=node_id)
 
+    def continue_node_deploy(self, context, node_id, topic=None):
+        """Signal to conductor service to start the next deployment action.
+
+        NOTE(rloo): this is an RPC cast, there will be no response or
+        exception raised by the conductor for this RPC.
+
+        :param context: request context.
+        :param node_id: node id or uuid.
+        :param topic: RPC topic. Defaults to self.topic.
+        """
+        cctxt = self.client.prepare(topic=topic or self.topic, version='1.45')
+        return cctxt.cast(context, 'continue_node_deploy',
+                          node_id=node_id)
+
     def validate_driver_interfaces(self, context, node_id, topic=None):
         """Validate the `core` and `standardized` interfaces for drivers.
 
@@ -379,6 +471,7 @@ class ConductorAPI(object):
 
         :param context: request context.
         :param node_id: node id or uuid.
+        :param topic: RPC topic. Defaults to self.topic.
         :raises: NodeLocked if node is locked by another conductor.
         :raises: NodeAssociated if the node contains an instance
             associated with it.
@@ -421,6 +514,21 @@ class ConductorAPI(object):
         return cctxt.call(context, 'set_console_mode', node_id=node_id,
                           enabled=enabled)
 
+    def create_port(self, context, port_obj, topic=None):
+        """Synchronously, have a conductor validate and create a port.
+
+        Create the port's information in the database and return a port object.
+        The conductor will lock related node and trigger specific driver
+        actions if they are needed.
+
+        :param context: request context.
+        :param port_obj: a created (but not saved) port object.
+        :param topic: RPC topic. Defaults to self.topic.
+        :returns: created port object.
+        """
+        cctxt = self.client.prepare(topic=topic or self.topic, version='1.41')
+        return cctxt.call(context, 'create_port', port_obj=port_obj)
+
     def update_port(self, context, port_obj, topic=None):
         """Synchronously, have a conductor update the port's information.
 
@@ -497,6 +605,7 @@ class ConductorAPI(object):
                        :mod:`ironic.common.boot_devices`.
         :param persistent: Whether to set next-boot, or make the change
                            permanent. Default: False.
+        :param topic: RPC topic. Defaults to self.topic.
         :raises: NodeLocked if node is locked by another conductor.
         :raises: UnsupportedDriverExtension if the node's driver doesn't
                  support management.
@@ -515,6 +624,7 @@ class ConductorAPI(object):
 
         :param context: request context.
         :param node_id: node id or uuid.
+        :param topic: RPC topic. Defaults to self.topic.
         :raises: NodeLocked if node is locked by another conductor.
         :raises: UnsupportedDriverExtension if the node's driver doesn't
                  support management.
@@ -532,6 +642,26 @@ class ConductorAPI(object):
         cctxt = self.client.prepare(topic=topic or self.topic, version='1.17')
         return cctxt.call(context, 'get_boot_device', node_id=node_id)
 
+    def inject_nmi(self, context, node_id, topic=None):
+        """Inject NMI for a node.
+
+        Inject NMI (Non Maskable Interrupt) for a node immediately.
+        Be aware that not all drivers support this.
+
+        :param context: request context.
+        :param node_id: node id or uuid.
+        :param topic: RPC topic. Defaults to self.topic.
+        :raises: NodeLocked if node is locked by another conductor.
+        :raises: UnsupportedDriverExtension if the node's driver doesn't
+                 support management or management.inject_nmi.
+        :raises: InvalidParameterValue when the wrong driver info is
+                 specified or an invalid boot device is specified.
+        :raises: MissingParameterValue if missing supplied info.
+
+        """
+        cctxt = self.client.prepare(topic=topic or self.topic, version='1.40')
+        return cctxt.call(context, 'inject_nmi', node_id=node_id)
+
     def get_supported_boot_devices(self, context, node_id, topic=None):
         """Get the list of supported devices.
 
@@ -539,6 +669,7 @@ class ConductorAPI(object):
 
         :param context: request context.
         :param node_id: node id or uuid.
+        :param topic: RPC topic. Defaults to self.topic.
         :raises: NodeLocked if node is locked by another conductor.
         :raises: UnsupportedDriverExtension if the node's driver doesn't
                  support management.
@@ -621,6 +752,11 @@ class ConductorAPI(object):
         :param topic: RPC topic. Defaults to self.topic.
         :raises: UnsupportedDriverExtension if the driver doesn't
             support RAID configuration.
+        :raises: InterfaceNotFoundInEntrypoint if the default interface for a
+                 hardware type is invalid.
+        :raises: NoValidDefaultForInterface if no default interface
+                 implementation can be found for this driver's RAID
+                 interface.
         :returns: A dictionary containing the properties that can be mentioned
             for logical disks and a textual description for them.
         """
@@ -647,17 +783,24 @@ class ConductorAPI(object):
         return cctxt.call(context, 'do_node_clean',
                           node_id=node_id, clean_steps=clean_steps)
 
-    def heartbeat(self, context, node_id, callback_url, topic=None):
+    def heartbeat(self, context, node_id, callback_url, agent_version,
+                  topic=None):
         """Process a node heartbeat.
 
         :param context: request context.
         :param node_id: node ID or UUID.
         :param callback_url: URL to reach back to the ramdisk.
         :param topic: RPC topic. Defaults to self.topic.
+        :param agent_version: the version of the agent that is heartbeating
         """
-        cctxt = self.client.prepare(topic=topic or self.topic, version='1.34')
+        new_kws = {}
+        version = '1.34'
+        if self.client.can_send_version('1.42'):
+            version = '1.42'
+            new_kws['agent_version'] = agent_version
+        cctxt = self.client.prepare(topic=topic or self.topic, version=version)
         return cctxt.call(context, 'heartbeat', node_id=node_id,
-                          callback_url=callback_url)
+                          callback_url=callback_url, **new_kws)
 
     def object_class_action_versions(self, context, objname, objmethod,
                                      object_versions, args, kwargs):
@@ -735,3 +878,209 @@ class ConductorAPI(object):
         cctxt = self.client.prepare(topic=self.topic, version='1.31')
         return cctxt.call(context, 'object_backport_versions', objinst=objinst,
                           object_versions=object_versions)
+
+    def destroy_volume_connector(self, context, connector, topic=None):
+        """Delete a volume connector.
+
+        Delete the volume connector. The conductor will lock the related node
+        during this operation.
+
+        :param context: request context
+        :param connector: volume connector object
+        :param topic: RPC topic. Defaults to self.topic.
+        :raises: NodeLocked if node is locked by another conductor
+        :raises: NodeNotFound if the node associated with the connector does
+                 not exist
+        :raises: VolumeConnectorNotFound if the volume connector cannot be
+                 found
+        """
+        cctxt = self.client.prepare(topic=topic or self.topic, version='1.35')
+        return cctxt.call(context, 'destroy_volume_connector',
+                          connector=connector)
+
+    def update_volume_connector(self, context, connector, topic=None):
+        """Update the volume connector's information.
+
+        Update the volume connector's information in the database and return
+        a volume connector object. The conductor will lock the related node
+        during this operation.
+
+        :param context: request context
+        :param connector: a changed (but not saved) volume connector object
+        :param topic: RPC topic. Defaults to self.topic.
+        :raises: InvalidParameterValue if the volume connector's UUID is being
+                 changed
+        :raises: NodeLocked if node is locked by another conductor
+        :raises: NodeNotFound if the node associated with the connector does
+                 not exist
+        :raises: VolumeConnectorNotFound if the volume connector cannot be
+                 found
+        :raises: VolumeConnectorTypeAndIdAlreadyExists if another connector
+                 already exists with the same values for type and connector_id
+                 fields
+        :returns: updated volume connector object, including all fields.
+
+        """
+        cctxt = self.client.prepare(topic=topic or self.topic, version='1.35')
+        return cctxt.call(context, 'update_volume_connector',
+                          connector=connector)
+
+    def destroy_volume_target(self, context, target, topic=None):
+        """Delete a volume target.
+
+        :param context: request context
+        :param target: volume target object
+        :param topic: RPC topic. Defaults to self.topic.
+        :raises: NodeLocked if node is locked by another conductor
+        :raises: NodeNotFound if the node associated with the target does
+                 not exist
+        :raises: VolumeTargetNotFound if the volume target cannot be found
+        """
+        cctxt = self.client.prepare(topic=topic or self.topic, version='1.37')
+        return cctxt.call(context, 'destroy_volume_target',
+                          target=target)
+
+    def update_volume_target(self, context, target, topic=None):
+        """Update the volume target's information.
+
+        Update the volume target's information in the database and return a
+        volume target object. The conductor will lock the related node during
+        this operation.
+
+        :param context: request context
+        :param target: a changed (but not saved) volume target object
+        :param topic: RPC topic. Defaults to self.topic.
+        :raises: InvalidParameterValue if the volume target's UUID is being
+                 changed
+        :raises: NodeLocked if the node is already locked
+        :raises: NodeNotFound if the node associated with the volume target
+                 does not exist
+        :raises: VolumeTargetNotFound if the volume target cannot be found
+        :raises: VolumeTargetBootIndexAlreadyExists if a volume target already
+                 exists with the same node ID and boot index values
+        :returns: updated volume target object, including all fields
+
+        """
+        cctxt = self.client.prepare(topic=topic or self.topic, version='1.37')
+        return cctxt.call(context, 'update_volume_target',
+                          target=target)
+
+    def vif_attach(self, context, node_id, vif_info, topic=None):
+        """Attach VIF to a node
+
+        :param context: request context.
+        :param node_id: node ID or UUID.
+        :param vif_info: a dictionary representing VIF object.
+            It must have an 'id' key, whose value is a unique
+            identifier for that VIF.
+        :param topic: RPC topic. Defaults to self.topic.
+        :raises: NodeLocked, if node has an exclusive lock held on it
+        :raises: NetworkError, if an error occurs during attaching the VIF.
+        :raises: InvalidParameterValue, if a parameter that's required for
+            VIF attach is wrong/missing.
+        """
+        cctxt = self.client.prepare(topic=topic or self.topic, version='1.38')
+        return cctxt.call(context, 'vif_attach', node_id=node_id,
+                          vif_info=vif_info)
+
+    def vif_detach(self, context, node_id, vif_id, topic=None):
+        """Detach VIF from a node
+
+        :param context: request context.
+        :param node_id: node ID or UUID.
+        :param vif_id: an ID of a VIF.
+        :param topic: RPC topic. Defaults to self.topic.
+        :raises: NodeLocked, if node has an exclusive lock held on it
+        :raises: NetworkError, if an error occurs during detaching the VIF.
+        :raises: InvalidParameterValue, if a parameter that's required for
+            VIF detach is wrong/missing.
+        """
+        cctxt = self.client.prepare(topic=topic or self.topic, version='1.38')
+        return cctxt.call(context, 'vif_detach', node_id=node_id,
+                          vif_id=vif_id)
+
+    def vif_list(self, context, node_id, topic=None):
+        """List attached VIFs for a node
+
+        :param context: request context.
+        :param node_id: node ID or UUID.
+        :param topic: RPC topic. Defaults to self.topic.
+        :returns: List of VIF dictionaries, each dictionary will have an
+            'id' entry with the ID of the VIF.
+        :raises: NetworkError, if an error occurs during listing the VIFs.
+        :raises: InvalidParameterValue, if a parameter that's required for
+            VIF list is wrong/missing.
+        """
+        cctxt = self.client.prepare(topic=topic or self.topic, version='1.38')
+        return cctxt.call(context, 'vif_list', node_id=node_id)
+
+    def do_node_rescue(self, context, node_id, rescue_password, topic=None):
+        """Signal to conductor service to perform a rescue.
+
+        :param context: request context.
+        :param node_id: node ID or UUID.
+        :param rescue_password: A string representing the password to be set
+            inside the rescue environment.
+        :param topic: RPC topic. Defaults to self.topic.
+        :raises: InstanceRescueFailure
+        :raises: NoFreeConductorWorker when there is no free worker to start
+                 async task.
+
+        The node must already be configured and in the appropriate
+        state before this method is called.
+
+        """
+        cctxt = self.client.prepare(topic=topic or self.topic, version='1.43')
+        return cctxt.call(context, 'do_node_rescue', node_id=node_id,
+                          rescue_password=rescue_password)
+
+    def do_node_unrescue(self, context, node_id, topic=None):
+        """Signal to conductor service to perform an unrescue.
+
+        :param context: request context.
+        :param node_id: node ID or UUID.
+        :param topic: RPC topic. Defaults to self.topic.
+        :raises: InstanceUnrescueFailure
+        :raises: NoFreeConductorWorker when there is no free worker to start
+                 async task.
+
+        The node must already be configured and in the appropriate
+        state before this method is called.
+
+        """
+        cctxt = self.client.prepare(topic=topic or self.topic, version='1.43')
+        return cctxt.call(context, 'do_node_unrescue', node_id=node_id)
+
+    def add_node_traits(self, context, node_id, traits, replace=False,
+                        topic=None):
+        """Add or replace traits for a node.
+
+        :param context: request context.
+        :param node_id: node ID or UUID.
+        :param traits: a list of traits to add to the node.
+        :param replace: True to replace all of the node's traits.
+        :param topic: RPC topic. Defaults to self.topic.
+        :raises: InvalidParameterValue if adding the traits would exceed the
+            per-node traits limit.
+        :raises: NodeLocked if node is locked by another conductor.
+        :raises: NodeNotFound if the node does not exist.
+        """
+        cctxt = self.client.prepare(topic=topic or self.topic, version='1.44')
+        return cctxt.call(context, 'add_node_traits', node_id=node_id,
+                          traits=traits, replace=replace)
+
+    def remove_node_traits(self, context, node_id, traits, topic=None):
+        """Remove some or all traits from a node.
+
+        :param context: request context.
+        :param node_id: node ID or UUID.
+        :param traits: a list of traits to remove from the node, or None. If
+            None, all traits will be removed from the node.
+        :param topic: RPC topic. Defaults to self.topic.
+        :raises: NodeLocked if node is locked by another conductor.
+        :raises: NodeNotFound if the node does not exist.
+        :raises: NodeTraitNotFound if one of the traits is not found.
+        """
+        cctxt = self.client.prepare(topic=topic or self.topic, version='1.44')
+        return cctxt.call(context, 'remove_node_traits', node_id=node_id,
+                          traits=traits)
