@@ -13,6 +13,7 @@
 # limitations under the License.
 
 from ironic_lib import metrics_utils
+from ironic_lib import utils as il_utils
 from oslo_log import log
 from oslo_utils import excutils
 from oslo_utils import units
@@ -21,8 +22,7 @@ import six.moves.urllib_parse as urlparse
 from ironic.common import dhcp_factory
 from ironic.common import exception
 from ironic.common.glance_service import service_utils
-from ironic.common.i18n import _, _LE, _LI, _LW
-from ironic.common import image_service
+from ironic.common.i18n import _
 from ironic.common import images
 from ironic.common import raid
 from ironic.common import states
@@ -32,6 +32,7 @@ from ironic.conductor import utils as manager_utils
 from ironic.conf import CONF
 from ironic.drivers import base
 from ironic.drivers.modules import agent_base_vendor
+from ironic.drivers.modules import boot_mode_utils
 from ironic.drivers.modules import deploy_utils
 
 
@@ -53,7 +54,7 @@ OPTIONAL_PROPERTIES = {
                            'Optional.'),
     'image_no_proxy': _('A comma-separated list of host names, IP addresses '
                         'and domain names (with optional :port) that will be '
-                        'excluded from proxying. To denote a doman name, use '
+                        'excluded from proxying. To denote a domain name, use '
                         'a dot to prefix the domain name. This value will be '
                         'ignored if ``image_http_proxy`` and '
                         '``image_https_proxy`` are not specified. Optional.'),
@@ -61,6 +62,7 @@ OPTIONAL_PROPERTIES = {
 
 COMMON_PROPERTIES = REQUIRED_PROPERTIES.copy()
 COMMON_PROPERTIES.update(OPTIONAL_PROPERTIES)
+COMMON_PROPERTIES.update(agent_base_vendor.VENDOR_PROPERTIES)
 
 PARTITION_IMAGE_LABELS = ('kernel', 'ramdisk', 'root_gb', 'root_mb', 'swap_mb',
                           'ephemeral_mb', 'ephemeral_format', 'configdrive',
@@ -68,56 +70,6 @@ PARTITION_IMAGE_LABELS = ('kernel', 'ramdisk', 'root_gb', 'root_mb', 'swap_mb',
                           'deploy_boot_mode')
 
 
-@METRICS.timer('build_instance_info_for_deploy')
-def build_instance_info_for_deploy(task):
-    """Build instance_info necessary for deploying to a node.
-
-    :param task: a TaskManager object containing the node
-    :returns: a dictionary containing the properties to be updated
-        in instance_info
-    :raises: exception.ImageRefValidationFailed if image_source is not
-        Glance href and is not HTTP(S) URL.
-    """
-    node = task.node
-    instance_info = node.instance_info
-    iwdi = node.driver_internal_info.get('is_whole_disk_image')
-    image_source = instance_info['image_source']
-    if service_utils.is_glance_image(image_source):
-        glance = image_service.GlanceImageService(version=2,
-                                                  context=task.context)
-        image_info = glance.show(image_source)
-        swift_temp_url = glance.swift_temp_url(image_info)
-        LOG.debug('Got image info: %(info)s for node %(node)s.',
-                  {'info': image_info, 'node': node.uuid})
-        instance_info['image_url'] = swift_temp_url
-        instance_info['image_checksum'] = image_info['checksum']
-        instance_info['image_disk_format'] = image_info['disk_format']
-        instance_info['image_container_format'] = (
-            image_info['container_format'])
-
-        if not iwdi:
-            instance_info['kernel'] = image_info['properties']['kernel_id']
-            instance_info['ramdisk'] = image_info['properties']['ramdisk_id']
-    else:
-        try:
-            image_service.HttpImageService().validate_href(image_source)
-        except exception.ImageRefValidationFailed:
-            with excutils.save_and_reraise_exception():
-                LOG.error(_LE("Agent deploy supports only HTTP(S) URLs as "
-                              "instance_info['image_source']. Either %s "
-                              "is not a valid HTTP(S) URL or "
-                              "is not reachable."), image_source)
-        instance_info['image_url'] = image_source
-
-    if not iwdi:
-        instance_info['image_type'] = 'partition'
-        i_info = deploy_utils.parse_instance_info(node)
-        instance_info.update(i_info)
-    else:
-        instance_info['image_type'] = 'whole-disk-image'
-    return instance_info
-
-
 @METRICS.timer('check_image_size')
 def check_image_size(task, image_source):
     """Check if the requested image is larger than the ram size.
@@ -131,8 +83,8 @@ def check_image_size(task, image_source):
     properties = node.properties
     # skip check if 'memory_mb' is not defined
     if 'memory_mb' not in properties:
-        LOG.warning(_LW('Skip the image size check as memory_mb is not '
-                        'defined in properties on node %s.'), node.uuid)
+        LOG.warning('Skip the image size check as memory_mb is not '
+                    'defined in properties on node %s.', node.uuid)
         return
 
     image_show = images.image_show(task.context, image_source)
@@ -264,7 +216,7 @@ class AgentDeployMixin(agent_base_vendor.AgentDeployMixin):
             for label in PARTITION_IMAGE_LABELS:
                 image_info[label] = node.instance_info.get(label)
             boot_option = deploy_utils.get_boot_option(node)
-            boot_mode = deploy_utils.get_boot_mode_for_deploy(node)
+            boot_mode = boot_mode_utils.get_boot_mode_for_deploy(node)
             if boot_mode:
                 image_info['deploy_boot_mode'] = boot_mode
             else:
@@ -313,6 +265,7 @@ class AgentDeployMixin(agent_base_vendor.AgentDeployMixin):
         task.process_event('resume')
         node = task.node
         iwdi = task.node.driver_internal_info.get('is_whole_disk_image')
+        cpu_arch = task.node.properties.get('cpu_arch')
         error = self.check_deploy_success(node)
         if error is not None:
             # TODO(jimrollenhagen) power off if using neutron dhcp to
@@ -322,33 +275,72 @@ class AgentDeployMixin(agent_base_vendor.AgentDeployMixin):
             LOG.error(msg)
             deploy_utils.set_failed_state(task, msg)
             return
-        if not iwdi:
-            root_uuid = self._get_uuid_from_result(task, 'root_uuid')
-            if deploy_utils.get_boot_mode_for_deploy(node) == 'uefi':
-                efi_sys_uuid = (
-                    self._get_uuid_from_result(task,
-                                               'efi_system_partition_uuid'))
-            else:
-                efi_sys_uuid = None
-            task.node.driver_internal_info['root_uuid_or_disk_id'] = root_uuid
+
+        # If `boot_option` is set to `netboot`, PXEBoot.prepare_instance()
+        # would need root_uuid of the whole disk image to add it into the
+        # pxe config to perform chain boot.
+        # IPA would have returned us the 'root_uuid_or_disk_id' if image
+        # being provisioned is a whole disk image. IPA would also provide us
+        # 'efi_system_partition_uuid' if the image being provisioned is a
+        # partition image.
+        # In case of local boot using partition image, we need both
+        # 'root_uuid_or_disk_id' and 'efi_system_partition_uuid' to configure
+        # bootloader for local boot.
+        # NOTE(mjturek): In the case of local boot using a partition image on
+        # ppc64* hardware we need to provide the 'PReP_Boot_partition_uuid' to
+        # direct where the bootloader should be installed.
+        driver_internal_info = task.node.driver_internal_info
+        root_uuid = self._get_uuid_from_result(task, 'root_uuid')
+        if root_uuid:
+            driver_internal_info['root_uuid_or_disk_id'] = root_uuid
+            task.node.driver_internal_info = driver_internal_info
             task.node.save()
-            self.prepare_instance_to_boot(task, root_uuid, efi_sys_uuid)
-        LOG.info(_LI('Image successfully written to node %s'), node.uuid)
-        LOG.debug('Rebooting node %s to instance', node.uuid)
-        if iwdi:
+        elif iwdi and CONF.agent.manage_agent_boot:
+            # IPA version less than 3.1.0 will not return root_uuid for
+            # whole disk image. Also IPA version introduced a requirement
+            # for hexdump utility that may not be always available. Need to
+            # fall back to older behavior for the same.
+            LOG.warning("With the deploy ramdisk based on Ironic Python Agent "
+                        "version 3.1.0 and beyond, the drivers using "
+                        "`direct` deploy interface performs `netboot` or "
+                        "`local` boot for whole disk image based on value "
+                        "of boot option setting. When you upgrade Ironic "
+                        "Python Agent in your deploy ramdisk, ensure that "
+                        "boot option is set appropriately for the node %s. "
+                        "The boot option can be set using configuration "
+                        "`[deploy]/default_boot_option` or as a `boot_option` "
+                        "capability in node's `properties['capabilities']`. "
+                        "Also please note that this functionality requires "
+                        "`hexdump` command in the ramdisk.", node.uuid)
+
+        efi_sys_uuid = None
+        if not iwdi:
+            if boot_mode_utils.get_boot_mode_for_deploy(node) == 'uefi':
+                efi_sys_uuid = (self._get_uuid_from_result(task,
+                                'efi_system_partition_uuid'))
+
+        prep_boot_part_uuid = None
+        if cpu_arch is not None and cpu_arch.startswith('ppc64'):
+            prep_boot_part_uuid = (self._get_uuid_from_result(task,
+                                   'PReP_Boot_partition_uuid'))
+
+        LOG.info('Image successfully written to node %s', node.uuid)
+
+        if CONF.agent.manage_agent_boot:
+            # It is necessary to invoke prepare_instance() of the node's
+            # boot interface, so that the any necessary configurations like
+            # setting of the boot mode (e.g. UEFI secure boot) which cannot
+            # be done on node during deploy stage can be performed.
+            LOG.debug('Executing driver specific tasks before booting up the '
+                      'instance for node %s', node.uuid)
+            self.prepare_instance_to_boot(task, root_uuid,
+                                          efi_sys_uuid, prep_boot_part_uuid)
+        else:
             manager_utils.node_set_boot_device(task, 'disk', persistent=True)
 
+        LOG.debug('Rebooting node %s to instance', node.uuid)
         self.reboot_and_finish_deploy(task)
 
-        # NOTE(TheJulia): If we deployed a whole disk image, we
-        # should expect a whole disk image and clean-up the tftp files
-        # on-disk incase the node is disregarding the boot preference.
-        # TODO(rameshg87): Not all in-tree drivers using reboot_to_instance
-        # have a boot interface. So include a check for now. Remove this
-        # check once all in-tree drivers have a boot interface.
-        if task.driver.boot and iwdi:
-            task.driver.boot.clean_up_ramdisk(task)
-
 
 class AgentDeploy(AgentDeployMixin, base.DeployInterface):
     """Interface for deploy-related actions."""
@@ -378,6 +370,19 @@ class AgentDeploy(AgentDeployMixin, base.DeployInterface):
             task.driver.boot.validate(task)
 
         node = task.node
+
+        # Validate node capabilities
+        deploy_utils.validate_capabilities(node)
+
+        if not task.driver.storage.should_write_image(task):
+            # NOTE(TheJulia): There is no reason to validate
+            # image properties if we will not be writing an image
+            # in a boot from volume case. As such, return to the caller.
+            LOG.debug('Skipping complete deployment interface validation '
+                      'for node %s as it is set to boot from a remote '
+                      'volume.', node.uuid)
+            return
+
         params = {}
         image_source = node.instance_info.get('image_source')
         params['instance_info.image_source'] = image_source
@@ -394,14 +399,19 @@ class AgentDeploy(AgentDeployMixin, base.DeployInterface):
 
         check_image_size(task, image_source)
         # Validate the root device hints
-        deploy_utils.parse_root_device_hints(node)
-
-        # Validate node capabilities
-        deploy_utils.validate_capabilities(node)
+        try:
+            root_device = node.properties.get('root_device')
+            il_utils.parse_root_device_hints(root_device)
+        except ValueError as e:
+            raise exception.InvalidParameterValue(
+                _('Failed to validate the root device hints for node '
+                  '%(node)s. Error: %(error)s') % {'node': node.uuid,
+                                                   'error': e})
 
         validate_image_proxies(node)
 
     @METRICS.timer('AgentDeploy.deploy')
+    @base.deploy_step(priority=100)
     @task_manager.require_exclusive_lock
     def deploy(self, task):
         """Perform a deployment to a node.
@@ -414,8 +424,21 @@ class AgentDeploy(AgentDeployMixin, base.DeployInterface):
         :param task: a TaskManager instance.
         :returns: status of the deploy. One of ironic.common.states.
         """
-        manager_utils.node_power_action(task, states.REBOOT)
-        return states.DEPLOYWAIT
+        if task.driver.storage.should_write_image(task):
+            manager_utils.node_power_action(task, states.REBOOT)
+            return states.DEPLOYWAIT
+        else:
+            # TODO(TheJulia): At some point, we should de-dupe this code
+            # as it is nearly identical to the iscsi deploy interface.
+            # This is not being done now as it is expected to be
+            # refactored in the near future.
+            manager_utils.node_power_action(task, states.POWER_OFF)
+            task.driver.network.remove_provisioning_network(task)
+            task.driver.network.configure_tenant_networks(task)
+            task.driver.boot.prepare_instance(task)
+            manager_utils.node_power_action(task, states.POWER_ON)
+            LOG.info('Deployment to node %s done', task.node.uuid)
+            return None
 
     @METRICS.timer('AgentDeploy.tear_down')
     @task_manager.require_exclusive_lock
@@ -427,12 +450,18 @@ class AgentDeploy(AgentDeployMixin, base.DeployInterface):
         :raises: NetworkError if the cleaning ports cannot be removed.
         :raises: InvalidParameterValue when the wrong power state is specified
              or the wrong driver info is specified for power management.
+        :raises: StorageError when the storage interface attached volumes fail
+             to detach.
         :raises: other exceptions by the node's power driver if something
              wrong occurred during the power action.
         """
         manager_utils.node_power_action(task, states.POWER_OFF)
-
+        task.driver.storage.detach_volumes(task)
+        deploy_utils.tear_down_storage_configuration(task)
         task.driver.network.unconfigure_tenant_networks(task)
+        # NOTE(mgoddard): If the deployment was unsuccessful the node may have
+        # ports on the provisioning network which were not deleted.
+        task.driver.network.remove_provisioning_network(task)
 
         return states.DELETED
 
@@ -445,25 +474,67 @@ class AgentDeploy(AgentDeployMixin, base.DeployInterface):
         :raises: NetworkError: if the previous cleaning ports cannot be removed
             or if new cleaning ports cannot be created.
         :raises: InvalidParameterValue when the wrong power state is specified
-             or the wrong driver info is specified for power management.
+            or the wrong driver info is specified for power management.
+        :raises: StorageError If the storage driver is unable to attach the
+            configured volumes.
         :raises: other exceptions by the node's power driver if something
-             wrong occurred during the power action.
+            wrong occurred during the power action.
         :raises: exception.ImageRefValidationFailed if image_source is not
             Glance href and is not HTTP(S) URL.
+        :raises: exception.InvalidParameterValue if network validation fails.
         :raises: any boot interface's prepare_ramdisk exceptions.
         """
-        # Nodes deployed by AgentDeploy always boot from disk now. So there
-        # is nothing to be done in prepare() when it's called during
-        # take over.
         node = task.node
+        deploy_utils.populate_storage_driver_internal_info(task)
         if node.provision_state == states.DEPLOYING:
+            # Validate network interface to ensure that it supports boot
+            # options configured on the node.
+            try:
+                task.driver.network.validate(task)
+            except exception.InvalidParameterValue:
+                # For 'neutron' network interface validation will fail
+                # if node is using 'netboot' boot option while provisioning
+                # a whole disk image. Updating 'boot_option' in node's
+                # 'instance_info' to 'local for backward compatibility.
+                # TODO(stendulker): Fail here once the default boot
+                # option is local.
+                with excutils.save_and_reraise_exception(reraise=False) as ctx:
+                    instance_info = node.instance_info
+                    capabilities = instance_info.get('capabilities', {})
+                    if 'boot_option' not in capabilities:
+                        capabilities['boot_option'] = 'local'
+                        instance_info['capabilities'] = capabilities
+                        node.instance_info = instance_info
+                        node.save()
+                        # Re-validate the network interface
+                        task.driver.network.validate(task)
+                    else:
+                        ctx.reraise = True
+
             # Adding the node to provisioning network so that the dhcp
             # options get added for the provisioning port.
             manager_utils.node_power_action(task, states.POWER_OFF)
-            task.driver.network.add_provisioning_network(task)
-        if node.provision_state not in [states.ACTIVE, states.ADOPTING]:
-            node.instance_info = build_instance_info_for_deploy(task)
-            node.save()
+            if task.driver.storage.should_write_image(task):
+                # NOTE(vdrok): in case of rebuild, we have tenant network
+                # already configured, unbind tenant ports if present
+                task.driver.network.unconfigure_tenant_networks(task)
+                task.driver.network.add_provisioning_network(task)
+            # Signal to storage driver to attach volumes
+            task.driver.storage.attach_volumes(task)
+            if not task.driver.storage.should_write_image(task):
+                # We have nothing else to do as this is handled in the
+                # backend storage system, and we can return to the caller
+                # as we do not need to boot the agent to deploy.
+                return
+        if node.provision_state in (states.ACTIVE, states.UNRESCUING):
+            # Call is due to conductor takeover
+            task.driver.boot.prepare_instance(task)
+        elif node.provision_state != states.ADOPTING:
+            if node.provision_state not in (states.RESCUING, states.RESCUEWAIT,
+                                            states.RESCUE, states.RESCUEFAIL):
+                node.instance_info = (
+                    deploy_utils.build_instance_info_for_deploy(task))
+                node.save()
             if CONF.agent.manage_agent_boot:
                 deploy_opts = deploy_utils.build_agent_options(node)
                 task.driver.boot.prepare_ramdisk(task, deploy_opts)
@@ -488,6 +559,7 @@ class AgentDeploy(AgentDeployMixin, base.DeployInterface):
         """
         if CONF.agent.manage_agent_boot:
             task.driver.boot.clean_up_ramdisk(task)
+        task.driver.boot.clean_up_instance(task)
         provider = dhcp_factory.DHCPFactory()
         provider.clean_dhcp(task)
 
@@ -555,17 +627,6 @@ class AgentDeploy(AgentDeployMixin, base.DeployInterface):
             task, manage_boot=CONF.agent.manage_agent_boot)
 
 
-class AgentVendorInterface(agent_base_vendor.BaseAgentVendor,
-                           AgentDeployMixin):
-    """Implementation of agent vendor interface.
-
-    Contains old lookup and heartbeat endpoints currently pending deprecation.
-
-    WARNING: This class is deprecated and will be removed in the Ocata release.
-    Drivers should stop relying on it.
-    """
-
-
 class AgentRAID(base.RAIDInterface):
     """Implementation of RAIDInterface which uses agent ramdisk."""
 
@@ -700,3 +761,107 @@ class AgentRAID(base.RAIDInterface):
         """
         task.node.raid_config = {}
         task.node.save()
+
+
+class AgentRescue(base.RescueInterface):
+    """Implementation of RescueInterface which uses agent ramdisk."""
+
+    def get_properties(self):
+        """Return the properties of the interface. """
+        return {}
+
+    @METRICS.timer('AgentRescue.rescue')
+    @task_manager.require_exclusive_lock
+    def rescue(self, task):
+        """Boot a rescue ramdisk on the node.
+
+        :param task: a TaskManager instance.
+        :raises: NetworkError if the tenant ports cannot be removed.
+        :raises: InvalidParameterValue when the wrong power state is specified
+             or the wrong driver info is specified for power management.
+        :raises: other exceptions by the node's power driver if something
+             wrong occurred during the power action.
+        :raises: any boot interface's prepare_ramdisk exceptions.
+        :returns: Returns states.RESCUEWAIT
+        """
+        manager_utils.node_power_action(task, states.POWER_OFF)
+        task.driver.boot.clean_up_instance(task)
+        task.driver.network.unconfigure_tenant_networks(task)
+        task.driver.network.add_rescuing_network(task)
+        if CONF.agent.manage_agent_boot:
+            ramdisk_opts = deploy_utils.build_agent_options(task.node)
+            # prepare_ramdisk will set the boot device
+            task.driver.boot.prepare_ramdisk(task, ramdisk_opts)
+        manager_utils.node_power_action(task, states.POWER_ON)
+
+        return states.RESCUEWAIT
+
+    @METRICS.timer('AgentRescue.unrescue')
+    @task_manager.require_exclusive_lock
+    def unrescue(self, task):
+        """Attempt to move a rescued node back to active state.
+
+        :param task: a TaskManager instance.
+        :raises: NetworkError if the rescue ports cannot be removed.
+        :raises: InvalidParameterValue when the wrong power state is specified
+             or the wrong driver info is specified for power management.
+        :raises: other exceptions by the node's power driver if something
+             wrong occurred during the power action.
+        :raises: any boot interface's prepare_instance exceptions.
+        :returns: Returns states.ACTIVE
+        """
+        manager_utils.node_power_action(task, states.POWER_OFF)
+        self.clean_up(task)
+        task.driver.network.configure_tenant_networks(task)
+        task.driver.boot.prepare_instance(task)
+        manager_utils.node_power_action(task, states.POWER_ON)
+
+        return states.ACTIVE
+
+    @METRICS.timer('AgentRescue.validate')
+    def validate(self, task):
+        """Validate that the node has required properties for agent rescue.
+
+        :param task: a TaskManager instance with the node being checked
+        :raises: InvalidParameterValue if 'instance_info/rescue_password' has
+            empty password or rescuing network UUID config option
+            has an invalid value.
+        :raises: MissingParameterValue if node is missing one or more required
+            parameters
+        """
+        # Validate rescuing network
+        task.driver.network.validate_rescue(task)
+        if CONF.agent.manage_agent_boot:
+            # Validate boot properties
+            task.driver.boot.validate(task)
+            # Validate boot properties related to rescue
+            task.driver.boot.validate_rescue(task)
+
+        node = task.node
+        rescue_pass = node.instance_info.get('rescue_password')
+        if rescue_pass is None:
+            msg = _("Node %(node)s is missing "
+                    "'instance_info/rescue_password'. "
+                    "It is required for rescuing node.")
+            raise exception.MissingParameterValue(msg % {'node': node.uuid})
+
+        if not rescue_pass.strip():
+            msg = (_("The 'instance_info/rescue_password' is an empty string "
+                     "for node %s. The 'rescue_password' must be a non-empty "
+                     "string value.") % node.uuid)
+            raise exception.InvalidParameterValue(msg)
+
+    @METRICS.timer('AgentRescue.clean_up')
+    def clean_up(self, task):
+        """Clean up after RESCUEWAIT timeout/failure or finishing rescue.
+
+        Rescue password should be removed from the node and ramdisk boot
+        environment should be cleaned if Ironic is managing the ramdisk boot.
+
+        :param task: a TaskManager instance with the node.
+        :raises: NetworkError if the rescue ports cannot be removed.
+        """
+        manager_utils.remove_node_rescue_password(task.node, save=True)
+        if CONF.agent.manage_agent_boot:
+            task.driver.boot.clean_up_ramdisk(task)
+        task.driver.network.remove_rescuing_network(task)
