@@ -17,31 +17,22 @@
 #    under the License.
 
 import collections
-import time
 
 from ironic_lib import metrics_utils
 from oslo_log import log
-from oslo_utils import excutils
 from oslo_utils import strutils
 from oslo_utils import timeutils
 import retrying
 
-from ironic.api.controllers.v1 import ramdisk
 from ironic.common import boot_devices
 from ironic.common import exception
-from ironic.common.i18n import _, _LE, _LI, _LW
-from ironic.common import policy
+from ironic.common.i18n import _
 from ironic.common import states
-from ironic.common import utils
-from ironic.conductor import rpcapi
-from ironic.conductor import task_manager
 from ironic.conductor import utils as manager_utils
 from ironic.conf import CONF
-from ironic.drivers import base
 from ironic.drivers.modules import agent_client
 from ironic.drivers.modules import deploy_utils
 from ironic.drivers import utils as driver_utils
-from ironic import objects
 
 LOG = log.getLogger(__name__)
 
@@ -137,6 +128,10 @@ def _cleaning_reboot(task):
     :param task: a TaskManager instance
     """
     try:
+        # NOTE(fellypefca): Call prepare_ramdisk on ensure that the
+        # baremetal node boots back into the ramdisk after reboot.
+        deploy_opts = deploy_utils.build_agent_options(task.node)
+        task.driver.boot.prepare_ramdisk(task, deploy_opts)
         manager_utils.node_power_action(task, states.REBOOT)
     except Exception as e:
         msg = (_('Reboot requested by clean step %(step)s failed for '
@@ -144,7 +139,7 @@ def _cleaning_reboot(task):
                {'step': task.node.clean_step,
                 'node': task.node.uuid,
                 'err': e})
-        LOG.error(msg)
+        LOG.error(msg, exc_info=not isinstance(e, exception.IronicException))
         # do not set cleaning_reboot if we didn't reboot
         manager_utils.cleaning_error_handler(task, msg)
         return
@@ -156,17 +151,6 @@ def _cleaning_reboot(task):
     task.node.save()
 
 
-def _notify_conductor_resume_clean(task):
-    LOG.debug('Sending RPC to conductor to resume cleaning for node %s',
-              task.node.uuid)
-    uuid = task.node.uuid
-    rpc = rpcapi.ConductorAPI()
-    topic = rpc.get_topic_for(task.node)
-    # Need to release the lock to let the conductor take it
-    task.release_resources()
-    rpc.continue_node_clean(task.context, uuid, topic=topic)
-
-
 def _get_completed_cleaning_command(task, commands):
     """Returns None or a completed cleaning command from the agent.
 
@@ -195,8 +179,8 @@ def _get_completed_cleaning_command(task, commands):
         LOG.debug('Clean step still running for node %(node)s: %(step)s',
                   {'step': last_step, 'node': task.node.uuid})
         return
-    elif (last_command['command_status'] == 'SUCCEEDED' and
-          last_step != task.node.clean_step):
+    elif (last_command['command_status'] == 'SUCCEEDED'
+          and last_step != task.node.clean_step):
         # A previous clean_step was running, the new command has not yet
         # started.
         LOG.debug('Clean step not yet started for node %(node)s: %(step)s',
@@ -207,15 +191,26 @@ def _get_completed_cleaning_command(task, commands):
 
 
 @METRICS.timer('log_and_raise_deployment_error')
-def log_and_raise_deployment_error(task, msg):
-    """Helper method to log the error and raise exception."""
-    LOG.error(msg)
-    deploy_utils.set_failed_state(task, msg)
+def log_and_raise_deployment_error(task, msg, collect_logs=True, exc=None):
+    """Helper method to log the error and raise exception.
+
+    :param task: a TaskManager instance containing the node to act on.
+    :param msg: the message to set in last_error of the node.
+    :param collect_logs: Boolean indicating whether to attempt to collect
+                         logs from IPA-based ramdisk. Defaults to True.
+                         Actual log collection is also affected by
+                         CONF.agent.deploy_logs_collect config option.
+    :param exc: Exception that caused the failure.
+    """
+    log_traceback = (exc is not None and
+                     not isinstance(exc, exception.IronicException))
+    LOG.error(msg, exc_info=log_traceback)
+    deploy_utils.set_failed_state(task, msg, collect_logs=collect_logs)
     raise exception.InstanceDeployFailure(msg)
 
 
-class AgentDeployMixin(object):
-    """Mixin with deploy methods."""
+class HeartbeatMixin(object):
+    """Mixin class implementing heartbeat processing."""
 
     def __init__(self):
         self._client = _get_client()
@@ -248,8 +243,145 @@ class AgentDeployMixin(object):
 
         """
 
-    @METRICS.timer('AgentDeployMixin._refresh_clean_steps')
-    def _refresh_clean_steps(self, task):
+    def refresh_clean_steps(self, task):
+        """Refresh the node's cached clean steps
+
+        :param task: a TaskManager instance
+
+        """
+
+    def continue_cleaning(self, task):
+        """Start the next cleaning step if the previous one is complete.
+
+        :param task: a TaskManager instance
+
+        """
+
+    @property
+    def heartbeat_allowed_states(self):
+        """Define node states where heartbeating is allowed"""
+        return (states.DEPLOYWAIT, states.CLEANWAIT, states.RESCUEWAIT)
+
+    @METRICS.timer('HeartbeatMixin.heartbeat')
+    def heartbeat(self, task, callback_url, agent_version):
+        """Process a heartbeat.
+
+        :param task: task to work with.
+        :param callback_url: agent HTTP API URL.
+        :param agent_version: The version of the agent that is heartbeating
+        """
+        # NOTE(pas-ha) immediately skip the rest if nothing to do
+        if task.node.provision_state not in self.heartbeat_allowed_states:
+            LOG.debug('Heartbeat from node %(node)s in unsupported '
+                      'provision state %(state)s, not taking any action.',
+                      {'node': task.node.uuid,
+                       'state': task.node.provision_state})
+            return
+
+        try:
+            task.upgrade_lock()
+        except exception.NodeLocked:
+            LOG.warning('Node %s is currently locked, skipping heartbeat '
+                        'processing (will retry on the next heartbeat)',
+                        task.node.uuid)
+            return
+
+        node = task.node
+        LOG.debug('Heartbeat from node %s', node.uuid)
+
+        driver_internal_info = node.driver_internal_info
+        driver_internal_info['agent_url'] = callback_url
+        driver_internal_info['agent_version'] = agent_version
+        node.driver_internal_info = driver_internal_info
+        node.save()
+
+        # Async call backs don't set error state on their own
+        # TODO(jimrollenhagen) improve error messages here
+        msg = _('Failed checking if deploy is done.')
+        try:
+            if node.maintenance:
+                # this shouldn't happen often, but skip the rest if it does.
+                LOG.debug('Heartbeat from node %(node)s in maintenance mode; '
+                          'not taking any action.', {'node': node.uuid})
+                return
+            elif (node.provision_state == states.DEPLOYWAIT
+                  and not self.deploy_has_started(task)):
+                msg = _('Node failed to deploy.')
+                self.continue_deploy(task)
+            elif (node.provision_state == states.DEPLOYWAIT
+                  and self.deploy_is_done(task)):
+                msg = _('Node failed to move to active state.')
+                self.reboot_to_instance(task)
+            elif (node.provision_state == states.DEPLOYWAIT
+                  and self.deploy_has_started(task)):
+                node.touch_provisioning()
+            elif node.provision_state == states.CLEANWAIT:
+                node.touch_provisioning()
+                if not node.clean_step:
+                    LOG.debug('Node %s just booted to start cleaning.',
+                              node.uuid)
+                    msg = _('Node failed to start the first cleaning step.')
+                    # First, cache the clean steps
+                    self.refresh_clean_steps(task)
+                    # Then set/verify node clean steps and start cleaning
+                    manager_utils.set_node_cleaning_steps(task)
+                    # The exceptions from RPC are not possible as we using cast
+                    # here
+                    manager_utils.notify_conductor_resume_clean(task)
+                else:
+                    msg = _('Node failed to check cleaning progress.')
+                    self.continue_cleaning(task)
+            elif (node.provision_state == states.RESCUEWAIT):
+                msg = _('Node failed to perform rescue operation.')
+                self._finalize_rescue(task)
+        except Exception as e:
+            err_info = {'msg': msg, 'e': e}
+            last_error = _('Asynchronous exception: %(msg)s '
+                           'Exception: %(e)s for node') % err_info
+            errmsg = last_error + ' %(node)s'
+            LOG.exception(errmsg, {'node': node.uuid})
+            if node.provision_state in (states.CLEANING, states.CLEANWAIT):
+                manager_utils.cleaning_error_handler(task, last_error)
+            elif node.provision_state in (states.DEPLOYING, states.DEPLOYWAIT):
+                deploy_utils.set_failed_state(
+                    task, last_error, collect_logs=bool(self._client))
+            elif node.provision_state in (states.RESCUING, states.RESCUEWAIT):
+                manager_utils.rescuing_error_handler(task, last_error)
+
+    def _finalize_rescue(self, task):
+        """Call ramdisk to prepare rescue mode and verify result.
+
+        :param task: A TaskManager instance
+        :raises: InstanceRescueFailure, if rescuing failed
+        """
+        node = task.node
+        try:
+            result = self._client.finalize_rescue(node)
+        except exception.IronicException as e:
+            raise exception.InstanceRescueFailure(node=node.uuid,
+                                                  instance=node.instance_uuid,
+                                                  reason=e)
+        if ((not result.get('command_status'))
+                or result.get('command_status') != 'SUCCEEDED'):
+            # NOTE(mariojv) Caller will clean up failed rescue in exception
+            # handler.
+            fail_reason = (_('Agent returned bad result for command '
+                             'finalize_rescue: %(result)s') %
+                           {'result': result.get('command_error')})
+            raise exception.InstanceRescueFailure(node=node.uuid,
+                                                  instance=node.instance_uuid,
+                                                  reason=fail_reason)
+        task.process_event('resume')
+        task.driver.rescue.clean_up(task)
+        task.driver.network.configure_tenant_networks(task)
+        task.process_event('done')
+
+
+class AgentDeployMixin(HeartbeatMixin):
+    """Mixin with deploy methods."""
+
+    @METRICS.timer('AgentDeployMixin.refresh_clean_steps')
+    def refresh_clean_steps(self, task):
         """Refresh the node's cached clean steps from the booted agent.
 
         Gets the node's clean steps from the booted agent and caches them.
@@ -334,7 +466,8 @@ class AgentDeployMixin(object):
                 info = task.node.driver_internal_info
                 info.pop('cleaning_reboot', None)
                 task.node.driver_internal_info = info
-                _notify_conductor_resume_clean(task)
+                task.node.save()
+                manager_utils.notify_conductor_resume_clean(task)
                 return
             else:
                 # Agent has no commands whatsoever
@@ -361,7 +494,7 @@ class AgentDeployMixin(object):
         elif command.get('command_status') == 'CLEAN_VERSION_MISMATCH':
             # Cache the new clean steps (and 'hardware_manager_version')
             try:
-                self._refresh_clean_steps(task)
+                self.refresh_clean_steps(task)
             except exception.NodeCleaningFailure as e:
                 msg = (_('Could not continue cleaning on node '
                          '%(node)s: %(err)s.') %
@@ -374,9 +507,9 @@ class AgentDeployMixin(object):
                 # version. Both are operator actions, unlike automated
                 # cleaning. Manual clean steps are not necessarily idempotent
                 # like automated clean steps and can be even longer running.
-                LOG.info(_LI('During manual cleaning, node %(node)s detected '
-                             'a clean version mismatch. Re-executing and '
-                             'continuing from current step %(step)s.'),
+                LOG.info('During manual cleaning, node %(node)s detected '
+                         'a clean version mismatch. Re-executing and '
+                         'continuing from current step %(step)s.',
                          {'node': node.uuid, 'step': node.clean_step})
 
                 driver_internal_info = node.driver_internal_info
@@ -385,10 +518,9 @@ class AgentDeployMixin(object):
                 node.save()
             else:
                 # Restart cleaning, agent must have rebooted to new version
-                LOG.info(_LI('During automated cleaning, node %s detected a '
-                             'clean version mismatch. Resetting clean steps '
-                             'and rebooting the node.'),
-                         node.uuid)
+                LOG.info('During automated cleaning, node %s detected a '
+                         'clean version mismatch. Resetting clean steps '
+                         'and rebooting the node.', node.uuid)
                 try:
                     manager_utils.set_node_cleaning_steps(task)
                 except exception.NodeCleaningFailure:
@@ -400,13 +532,13 @@ class AgentDeployMixin(object):
                     LOG.exception(msg)
                     return manager_utils.cleaning_error_handler(task, msg)
 
-            _notify_conductor_resume_clean(task)
+            manager_utils.notify_conductor_resume_clean(task)
 
         elif command.get('command_status') == 'SUCCEEDED':
             clean_step_hook = _get_post_clean_step_hook(node)
             if clean_step_hook is not None:
                 LOG.debug('For node %(node)s, executing post clean step '
-                          'hook %(method)s for clean step %(step)s' %
+                          'hook %(method)s for clean step %(step)s',
                           {'method': clean_step_hook.__name__,
                            'node': node.uuid,
                            'step': node.clean_step})
@@ -415,10 +547,11 @@ class AgentDeployMixin(object):
                 except Exception as e:
                     msg = (_('For node %(node)s, post clean step hook '
                              '%(method)s failed for clean step %(step)s.'
-                             'Error: %(error)s') %
+                             '%(cls)s: %(error)s') %
                            {'method': clean_step_hook.__name__,
                             'node': node.uuid,
                             'error': e,
+                            'cls': e.__class__.__name__,
                             'step': node.clean_step})
                     LOG.exception(msg)
                     return manager_utils.cleaning_error_handler(task, msg)
@@ -427,9 +560,9 @@ class AgentDeployMixin(object):
                 _cleaning_reboot(task)
                 return
 
-            LOG.info(_LI('Agent on node %s returned cleaning command success, '
-                         'moving to next clean step'), node.uuid)
-            _notify_conductor_resume_clean(task)
+            LOG.info('Agent on node %s returned cleaning command success, '
+                     'moving to next clean step', node.uuid)
+            manager_utils.notify_conductor_resume_clean(task)
         else:
             msg = (_('Agent returned unknown status for clean step %(step)s '
                      'on node %(node)s : %(err)s.') %
@@ -439,85 +572,6 @@ class AgentDeployMixin(object):
             LOG.error(msg)
             return manager_utils.cleaning_error_handler(task, msg)
 
-    @METRICS.timer('AgentDeployMixin.heartbeat')
-    def heartbeat(self, task, callback_url):
-        """Process a heartbeat.
-
-        :param task: task to work with.
-        :param callback_url: agent HTTP API URL.
-        """
-        # TODO(dtantsur): upgrade lock only if we actually take action other
-        # than updating the last timestamp.
-        task.upgrade_lock()
-
-        node = task.node
-        driver_internal_info = node.driver_internal_info
-        LOG.debug(
-            'Heartbeat from %(node)s, last heartbeat at %(heartbeat)s.',
-            {'node': node.uuid,
-             'heartbeat': driver_internal_info.get('agent_last_heartbeat')})
-        driver_internal_info['agent_last_heartbeat'] = int(time.time())
-        try:
-            driver_internal_info['agent_url'] = callback_url
-        except KeyError:
-            raise exception.MissingParameterValue(_('For heartbeat operation, '
-                                                    '"agent_url" must be '
-                                                    'specified.'))
-
-        node.driver_internal_info = driver_internal_info
-        node.save()
-
-        # Async call backs don't set error state on their own
-        # TODO(jimrollenhagen) improve error messages here
-        msg = _('Failed checking if deploy is done.')
-        try:
-            if node.maintenance:
-                # this shouldn't happen often, but skip the rest if it does.
-                LOG.debug('Heartbeat from node %(node)s in maintenance mode; '
-                          'not taking any action.', {'node': node.uuid})
-                return
-            elif (node.provision_state == states.DEPLOYWAIT and
-                  not self.deploy_has_started(task)):
-                msg = _('Node failed to get image for deploy.')
-                self.continue_deploy(task)
-            elif (node.provision_state == states.DEPLOYWAIT and
-                  self.deploy_is_done(task)):
-                msg = _('Node failed to move to active state.')
-                self.reboot_to_instance(task)
-            elif (node.provision_state == states.DEPLOYWAIT and
-                  self.deploy_has_started(task)):
-                node.touch_provisioning()
-            elif node.provision_state == states.CLEANWAIT:
-                node.touch_provisioning()
-                try:
-                    if not node.clean_step:
-                        LOG.debug('Node %s just booted to start cleaning.',
-                                  node.uuid)
-                        msg = _('Node failed to start the first cleaning '
-                                'step.')
-                        # First, cache the clean steps
-                        self._refresh_clean_steps(task)
-                        # Then set/verify node clean steps and start cleaning
-                        manager_utils.set_node_cleaning_steps(task)
-                        _notify_conductor_resume_clean(task)
-                    else:
-                        msg = _('Node failed to check cleaning progress.')
-                        self.continue_cleaning(task)
-                except exception.NoFreeConductorWorker:
-                    # waiting for the next heartbeat, node.last_error and
-                    # logging message is filled already via conductor's hook
-                    pass
-
-        except Exception as e:
-            err_info = {'node': node.uuid, 'msg': msg, 'e': e}
-            last_error = _('Asynchronous exception for node %(node)s: '
-                           '%(msg)s Exception: %(e)s') % err_info
-            LOG.exception(last_error)
-            if node.provision_state in (states.CLEANING, states.CLEANWAIT):
-                manager_utils.cleaning_error_handler(task, last_error)
-            elif node.provision_state in (states.DEPLOYING, states.DEPLOYWAIT):
-                deploy_utils.set_failed_state(task, last_error)
-
     @METRICS.timer('AgentDeployMixin.reboot_and_finish_deploy')
     def reboot_and_finish_deploy(self, task):
         """Helper method to trigger reboot on the node and finish deploy.
@@ -556,13 +610,14 @@ class AgentDeployMixin(object):
                     self._client.power_off(node)
                     _wait_until_powered_off(task)
                 except Exception as e:
-                    LOG.warning(
-                        _LW('Failed to soft power off node %(node_uuid)s '
-                            'in at least %(timeout)d seconds. '
-                            'Error: %(error)s'),
-                        {'node_uuid': node.uuid,
-                         'timeout': (wait * (attempts - 1)) / 1000,
-                         'error': e})
+                    LOG.warning('Failed to soft power off node %(node_uuid)s '
+                                'in at least %(timeout)d seconds. '
+                                '%(cls)s: %(error)s',
+                                {'node_uuid': node.uuid,
+                                 'timeout': (wait * (attempts - 1)) / 1000,
+                                 'cls': e.__class__.__name__, 'error': e},
+                                exc_info=not isinstance(
+                                    e, exception.IronicException))
                     manager_utils.node_power_action(task, states.POWER_OFF)
             else:
                 # Flush the file system prior to hard rebooting the node
@@ -573,28 +628,46 @@ class AgentDeployMixin(object):
                         error = _('The version of the IPA ramdisk used in '
                                   'the deployment do not support the '
                                   'command "sync"')
-                    LOG.warning(_LW(
+                    LOG.warning(
                         'Failed to flush the file system prior to hard '
-                        'rebooting the node %(node)s. Error: %(error)s'),
+                        'rebooting the node %(node)s. Error: %(error)s',
                         {'node': node.uuid, 'error': error})
 
                 manager_utils.node_power_action(task, states.POWER_OFF)
+        except Exception as e:
+            msg = (_('Error rebooting node %(node)s after deploy. '
+                     '%(cls)s: %(error)s') %
+                   {'node': node.uuid, 'cls': e.__class__.__name__,
+                    'error': e})
+            log_and_raise_deployment_error(task, msg, exc=e)
 
+        try:
             task.driver.network.remove_provisioning_network(task)
             task.driver.network.configure_tenant_networks(task)
 
             manager_utils.node_power_action(task, states.POWER_ON)
         except Exception as e:
             msg = (_('Error rebooting node %(node)s after deploy. '
-                     'Error: %(error)s') %
-                   {'node': node.uuid, 'error': e})
-            log_and_raise_deployment_error(task, msg)
+                     '%(cls)s: %(error)s') %
+                   {'node': node.uuid, 'cls': e.__class__.__name__,
+                    'error': e})
+            # NOTE(mgoddard): Don't collect logs since the node has been
+            # powered off.
+            log_and_raise_deployment_error(task, msg, collect_logs=False,
+                                           exc=e)
 
-        task.process_event('done')
-        LOG.info(_LI('Deployment to node %s done'), task.node.uuid)
+        if not node.deploy_step:
+            # TODO(rloo): delete this 'if' part after deprecation period, when
+            # we expect all (out-of-tree) drivers to support deploy steps.
+            # After which we will always notify_conductor_resume_deploy().
+            task.process_event('done')
+            LOG.info('Deployment to node %s done', task.node.uuid)
+        else:
+            manager_utils.notify_conductor_resume_deploy(task)
 
     @METRICS.timer('AgentDeployMixin.prepare_instance_to_boot')
-    def prepare_instance_to_boot(self, task, root_uuid, efi_sys_uuid):
+    def prepare_instance_to_boot(self, task, root_uuid, efi_sys_uuid,
+                                 prep_boot_part_uuid=None):
         """Prepares instance to boot.
 
         :param task: a TaskManager object containing the node
@@ -608,19 +681,22 @@ class AgentDeployMixin(object):
             # Install the boot loader
             self.configure_local_boot(
                 task, root_uuid=root_uuid,
-                efi_system_part_uuid=efi_sys_uuid)
+                efi_system_part_uuid=efi_sys_uuid,
+                prep_boot_part_uuid=prep_boot_part_uuid)
         try:
             task.driver.boot.prepare_instance(task)
         except Exception as e:
-            LOG.error(_LE('Deploy failed for instance %(instance)s. '
-                          'Error: %(error)s'),
-                      {'instance': node.instance_uuid, 'error': e})
-            msg = _('Failed to continue agent deployment.')
-            log_and_raise_deployment_error(task, msg)
+            LOG.error('Preparing instance for booting failed for instance '
+                      '%(instance)s. %(cls)s: %(error)s',
+                      {'instance': node.instance_uuid,
+                       'cls': e.__class__.__name__, 'error': e})
+            msg = _('Failed to prepare instance for booting')
+            log_and_raise_deployment_error(task, msg, exc=e)
 
     @METRICS.timer('AgentDeployMixin.configure_local_boot')
     def configure_local_boot(self, task, root_uuid=None,
-                             efi_system_part_uuid=None):
+                             efi_system_part_uuid=None,
+                             prep_boot_part_uuid=None):
         """Helper method to configure local boot on the node.
 
         This method triggers bootloader installation on the node.
@@ -634,6 +710,8 @@ class AgentDeployMixin(object):
             have a bootloader installed.
         :param efi_system_part_uuid: The UUID of the efi system partition.
             This is used only in uefi boot mode.
+        :param prep_boot_part_uuid: The UUID of the PReP Boot partition.
+            This is used only for booting ppc64* hardware.
         :raises: InstanceDeployFailure if bootloader installation failed or
             on encountering error while setting the boot device on the node.
         """
@@ -647,7 +725,8 @@ class AgentDeployMixin(object):
                        'efi': efi_system_part_uuid})
             result = self._client.install_bootloader(
                 node, root_uuid=root_uuid,
-                efi_system_part_uuid=efi_system_part_uuid)
+                efi_system_part_uuid=efi_system_part_uuid,
+                prep_boot_part_uuid=prep_boot_part_uuid)
             if result['command_status'] == 'FAILED':
                 msg = (_("Failed to install a bootloader when "
                          "deploying node %(node)s. Error: %(error)s") %
@@ -662,250 +741,6 @@ class AgentDeployMixin(object):
                      "when deploying node %(node)s. Error: %(error)s") %
                    {'boot_dev': boot_devices.DISK, 'node': node.uuid,
                     'error': e})
-            log_and_raise_deployment_error(task, msg)
+            log_and_raise_deployment_error(task, msg, exc=e)
 
-        LOG.info(_LI('Local boot successfully configured for node %s'),
-                 node.uuid)
-
-
-# TODO(dtantsur): deprecate and remove it as soon as we stop using it ourselves
-# in the Ocata release.
-class BaseAgentVendor(AgentDeployMixin, base.VendorInterface):
-
-    def __init__(self):
-        self.supported_payload_versions = ['2']
-        super(BaseAgentVendor, self).__init__()
-
-    def get_properties(self):
-        """Return the properties of the interface.
-
-        :returns: dictionary of <property name>:<property description> entries.
-        """
-        return VENDOR_PROPERTIES
-
-    def validate(self, task, method, **kwargs):
-        """Validate the driver-specific Node deployment info.
-
-        No validation necessary.
-
-        :param task: a TaskManager instance
-        :param method: method to be validated
-        """
-        pass
-
-    def driver_validate(self, method, **kwargs):
-        """Validate the driver deployment info.
-
-        :param method: method to be validated.
-        """
-        version = kwargs.get('version')
-
-        if not version:
-            raise exception.MissingParameterValue(_('Missing parameter '
-                                                    'version'))
-        if version not in self.supported_payload_versions:
-            raise exception.InvalidParameterValue(_('Unknown lookup '
-                                                    'payload version: %s')
-                                                  % version)
-
-    @METRICS.timer('BaseAgentVendor.heartbeat')
-    @base.passthru(['POST'])
-    @task_manager.require_exclusive_lock
-    def heartbeat(self, task, **kwargs):
-        """Method for agent to periodically check in.
-
-        The agent should be sending its agent_url (so Ironic can talk back)
-        as a kwarg. kwargs should have the following format::
-
-         {
-             'agent_url': 'http://AGENT_HOST:AGENT_PORT'
-         }
-
-        AGENT_PORT defaults to 9999.
-        """
-        try:
-            callback_url = kwargs['agent_url']
-        except KeyError:
-            raise exception.MissingParameterValue(_('For heartbeat operation, '
-                                                    '"agent_url" must be '
-                                                    'specified.'))
-
-        super(BaseAgentVendor, self).heartbeat(task, callback_url)
-
-    @METRICS.timer('BaseAgentVendor.lookup')
-    @base.driver_passthru(['POST'], async=False)
-    def lookup(self, context, **kwargs):
-        """Find a matching node for the agent.
-
-        Method to be called the first time a ramdisk agent checks in. This
-        can be because this is a node just entering decom or a node that
-        rebooted for some reason. We will use the mac addresses listed in the
-        kwargs to find the matching node, then return the node object to the
-        agent. The agent can that use that UUID to use the node vendor
-        passthru method.
-
-        Currently, we don't handle the instance where the agent doesn't have
-        a matching node (i.e. a brand new, never been in Ironic node).
-
-        Additionally, we may pass on useful configurations to the agent, which
-        it would then be responsible for applying if relevant. Today these are
-        limited to heartbeat_timeout and metrics configuration.
-
-        kwargs should have the following format::
-
-         {
-             "version": "2"
-             "inventory": {
-                 "interfaces": [
-                     {
-                         "name": "eth0",
-                         "mac_address": "00:11:22:33:44:55",
-                         "switch_port_descr": "port24",
-                         "switch_chassis_descr": "tor1"
-                     }, ...
-                 ], ...
-             },
-             "node_uuid": "ab229209-0139-4588-bbe5-64ccec81dd6e"
-         }
-
-        The interfaces list should include a list of the non-IPMI MAC addresses
-        in the form aa:bb:cc:dd:ee:ff.
-
-        node_uuid argument is optional. If it's provided (e.g. as a result of
-        inspection run before lookup), this method will just return a node and
-        options.
-
-        This method will also return the timeout for heartbeats. The driver
-        will expect the agent to heartbeat before that timeout, or it will be
-        considered down. This will be in a root level key called
-        'heartbeat_timeout'
-
-        :raises: NotFound if no matching node is found.
-        :raises: InvalidParameterValue with unknown payload version
-        """
-        LOG.warning(
-            _LW('Agent lookup vendor passthru is deprecated and will be '
-                'removed in the Ocata release; please update your '
-                'ironic-python-agent image to the Newton version'))
-        LOG.debug('Agent lookup using data %s', kwargs)
-        uuid = kwargs.get('node_uuid')
-        if uuid:
-            node = objects.Node.get_by_uuid(context, uuid)
-        else:
-            inventory = kwargs.get('inventory')
-            interfaces = self._get_interfaces(inventory)
-            mac_addresses = self._get_mac_addresses(interfaces)
-
-            node = self._find_node_by_macs(context, mac_addresses)
-
-        LOG.info(_LI('Initial lookup for node %s succeeded, agent is running '
-                     'and waiting for commands'), node.uuid)
-
-        ndict = node.as_dict()
-        cdict = context.to_dict()
-        show_driver_secrets = policy.check('show_password', cdict, cdict)
-        if not show_driver_secrets:
-            ndict['driver_info'] = strutils.mask_dict_password(
-                ndict['driver_info'], "******")
-
-        return {
-            # heartbeat_timeout is a config, so moving it into the
-            # config namespace. Instead of a separate deprecation,
-            # this will die when the vendor_passthru version of
-            # lookup goes away.
-            'heartbeat_timeout': CONF.api.ramdisk_heartbeat_timeout,
-            'node': ndict,
-            'config': ramdisk.config(),
-        }
-
-    def _get_interfaces(self, inventory):
-        interfaces = []
-        try:
-            interfaces = inventory['interfaces']
-        except (KeyError, TypeError):
-            raise exception.InvalidParameterValue(_(
-                'Malformed network interfaces lookup: %s') % inventory)
-
-        return interfaces
-
-    def _get_mac_addresses(self, interfaces):
-        """Returns MACs for the network devices."""
-        mac_addresses = []
-
-        for interface in interfaces:
-            try:
-                mac_addresses.append(utils.validate_and_normalize_mac(
-                    interface.get('mac_address')))
-            except exception.InvalidMAC:
-                LOG.warning(_LW('Malformed MAC: %s'), interface.get(
-                    'mac_address'))
-        return mac_addresses
-
-    def _find_node_by_macs(self, context, mac_addresses):
-        """Get nodes for a given list of MAC addresses.
-
-        Given a list of MAC addresses, find the ports that match the MACs
-        and return the node they are all connected to.
-
-        :raises: NodeNotFound if the ports point to multiple nodes or no
-        nodes.
-        """
-        ports = self._find_ports_by_macs(context, mac_addresses)
-        if not ports:
-            raise exception.NodeNotFound(_(
-                'No ports matching the given MAC addresses %s exist in the '
-                'database.') % mac_addresses)
-        node_id = self._get_node_id(ports)
-        try:
-            node = objects.Node.get_by_id(context, node_id)
-        except exception.NodeNotFound:
-            with excutils.save_and_reraise_exception():
-                LOG.exception(_LE('Could not find matching node for the '
-                                  'provided MACs %s.'), mac_addresses)
-
-        return node
-
-    def _find_ports_by_macs(self, context, mac_addresses):
-        """Get ports for a given list of MAC addresses.
-
-        Given a list of MAC addresses, find the ports that match the MACs
-        and return them as a list of Port objects, or an empty list if there
-        are no matches
-        """
-        ports = []
-        for mac in mac_addresses:
-            # Will do a search by mac if the mac isn't malformed
-            try:
-                port_ob = objects.Port.get_by_address(context, mac)
-                ports.append(port_ob)
-
-            except exception.PortNotFound:
-                LOG.warning(_LW('MAC address %s not found in database'), mac)
-
-        return ports
-
-    def _get_node_id(self, ports):
-        """Get a node ID for a list of ports.
-
-        Given a list of ports, either return the node_id they all share or
-        raise a NotFound if there are multiple node_ids, which indicates some
-        ports are connected to one node and the remaining port(s) are connected
-        to one or more other nodes.
-
-        :raises: NodeNotFound if the MACs match multiple nodes. This
-        could happen if you swapped a NIC from one server to another and
-        don't notify Ironic about it or there is a MAC collision (since
-        they're not guaranteed to be unique).
-        """
-        # See if all the ports point to the same node
-        node_ids = set(port_ob.node_id for port_ob in ports)
-        if len(node_ids) > 1:
-            raise exception.NodeNotFound(_(
-                'Ports matching mac addresses match multiple nodes. MACs: '
-                '%(macs)s. Port ids: %(port_ids)s') %
-                {'macs': [port_ob.address for port_ob in ports], 'port_ids':
-                 [port_ob.uuid for port_ob in ports]}
-            )
-
-        # Only have one node_id left, return it.
-        return node_ids.pop()
+        LOG.info('Local boot successfully configured for node %s', node.uuid)
