@@ -21,7 +21,6 @@ Ironic console utilities.
 
 import errno
 import os
-import psutil
 import signal
 import subprocess
 import time
@@ -29,11 +28,12 @@ import time
 from ironic_lib import utils as ironic_utils
 from oslo_log import log as logging
 from oslo_service import loopingcall
+from oslo_utils import fileutils
 from oslo_utils import netutils
+import psutil
 
 from ironic.common import exception
-from ironic.common.i18n import _, _LE, _LW
-from ironic.common import utils
+from ironic.common.i18n import _
 from ironic.conf import CONF
 
 
@@ -107,8 +107,8 @@ def _stop_console(node_uuid):
                      "Reason: %(err)s.") % {'node': node_uuid, 'err': exc})
             raise exception.ConsoleError(message=msg)
         else:
-            LOG.warning(_LW("Console process for node %s is not running "
-                            "but pid file exists."), node_uuid)
+            LOG.warning("Console process for node %s is not running "
+                        "but pid file exists.", node_uuid)
     finally:
         ironic_utils.unlink_without_raise(_get_console_pid_file(node_uuid))
 
@@ -117,13 +117,13 @@ def make_persistent_password_file(path, password):
     """Writes a file containing a password until deleted."""
 
     try:
-        utils.delete_if_exists(path)
+        fileutils.delete_if_exists(path)
         with open(path, 'wb') as file:
             os.chmod(path, 0o600)
             file.write(password.encode())
         return path
     except Exception as e:
-        utils.delete_if_exists(path)
+        fileutils.delete_if_exists(path)
         raise exception.PasswordFileFailedToCreate(error=e)
 
 
@@ -199,8 +199,8 @@ def start_shellinabox_console(node_uuid, port, console_cmd):
         # if it is, then the shellinaboxd is invoked successfully as a daemon.
         # otherwise check the error.
         if locals['returncode'] is not None:
-            if (locals['returncode'] == 0 and os.path.exists(pid_file) and
-                psutil.pid_exists(_get_console_pid(node_uuid))):
+            if (locals['returncode'] == 0 and os.path.exists(pid_file)
+                and psutil.pid_exists(_get_console_pid(node_uuid))):
                 raise loopingcall.LoopingCallDone()
             else:
                 (stdout, stderr) = popen_obj.communicate()
@@ -241,8 +241,8 @@ def stop_shellinabox_console(node_uuid):
     try:
         _stop_console(node_uuid)
     except exception.NoConsolePid:
-        LOG.warning(_LW("No console pid found for node %s while trying to "
-                        "stop shellinabox console."), node_uuid)
+        LOG.warning("No console pid found for node %s while trying to "
+                    "stop shellinabox console.", node_uuid)
 
 
 def get_socat_console_url(port):
@@ -251,7 +251,7 @@ def get_socat_console_url(port):
     :param port: the terminal port (integer) for the node
     :return: an access URL to the socat console of the node
     """
-    console_host = CONF.my_ip
+    console_host = CONF.console.socat_address
     if netutils.is_valid_ipv6(console_host):
         console_host = '[%s]' % console_host
 
@@ -282,13 +282,18 @@ def start_socat_console(node_uuid, port, console_cmd):
 
     # put together the command and arguments for invoking the console
     args = ['socat']
+    # set timeout check for user's connection. If the timeout value
+    # is not 0, after timeout seconds of inactivity on the client side,
+    # the connection will be closed.
+    if CONF.console.terminal_timeout > 0:
+        args.append('-T%d' % CONF.console.terminal_timeout)
     args.append('-L%s' % pid_file)
 
-    console_host = CONF.my_ip
+    console_host = CONF.console.socat_address
     if netutils.is_valid_ipv6(console_host):
-        arg = 'TCP6-LISTEN:%(port)s,bind=[%(host)s],reuseaddr,fork'
+        arg = 'TCP6-LISTEN:%(port)s,bind=[%(host)s],reuseaddr'
     else:
-        arg = 'TCP4-LISTEN:%(port)s,bind=%(host)s,reuseaddr,fork'
+        arg = 'TCP4-LISTEN:%(port)s,bind=%(host)s,reuseaddr'
     args.append(arg % {'host': console_host,
                        'port': port})
 
@@ -305,7 +310,7 @@ def start_socat_console(node_uuid, port, console_cmd):
         error = _("%(exec_error)s\n"
                   "Command: %(command)s") % {'exec_error': str(e),
                                              'command': ' '.join(args)}
-        LOG.exception(_LE('Unable to start socat console'))
+        LOG.exception('Unable to start socat console')
         raise exception.ConsoleSubprocessFailed(error=error)
 
     # NOTE: we need to check if socat fails to start here.
@@ -319,8 +324,8 @@ def start_socat_console(node_uuid, port, console_cmd):
         if wait_state['returncode'] is None:
             # If the pid file is created and the process is running,
             # we stop checking it periodically.
-            if (os.path.exists(pid_file) and
-                    psutil.pid_exists(_get_console_pid(node_uuid))):
+            if (os.path.exists(pid_file)
+                    and psutil.pid_exists(_get_console_pid(node_uuid))):
                 raise loopingcall.LoopingCallDone()
         else:
             # socat returned, it failed to start.
@@ -361,5 +366,5 @@ def stop_socat_console(node_uuid):
     try:
         _stop_console(node_uuid)
     except exception.NoConsolePid:
-        LOG.warning(_LW("No console pid found for node %s while trying to "
-                        "stop socat console."), node_uuid)
+        LOG.warning("No console pid found for node %s while trying to "
+                    "stop socat console.", node_uuid)
