@@ -31,14 +31,12 @@ from ironic.common import boot_devices
 from ironic.common import exception
 from ironic.common.glance_service import service_utils
 from ironic.common.i18n import _
-from ironic.common.i18n import _LE
-from ironic.common.i18n import _LI
-from ironic.common.i18n import _LW
 from ironic.common import images
 from ironic.common import swift
 from ironic.common import utils
 from ironic.conductor import utils as manager_utils
 from ironic.conf import CONF
+from ironic.drivers.modules import boot_mode_utils
 from ironic.drivers.modules import deploy_utils
 
 ilo_client = importutils.try_import('proliantutils.ilo.client')
@@ -59,8 +57,28 @@ REQUIRED_PROPERTIES = {
 OPTIONAL_PROPERTIES = {
     'client_port': _("port to be used for iLO operations. Optional."),
     'client_timeout': _("timeout (in seconds) for iLO operations. Optional."),
-    'ca_file': _("CA certificate file to validate iLO. optional"),
+    'ca_file': _("CA certificate file to validate iLO. Optional")
 }
+
+SNMP_PROPERTIES = {
+    'snmp_auth_user': _("User for SNMPv3. "
+                        "Required for SNMP inspection"),
+    'snmp_auth_prot_password': _("Authentication Protocol Passphrase. "
+                                 "Required for SNMP inspection"),
+    'snmp_auth_priv_password': _("Authentication Privacy Passphrase. "
+                                 "Required for SNMP inspection"),
+}
+
+SNMP_OPTIONAL_PROPERTIES = {
+    'snmp_auth_protocol': _("Authentication Protocol. Optional, used "
+                            "for SNMP inspection. If not specified, the "
+                            "default value as 'MD5' is used."),
+    'snmp_auth_priv_protocol': _("Privacy Protocol. Optional, "
+                                 "used for SNMP inspection. "
+                                 "If not specified, the default value "
+                                 "as 'DES' is used.")
+}
+
 CONSOLE_PROPERTIES = {
     'console_port': _("node's UDP port to connect to. Only required for "
                       "console access.")
@@ -138,7 +156,7 @@ def copy_image_to_swift(source_file_path, destination_object_name):
     container = CONF.ilo.swift_ilo_container
     timeout = CONF.ilo.swift_object_expiry_timeout
 
-    object_headers = {'X-Delete-After': timeout}
+    object_headers = {'X-Delete-After': str(timeout)}
     swift_api = swift.SwiftAPI()
     swift_api.create_object(container, destination_object_name,
                             source_file_path, object_headers=object_headers)
@@ -166,18 +184,19 @@ def remove_image_from_swift(object_name, associated_with=None):
         swift_api = swift.SwiftAPI()
         swift_api.delete_object(container, object_name)
     except exception.SwiftObjectNotFoundError as e:
-        LOG.warning(
-            _LW("Temporary object %(associated_with_msg)s"
-                "was already deleted from Swift. Error: %(err)s"),
-            {'associated_with_msg': ("associated with %s " % associated_with
-                                     if associated_with else ""), 'err': e})
+        LOG.info("Temporary object %(associated_with_msg)s"
+                 "was already deleted from Swift. Error: %(err)s",
+                 {'associated_with_msg':
+                     ("associated with %s " % associated_with
+                         if associated_with else ""), 'err': e})
     except exception.SwiftOperationError as e:
-        LOG.exception(
-            _LE("Error while deleting temporary swift object %(object_name)s "
-                "%(associated_with_msg)s from %(container)s. Error: %(err)s"),
-            {'object_name': object_name, 'container': container,
-             'associated_with_msg': ("associated with %s" % associated_with
-                                     if associated_with else ""), 'err': e})
+        LOG.exception("Error while deleting temporary swift object "
+                      "%(object_name)s %(associated_with_msg)s from "
+                      "%(container)s. Error: %(err)s",
+                      {'object_name': object_name, 'container': container,
+                       'associated_with_msg':
+                           ("associated with %s" % associated_with
+                               if associated_with else ""), 'err': e})
 
 
 def parse_driver_info(node):
@@ -224,6 +243,10 @@ def parse_driver_info(node):
             except ValueError:
                 not_integers.append(param)
 
+    snmp_info = _parse_snmp_driver_info(info)
+    if snmp_info:
+        d_info.update(snmp_info)
+
     for param in CONSOLE_PROPERTIES:
         value = info.get(param)
         if value:
@@ -240,6 +263,49 @@ def parse_driver_info(node):
     return d_info
 
 
+def _parse_snmp_driver_info(info):
+    """Parses the SNMP related driver_info parameters.
+
+    :param info: driver_info dictionary.
+    :returns: a dictionary containing SNMP information.
+    :raises exception.MissingParameterValue: if any of the mandatory
+        parameter values are not provided.
+    :raises exception.InvalidParameterValue: if the value provided
+        for SNMP_OPTIONAL_PROPERTIES has an invalid value.
+    """
+    snmp_info = {}
+    missing_info = []
+    valid_values = {'snmp_auth_protocol': ['MD5', 'SHA'],
+                    'snmp_auth_priv_protocol': ['AES', 'DES']}
+    if info.get('snmp_auth_user'):
+        for param in SNMP_PROPERTIES:
+            try:
+                snmp_info[param] = info[param]
+            except KeyError:
+                missing_info.append(param)
+        if missing_info:
+            raise exception.MissingParameterValue(_(
+                "The following required SNMP parameters are missing from the "
+                "node's driver_info: %s") % missing_info)
+
+        for param in SNMP_OPTIONAL_PROPERTIES:
+            value = None
+            try:
+                value = six.text_type(info[param]).upper()
+            except KeyError:
+                pass
+            if value:
+                if value not in valid_values[param]:
+                    raise exception.InvalidParameterValue(_(
+                        "Invalid value %(value)s given for driver_info "
+                        "parameter %(param)s") % {'param': param,
+                                                  'value': info[param]})
+                snmp_info[param] = value
+    else:
+        snmp_info = None
+    return snmp_info
+
+
 def get_ilo_object(node):
     """Gets an IloClient object from proliantutils library.
 
@@ -253,49 +319,32 @@ def get_ilo_object(node):
         is missing on the node
     """
     driver_info = parse_driver_info(node)
+    snmp_info = _parse_snmp_driver_info(driver_info)
+    info = {}
+    # This mapping is done as per what proliantutils expect the input
+    # to be. This will be removed once proliantutils is fixed for this
+    # in its next release.
+    if snmp_info:
+        info['snmp_inspection'] = True
+        info['auth_user'] = snmp_info['snmp_auth_user']
+        info['auth_prot_pp'] = snmp_info['snmp_auth_prot_password']
+        info['auth_priv_pp'] = snmp_info['snmp_auth_priv_password']
+        if snmp_info.get('snmp_auth_protocol'):
+            info['auth_protocol'] = str(snmp_info['snmp_auth_protocol'])
+        if snmp_info.get('snmp_auth_priv_protocol'):
+            info['priv_protocol'] = str(snmp_info['snmp_auth_priv_protocol'])
+    else:
+        info = None
     ilo_object = ilo_client.IloClient(driver_info['ilo_address'],
                                       driver_info['ilo_username'],
                                       driver_info['ilo_password'],
                                       driver_info['client_timeout'],
                                       driver_info['client_port'],
-                                      cacert=driver_info.get('ca_file'))
+                                      cacert=driver_info.get('ca_file'),
+                                      snmp_credentials=info)
     return ilo_object
 
 
-def get_ilo_license(node):
-    """Gives the current installed license on the node.
-
-    Given an ironic node object, this method queries the iLO
-    for currently installed license and returns it back.
-
-    :param node: an ironic node object.
-    :returns: a constant defined in this module which
-        refers to the current license installed on the node.
-    :raises: InvalidParameterValue on invalid inputs.
-    :raises: MissingParameterValue if some mandatory information
-        is missing on the node
-    :raises: IloOperationError if it failed to retrieve the
-        installed licenses from the iLO.
-    """
-    # Get the ilo client object, and then the license from the iLO
-    ilo_object = get_ilo_object(node)
-    try:
-        license_info = ilo_object.get_all_licenses()
-    except ilo_error.IloError as ilo_exception:
-        raise exception.IloOperationError(operation=_('iLO license check'),
-                                          error=str(ilo_exception))
-
-    # Check the license to see if the given license exists
-    current_license_type = license_info['LICENSE_TYPE']
-
-    if current_license_type.endswith("Advanced"):
-        return ADVANCED_LICENSE
-    elif current_license_type.endswith("Essentials"):
-        return ESSENTIALS_LICENSE
-    else:
-        return STANDARD_LICENSE
-
-
 def update_ipmi_properties(task):
     """Update ipmi properties to node driver_info
 
@@ -389,7 +438,7 @@ def attach_vmedia(node, device, url):
         raise exception.IloOperationError(
             operation=operation, error=ilo_exception)
 
-    LOG.info(_LI("Attached virtual media %s successfully."), device)
+    LOG.info("Attached virtual media %s successfully.", device)
 
 
 def set_boot_mode(node, boot_mode):
@@ -407,7 +456,7 @@ def set_boot_mode(node, boot_mode):
         p_boot_mode = DEFAULT_BOOT_MODE
 
     if BOOT_MODE_ILO_TO_GENERIC[p_boot_mode.lower()] == boot_mode:
-        LOG.info(_LI("Node %(uuid)s pending boot mode is %(boot_mode)s."),
+        LOG.info("Node %(uuid)s pending boot mode is %(boot_mode)s.",
                  {'uuid': node.uuid, 'boot_mode': boot_mode})
         return
 
@@ -419,7 +468,7 @@ def set_boot_mode(node, boot_mode):
         raise exception.IloOperationError(
             operation=operation, error=ilo_exception)
 
-    LOG.info(_LI("Node %(uuid)s boot mode is set to %(boot_mode)s."),
+    LOG.info("Node %(uuid)s boot mode is set to %(boot_mode)s.",
              {'uuid': node.uuid, 'boot_mode': boot_mode})
 
 
@@ -435,14 +484,24 @@ def update_boot_mode(task):
     """
 
     node = task.node
-    boot_mode = deploy_utils.get_boot_mode_for_deploy(node)
+    boot_mode = boot_mode_utils.get_boot_mode_for_deploy(node)
 
-    if boot_mode is not None:
+    # No boot mode found. Check if default_boot_mode is defined
+    if not boot_mode and (CONF.ilo.default_boot_mode in ['bios', 'uefi']):
+        boot_mode = CONF.ilo.default_boot_mode
+        instance_info = node.instance_info
+        instance_info['deploy_boot_mode'] = boot_mode
+        node.instance_info = instance_info
+        node.save()
+
+    # Boot mode is computed, setting it for the deploy
+    if boot_mode:
         LOG.debug("Node %(uuid)s boot mode is being set to %(boot_mode)s",
                   {'uuid': node.uuid, 'boot_mode': boot_mode})
         set_boot_mode(node, boot_mode)
         return
 
+    # Computing boot mode based on boot mode settings on bare metal
     LOG.debug("Check pending boot mode for node %s.", node.uuid)
     ilo_object = get_ilo_object(node)
 
@@ -530,7 +589,7 @@ def setup_vmedia_for_boot(task, boot_iso, parameters=None):
     :raises: SwiftOperationError, if any operation with Swift fails.
     :raises: IloOperationError, if attaching virtual media failed.
     """
-    LOG.info(_LI("Setting up node %s to boot from virtual media"),
+    LOG.info("Setting up node %s to boot from virtual media",
              task.node.uuid)
     if parameters:
         floppy_image_temp_url = _prepare_floppy_image(task, parameters)
@@ -566,8 +625,8 @@ def eject_vmedia_devices(task):
         try:
             ilo_object.eject_virtual_media(device)
         except ilo_error.IloError as ilo_exception:
-            LOG.error(_LE("Error while ejecting virtual media %(device)s "
-                          "from node %(uuid)s. Error: %(error)s"),
+            LOG.error("Error while ejecting virtual media %(device)s "
+                      "from node %(uuid)s. Error: %(error)s",
                       {'device': device, 'uuid': task.node.uuid,
                        'error': ilo_exception})
             operation = _("Eject virtual media %s") % device.lower()
@@ -675,7 +734,7 @@ def update_secure_boot_mode(task, mode):
     """
     if deploy_utils.is_secure_boot_requested(task.node):
         set_secure_boot_mode(task, mode)
-        LOG.info(_LI('Changed secure boot to %(mode)s for node %(node)s'),
+        LOG.info('Changed secure boot to %(mode)s for node %(node)s',
                  {'mode': mode, 'node': task.node.uuid})
 
 
@@ -713,8 +772,7 @@ def verify_image_checksum(image_location, expected_checksum):
         with open(image_location, 'rb') as fd:
             actual_checksum = utils.hash_file(fd)
     except IOError as e:
-        LOG.error(_LE("Error opening file: %(file)s"),
-                  {'file': image_location})
+        LOG.error("Error opening file: %(file)s", {'file': image_location})
         raise exception.ImageRefValidationFailed(image_href=image_location,
                                                  reason=e)
 
