@@ -24,8 +24,12 @@ def register_all():
     # NOTE(danms): You must make sure your object gets imported in this
     # function in order for it to be registered by services that may
     # need to receive it via RPC.
+    __import__('ironic.objects.bios')
     __import__('ironic.objects.chassis')
     __import__('ironic.objects.conductor')
     __import__('ironic.objects.node')
     __import__('ironic.objects.port')
     __import__('ironic.objects.portgroup')
+    __import__('ironic.objects.trait')
+    __import__('ironic.objects.volume_connector')
+    __import__('ironic.objects.volume_target')
