@@ -23,22 +23,28 @@ inline callbacks.
 
 import copy
 import os
+import subprocess
 import sys
 import tempfile
 
 import eventlet
-eventlet.monkey_patch(os=False)
+eventlet.monkey_patch(os=False)  # noqa E402
 import fixtures
+from ironic_lib import utils
+from oslo_concurrency import processutils
 from oslo_config import fixture as config_fixture
 from oslo_log import log as logging
 from oslo_serialization import jsonutils
 from oslo_utils import uuidutils
-import testtools
+from oslotest import base as oslo_test_base
 
 from ironic.common import config as ironic_config
 from ironic.common import context as ironic_context
+from ironic.common import driver_factory
 from ironic.common import hash_ring
+from ironic.common import utils as common_utils
 from ironic.conf import CONF
+from ironic.drivers import base as drivers_base
 from ironic.objects import base as objects_base
 from ironic.tests.unit import policy_fixture
 
@@ -68,39 +74,20 @@ class TestingException(Exception):
     pass
 
 
-class TestCase(testtools.TestCase):
+class TestCase(oslo_test_base.BaseTestCase):
     """Test case base class for all unit tests."""
 
+    # By default block execution of utils.execute() and related functions.
+    block_execute = True
+
     def setUp(self):
         """Run before each test method to initialize test environment."""
         super(TestCase, self).setUp()
         self.context = ironic_context.get_admin_context()
-        test_timeout = os.environ.get('OS_TEST_TIMEOUT', 0)
-        try:
-            test_timeout = int(test_timeout)
-        except ValueError:
-            # If timeout value is invalid do not set a timeout.
-            test_timeout = 0
-        if test_timeout > 0:
-            self.useFixture(fixtures.Timeout(test_timeout, gentle=True))
-        self.useFixture(fixtures.NestedTempfile())
-        self.useFixture(fixtures.TempHomeDir())
-
-        if (os.environ.get('OS_STDOUT_CAPTURE') == 'True' or
-                os.environ.get('OS_STDOUT_CAPTURE') == '1'):
-            stdout = self.useFixture(fixtures.StringStream('stdout')).stream
-            self.useFixture(fixtures.MonkeyPatch('sys.stdout', stdout))
-        if (os.environ.get('OS_STDERR_CAPTURE') == 'True' or
-                os.environ.get('OS_STDERR_CAPTURE') == '1'):
-            stderr = self.useFixture(fixtures.StringStream('stderr')).stream
-            self.useFixture(fixtures.MonkeyPatch('sys.stderr', stderr))
-
-        self.log_fixture = self.useFixture(fixtures.FakeLogger())
         self._set_config()
         # NOTE(danms): Make sure to reset us back to non-remote objects
         # for each test to avoid interactions. Also, backup the object
         # registry
-        objects_base.IronicObject.indirection_api = None
         self._base_test_obj_backup = copy.copy(
             objects_base.IronicObjectRegistry.obj_classes())
         self.addCleanup(self._restore_obj_registry)
@@ -110,16 +97,57 @@ class TestCase(testtools.TestCase):
         self.useFixture(fixtures.EnvironmentVariable('http_proxy'))
         self.policy = self.useFixture(policy_fixture.PolicyFixture())
 
+        driver_factory.HardwareTypesFactory._extension_manager = None
+        for factory in driver_factory._INTERFACE_LOADERS.values():
+            factory._extension_manager = None
+
+        # Ban running external processes via 'execute' like functions. If the
+        # patched function is called, an exception is raised to warn the
+        # tester.
+        if self.block_execute:
+            # NOTE(jlvillal): Intentionally not using mock as if you mock a
+            # mock it causes things to not work correctly. As doing an
+            # autospec=True causes strangeness. By using a simple function we
+            # can then mock it without issue.
+            self.patch(processutils, 'execute', do_not_call)
+            self.patch(subprocess, 'call', do_not_call)
+            self.patch(subprocess, 'check_call', do_not_call)
+            self.patch(subprocess, 'check_output', do_not_call)
+            self.patch(utils, 'execute', do_not_call)
+            # subprocess.Popen is a class
+            self.patch(subprocess, 'Popen', DoNotCallPopen)
+
     def _set_config(self):
         self.cfg_fixture = self.useFixture(config_fixture.Config(CONF))
         self.config(use_stderr=False,
                     fatal_exception_format_errors=True,
                     tempdir=tempfile.tempdir)
-        self.config(cleaning_network_uuid=uuidutils.generate_uuid(),
+        self.config(cleaning_network=uuidutils.generate_uuid(),
                     group='neutron')
-        self.config(provisioning_network_uuid=uuidutils.generate_uuid(),
+        self.config(provisioning_network=uuidutils.generate_uuid(),
                     group='neutron')
-        self.config(enabled_network_interfaces=['flat', 'noop', 'neutron'])
+        self.config(rescuing_network=uuidutils.generate_uuid(),
+                    group='neutron')
+        self.config(enabled_hardware_types=['fake-hardware',
+                                            'manual-management'])
+        for iface in drivers_base.ALL_INTERFACES:
+            # Restore some reasonable defaults
+            if iface == 'network':
+                values = ['flat', 'noop', 'neutron']
+            else:
+                values = ['fake']
+
+            if iface == 'deploy':
+                values.extend(['iscsi', 'direct'])
+            elif iface == 'boot':
+                values.append('pxe')
+            elif iface == 'storage':
+                values.append('noop')
+            elif iface not in {'network', 'power', 'management'}:
+                values.append('no-%s' % iface)
+
+            self.config(**{'enabled_%s_interfaces' % iface: values,
+                           'default_%s_interface' % iface: None})
         self.set_defaults(host='fake-mini',
                           debug=True)
         self.set_defaults(connection="sqlite://",
@@ -142,6 +170,12 @@ class TestCase(testtools.TestCase):
         """Override config options for a test."""
         self.cfg_fixture.config(**kw)
 
+    def config_temp_dir(self, option, group=None):
+        """Override a config option with a temporary directory."""
+        temp_dir = tempfile.mkdtemp()
+        self.addCleanup(lambda: common_utils.rmtree_without_raise(temp_dir))
+        self.config(**{option: temp_dir, 'group': group})
+
     def set_defaults(self, **kw):
         """Set default values of config options."""
         group = kw.pop('group', None)
@@ -168,3 +202,52 @@ class TestCase(testtools.TestCase):
         """Asserts that 2 complex data structures are json equivalent."""
         self.assertEqual(jsonutils.dumps(expected, sort_keys=True),
                          jsonutils.dumps(observed, sort_keys=True))
+
+    def assertNotificationEqual(self, notif_args, service, host, event_type,
+                                level):
+        """Asserts properties of arguments passed when creating a notification.
+
+           :param notif_args: dict of arguments notification instantiated with
+           :param service: expected service that emits the notification
+           :param host: expected host that emits the notification
+           :param event_type: expected value of EventType field of notification
+                              as a string
+           :param level: expected NotificationLevel
+       """
+        self.assertEqual(service, notif_args['publisher'].service)
+        self.assertEqual(host, notif_args['publisher'].host)
+        self.assertEqual(event_type, notif_args['event_type'].
+                         to_event_type_field())
+        self.assertEqual(level, notif_args['level'])
+
+
+def do_not_call(*args, **kwargs):
+    """Helper function to raise an exception if it is called"""
+    raise Exception(
+        "Don't call ironic_lib.utils.execute() / "
+        "processutils.execute() or similar functions in tests!")
+
+
+class DoNotCallPopen(object):
+    """Helper class to mimic subprocess.popen()
+
+    It's job is to raise an exception if it is called. We create stub functions
+    so mocks that use autospec=True will work.
+    """
+    def __init__(self, *args, **kwargs):
+        do_not_call(*args, **kwargs)
+
+    def communicate(input=None):
+        pass
+
+    def kill():
+        pass
+
+    def poll():
+        pass
+
+    def terminate():
+        pass
+
+    def wait():
+        pass
