@@ -21,12 +21,12 @@ import os.path
 import shutil
 import tempfile
 
+import jinja2
 import mock
-import netaddr
 from oslo_concurrency import processutils
 from oslo_config import cfg
+from oslo_utils import netutils
 import six
-import six.moves.builtins as __builtin__
 
 from ironic.common import exception
 from ironic.common import utils
@@ -37,12 +37,6 @@ CONF = cfg.CONF
 
 class BareMetalUtilsTestCase(base.TestCase):
 
-    def test_random_alnum(self):
-        s = utils.random_alnum(10)
-        self.assertEqual(10, len(s))
-        s = utils.random_alnum(100)
-        self.assertEqual(100, len(s))
-
     def test_create_link(self):
         with mock.patch.object(os, "symlink", autospec=True) as symlink_mock:
             symlink_mock.return_value = None
@@ -58,100 +52,6 @@ class BareMetalUtilsTestCase(base.TestCase):
 
 class ExecuteTestCase(base.TestCase):
 
-    def test_retry_on_failure(self):
-        fd, tmpfilename = tempfile.mkstemp()
-        _, tmpfilename2 = tempfile.mkstemp()
-        try:
-            fp = os.fdopen(fd, 'w+')
-            fp.write('''#!/bin/sh
-# If stdin fails to get passed during one of the runs, make a note.
-if ! grep -q foo
-then
-    echo 'failure' > "$1"
-fi
-# If stdin has failed to get passed during this or a previous run, exit early.
-if grep failure "$1"
-then
-    exit 1
-fi
-runs="$(cat $1)"
-if [ -z "$runs" ]
-then
-    runs=0
-fi
-runs=$(($runs + 1))
-echo $runs > "$1"
-exit 1
-''')
-            fp.close()
-            os.chmod(tmpfilename, 0o755)
-            try:
-                self.assertRaises(processutils.ProcessExecutionError,
-                                  utils.execute,
-                                  tmpfilename, tmpfilename2, attempts=10,
-                                  process_input=b'foo',
-                                  delay_on_retry=False)
-            except OSError as e:
-                if e.errno == errno.EACCES:
-                    self.skipTest("Permissions error detected. "
-                                  "Are you running with a noexec /tmp?")
-                else:
-                    raise
-            fp = open(tmpfilename2, 'r')
-            runs = fp.read()
-            fp.close()
-            self.assertNotEqual(runs.strip(), 'failure', 'stdin did not '
-                                'always get passed '
-                                'correctly')
-            runs = int(runs.strip())
-            self.assertEqual(10, runs,
-                             'Ran %d times instead of 10.' % (runs,))
-        finally:
-            os.unlink(tmpfilename)
-            os.unlink(tmpfilename2)
-
-    def test_unknown_kwargs_raises_error(self):
-        self.assertRaises(processutils.UnknownArgumentError,
-                          utils.execute,
-                          '/usr/bin/env', 'true',
-                          this_is_not_a_valid_kwarg=True)
-
-    def test_check_exit_code_boolean(self):
-        utils.execute('/usr/bin/env', 'false', check_exit_code=False)
-        self.assertRaises(processutils.ProcessExecutionError,
-                          utils.execute,
-                          '/usr/bin/env', 'false', check_exit_code=True)
-
-    def test_no_retry_on_success(self):
-        fd, tmpfilename = tempfile.mkstemp()
-        _, tmpfilename2 = tempfile.mkstemp()
-        try:
-            fp = os.fdopen(fd, 'w+')
-            fp.write('''#!/bin/sh
-# If we've already run, bail out.
-grep -q foo "$1" && exit 1
-# Mark that we've run before.
-echo foo > "$1"
-# Check that stdin gets passed correctly.
-grep foo
-''')
-            fp.close()
-            os.chmod(tmpfilename, 0o755)
-            try:
-                utils.execute(tmpfilename,
-                              tmpfilename2,
-                              process_input=b'foo',
-                              attempts=2)
-            except OSError as e:
-                if e.errno == errno.EACCES:
-                    self.skipTest("Permissions error detected. "
-                                  "Are you running with a noexec /tmp?")
-                else:
-                    raise
-        finally:
-            os.unlink(tmpfilename)
-            os.unlink(tmpfilename2)
-
     @mock.patch.object(processutils, 'execute', autospec=True)
     @mock.patch.object(os.environ, 'copy', return_value={}, autospec=True)
     def test_execute_use_standard_locale_no_env_variables(self, env_mock,
@@ -192,75 +92,6 @@ grep foo
 
 
 class GenericUtilsTestCase(base.TestCase):
-    def test_hostname_unicode_sanitization(self):
-        hostname = u"\u7684.test.example.com"
-        self.assertEqual(b"test.example.com",
-                         utils.sanitize_hostname(hostname))
-
-    def test_hostname_sanitize_periods(self):
-        hostname = "....test.example.com..."
-        self.assertEqual(b"test.example.com",
-                         utils.sanitize_hostname(hostname))
-
-    def test_hostname_sanitize_dashes(self):
-        hostname = "----test.example.com---"
-        self.assertEqual(b"test.example.com",
-                         utils.sanitize_hostname(hostname))
-
-    def test_hostname_sanitize_characters(self):
-        hostname = "(#@&$!(@*--#&91)(__=+--test-host.example!!.com-0+"
-        self.assertEqual(b"91----test-host.example.com-0",
-                         utils.sanitize_hostname(hostname))
-
-    def test_hostname_translate(self):
-        hostname = "<}\x1fh\x10e\x08l\x02l\x05o\x12!{>"
-        self.assertEqual(b"hello", utils.sanitize_hostname(hostname))
-
-    def test_read_cached_file(self):
-        with mock.patch.object(
-                os.path, "getmtime", autospec=True) as getmtime_mock:
-            getmtime_mock.return_value = 1
-
-            cache_data = {"data": 1123, "mtime": 1}
-            data = utils.read_cached_file("/this/is/a/fake", cache_data)
-            self.assertEqual(cache_data["data"], data)
-            getmtime_mock.assert_called_once_with(mock.ANY)
-
-    def test_read_modified_cached_file(self):
-        with mock.patch.object(
-                os.path, "getmtime", autospec=True) as getmtime_mock:
-            with mock.patch.object(
-                    __builtin__, 'open', autospec=True) as open_mock:
-                getmtime_mock.return_value = 2
-                fake_contents = "lorem ipsum"
-                fake_file = mock.Mock()
-                fake_file.read.return_value = fake_contents
-                fake_context_manager = mock.MagicMock()
-                fake_context_manager.__enter__.return_value = fake_file
-                fake_context_manager.__exit__.return_value = None
-                open_mock.return_value = fake_context_manager
-
-                cache_data = {"data": 1123, "mtime": 1}
-                self.reload_called = False
-
-                def test_reload(reloaded_data):
-                    self.assertEqual(fake_contents, reloaded_data)
-                    self.reload_called = True
-
-                data = utils.read_cached_file("/this/is/a/fake",
-                                              cache_data,
-                                              reload_func=test_reload)
-
-                self.assertEqual(fake_contents, data)
-                self.assertTrue(self.reload_called)
-                getmtime_mock.assert_called_once_with(mock.ANY)
-                open_mock.assert_called_once_with(mock.ANY)
-                fake_file.read.assert_called_once_with()
-                fake_context_manager.__exit__.assert_called_once_with(mock.ANY,
-                                                                      mock.ANY,
-                                                                      mock.ANY)
-                fake_context_manager.__enter__.assert_called_once_with()
-
     @mock.patch.object(utils, 'hashlib', autospec=True)
     def test__get_hash_object(self, hashlib_mock):
         algorithms_available = ('md5', 'sha1', 'sha224',
@@ -296,6 +127,16 @@ class GenericUtilsTestCase(base.TestCase):
         # | THEN |
         self.assertEqual(expected, actual)
 
+    def test_hash_file_for_md5_not_binary(self):
+        # | GIVEN |
+        data = u'Mary had a little lamb, its fleece as white as sno\u0449'
+        file_like_object = six.StringIO(data)
+        expected = hashlib.md5(data.encode('utf-8')).hexdigest()
+        # | WHEN |
+        actual = utils.hash_file(file_like_object)  # using default, 'md5'
+        # | THEN |
+        self.assertEqual(expected, actual)
+
     def test_hash_file_for_sha1(self):
         # | GIVEN |
         data = b'Mary had a little lamb, its fleece as white as snow'
@@ -324,66 +165,29 @@ class GenericUtilsTestCase(base.TestCase):
         self.assertRaises(exception.InvalidParameterValue, utils.hash_file,
                           file_like_object, 'hickory-dickory-dock')
 
-    def test_is_valid_boolstr(self):
-        self.assertTrue(utils.is_valid_boolstr('true'))
-        self.assertTrue(utils.is_valid_boolstr('false'))
-        self.assertTrue(utils.is_valid_boolstr('yes'))
-        self.assertTrue(utils.is_valid_boolstr('no'))
-        self.assertTrue(utils.is_valid_boolstr('y'))
-        self.assertTrue(utils.is_valid_boolstr('n'))
-        self.assertTrue(utils.is_valid_boolstr('1'))
-        self.assertTrue(utils.is_valid_boolstr('0'))
+    def test_file_has_content_equal(self):
+        data = b'Mary had a little lamb, its fleece as white as snow'
+        ref = data
+        with mock.patch('ironic.common.utils.open',
+                        mock.mock_open(read_data=data)) as mopen:
+            self.assertTrue(utils.file_has_content('foo', ref))
+            mopen.assert_called_once_with('foo', 'rb')
 
-        self.assertFalse(utils.is_valid_boolstr('maybe'))
-        self.assertFalse(utils.is_valid_boolstr('only on tuesdays'))
+    def test_file_has_content_equal_not_binary(self):
+        data = u'Mary had a little lamb, its fleece as white as sno\u0449'
+        ref = data
+        with mock.patch('ironic.common.utils.open',
+                        mock.mock_open(read_data=data)) as mopen:
+            self.assertTrue(utils.file_has_content('foo', ref))
+            mopen.assert_called_once_with('foo', 'rb')
 
-    def test_is_valid_ipv6_cidr(self):
-        self.assertTrue(utils.is_valid_ipv6_cidr("2600::/64"))
-        self.assertTrue(utils.is_valid_ipv6_cidr(
-            "abcd:ef01:2345:6789:abcd:ef01:192.168.254.254/48"))
-        self.assertTrue(utils.is_valid_ipv6_cidr(
-            "0000:0000:0000:0000:0000:0000:0000:0001/32"))
-        self.assertTrue(utils.is_valid_ipv6_cidr(
-            "0000:0000:0000:0000:0000:0000:0000:0001"))
-        self.assertFalse(utils.is_valid_ipv6_cidr("foo"))
-        self.assertFalse(utils.is_valid_ipv6_cidr("127.0.0.1"))
-
-    def test_get_shortened_ipv6(self):
-        self.assertEqual("abcd:ef01:2345:6789:abcd:ef01:c0a8:fefe",
-                         utils.get_shortened_ipv6(
-                             "abcd:ef01:2345:6789:abcd:ef01:192.168.254.254"))
-        self.assertEqual("::1", utils.get_shortened_ipv6(
-            "0000:0000:0000:0000:0000:0000:0000:0001"))
-        self.assertEqual("caca::caca:0:babe:201:102",
-                         utils.get_shortened_ipv6(
-                             "caca:0000:0000:caca:0000:babe:0201:0102"))
-        self.assertRaises(netaddr.AddrFormatError, utils.get_shortened_ipv6,
-                          "127.0.0.1")
-        self.assertRaises(netaddr.AddrFormatError, utils.get_shortened_ipv6,
-                          "failure")
-
-    def test_get_shortened_ipv6_cidr(self):
-        self.assertEqual("2600::/64", utils.get_shortened_ipv6_cidr(
-            "2600:0000:0000:0000:0000:0000:0000:0000/64"))
-        self.assertEqual("2600::/64", utils.get_shortened_ipv6_cidr(
-            "2600::1/64"))
-        self.assertRaises(netaddr.AddrFormatError,
-                          utils.get_shortened_ipv6_cidr,
-                          "127.0.0.1")
-        self.assertRaises(netaddr.AddrFormatError,
-                          utils.get_shortened_ipv6_cidr,
-                          "failure")
-
-    def test_is_valid_mac(self):
-        self.assertTrue(utils.is_valid_mac("52:54:00:cf:2d:31"))
-        self.assertTrue(utils.is_valid_mac(u"52:54:00:cf:2d:31"))
-        self.assertFalse(utils.is_valid_mac("127.0.0.1"))
-        self.assertFalse(utils.is_valid_mac("not:a:mac:address"))
-        self.assertFalse(utils.is_valid_mac("52-54-00-cf-2d-31"))
-        self.assertFalse(utils.is_valid_mac("aa bb cc dd ee ff"))
-        self.assertTrue(utils.is_valid_mac("AA:BB:CC:DD:EE:FF"))
-        self.assertFalse(utils.is_valid_mac("AA BB CC DD EE FF"))
-        self.assertFalse(utils.is_valid_mac("AA-BB-CC-DD-EE-FF"))
+    def test_file_has_content_differ(self):
+        data = b'Mary had a little lamb, its fleece as white as snow'
+        ref = data + b'!'
+        with mock.patch('ironic.common.utils.open',
+                        mock.mock_open(read_data=data)) as mopen:
+            self.assertFalse(utils.file_has_content('foo', ref))
+            mopen.assert_called_once_with('foo', 'rb')
 
     def test_is_valid_datapath_id(self):
         self.assertTrue(utils.is_valid_datapath_id("525400cf2d319fdf"))
@@ -459,7 +263,8 @@ class GenericUtilsTestCase(base.TestCase):
 
     def test_validate_and_normalize_mac(self):
         mac = 'AA:BB:CC:DD:EE:FF'
-        with mock.patch.object(utils, 'is_valid_mac', autospec=True) as m_mock:
+        with mock.patch.object(netutils, 'is_valid_mac',
+                               autospec=True) as m_mock:
             m_mock.return_value = True
             self.assertEqual(mac.lower(),
                              utils.validate_and_normalize_mac(mac))
@@ -474,7 +279,8 @@ class GenericUtilsTestCase(base.TestCase):
                                  datapath_id))
 
     def test_validate_and_normalize_mac_invalid_format(self):
-        with mock.patch.object(utils, 'is_valid_mac', autospec=True) as m_mock:
+        with mock.patch.object(netutils, 'is_valid_mac',
+                               autospec=True) as m_mock:
             m_mock.return_value = False
             self.assertRaises(exception.InvalidMAC,
                               utils.validate_and_normalize_mac, 'invalid-mac')
@@ -510,8 +316,8 @@ class GenericUtilsTestCase(base.TestCase):
         valid_no_proxy = [
             ('a' * 63 + '.' + '0' * 63 + '.c.' + 'd' * 61 + '.' + 'e' * 61),
             ('A' * 63 + '.' + '0' * 63 + '.C.' + 'D' * 61 + '.' + 'E' * 61),
-            ('.' + 'a' * 62 + '.' + '0' * 62 + '.c.' + 'd' * 61 + '.' +
-             'e' * 61),
+            ('.' + 'a' * 62 + '.' + '0' * 62 + '.c.' + 'd' * 61 + '.'
+             + 'e' * 61),
             ',,example.com:3128,',
             '192.168.1.1',  # IP should be valid
         ]
@@ -530,14 +336,14 @@ class GenericUtilsTestCase(base.TestCase):
 
         # Invalid values for 'no_proxy'
         invalid_no_proxy = [
-            ('A' * 64 + '.' + '0' * 63 + '.C.' + 'D' * 61 + '.' +
-             'E' * 61),  # too long (> 253)
+            ('A' * 64 + '.' + '0' * 63 + '.C.' + 'D' * 61 + '.'
+             + 'E' * 61),  # too long (> 253)
             ('a' * 100),
             'a..com',
-            ('.' + 'a' * 63 + '.' + '0' * 62 + '.c.' + 'd' * 61 + '.' +
-             'e' * 61),  # too long (> 251 after deleting .)
-            ('*.' + 'a' * 60 + '.' + '0' * 60 + '.c.' + 'd' * 61 + '.' +
-             'e' * 61),  # starts with *.
+            ('.' + 'a' * 63 + '.' + '0' * 62 + '.c.' + 'd' * 61 + '.'
+             + 'e' * 61),  # too long (> 251 after deleting .)
+            ('*.' + 'a' * 60 + '.' + '0' * 60 + '.c.' + 'd' * 61 + '.'
+             + 'e' * 61),  # starts with *.
             'c.-a.com',
             'c.a-.com',
         ]
@@ -547,6 +353,16 @@ class GenericUtilsTestCase(base.TestCase):
                 utils.is_valid_no_proxy(no_proxy),
                 msg="'no_proxy' value should be invalid: {}".format(no_proxy))
 
+    @mock.patch.object(utils, 'LOG', autospec=True)
+    def test_warn_about_deprecated_extra_vif_port_id(self, mock_log):
+        # Set variable to default value
+        utils.warn_deprecated_extra_vif_port_id = False
+        utils.warn_about_deprecated_extra_vif_port_id()
+        utils.warn_about_deprecated_extra_vif_port_id()
+        self.assertEqual(1, mock_log.warning.call_count)
+        self.assertIn("extra['vif_port_id'] is deprecated and will not",
+                      mock_log.warning.call_args[0][0])
+
 
 class TempFilesTestCase(base.TestCase):
 
@@ -602,27 +418,31 @@ class TempFilesTestCase(base.TestCase):
         mock_dir_writable.assert_called_once_with('/fake/path')
         mock_free_space.assert_called_once_with('/fake/path', 5)
 
-    @mock.patch.object(os.path, 'exists', autospec=True)
     @mock.patch.object(utils, '_check_dir_writable', autospec=True)
     @mock.patch.object(utils, '_check_dir_free_space', autospec=True)
-    def test_check_dir_no_dir(self, mock_free_space, mock_dir_writable,
-                              mock_exists):
-        mock_exists.return_value = False
+    def test_check_dir_no_dir(self, mock_free_space, mock_dir_writable):
         self.config(tempdir='/fake/path')
-        self.assertRaises(exception.PathNotFound, utils.check_dir)
-        mock_exists.assert_called_once_with(CONF.tempdir)
+        # NOTE(dtantsur): self.config uses os.path.exists, so we cannot mock
+        # on the method level.
+        with mock.patch.object(os.path, 'exists',
+                               autospec=True) as mock_exists:
+            mock_exists.return_value = False
+            self.assertRaises(exception.PathNotFound, utils.check_dir)
+            mock_exists.assert_called_once_with(CONF.tempdir)
         self.assertFalse(mock_free_space.called)
         self.assertFalse(mock_dir_writable.called)
 
-    @mock.patch.object(os.path, 'exists', autospec=True)
     @mock.patch.object(utils, '_check_dir_writable', autospec=True)
     @mock.patch.object(utils, '_check_dir_free_space', autospec=True)
-    def test_check_dir_ok(self, mock_free_space, mock_dir_writable,
-                          mock_exists):
-        mock_exists.return_value = True
+    def test_check_dir_ok(self, mock_free_space, mock_dir_writable):
         self.config(tempdir='/fake/path')
-        utils.check_dir()
-        mock_exists.assert_called_once_with(CONF.tempdir)
+        # NOTE(dtantsur): self.config uses os.path.exists, so we cannot mock
+        # on the method level.
+        with mock.patch.object(os.path, 'exists',
+                               autospec=True) as mock_exists:
+            mock_exists.return_value = True
+            utils.check_dir()
+            mock_exists.assert_called_once_with(CONF.tempdir)
         mock_dir_writable.assert_called_once_with(CONF.tempdir)
         mock_free_space.assert_called_once_with(CONF.tempdir, 1)
 
@@ -729,22 +549,68 @@ class GetUpdatedCapabilitiesTestCase(base.TestCase):
         self.assertIsInstance(cap_returned, str)
 
     def test_validate_network_port(self):
-        port = utils.validate_network_port('1', 'message')
-        self.assertEqual(1, port)
+        port = utils.validate_network_port('0', 'message')
+        self.assertEqual(0, port)
         port = utils.validate_network_port('65535')
         self.assertEqual(65535, port)
 
     def test_validate_network_port_fail(self):
         self.assertRaisesRegex(exception.InvalidParameterValue,
-                               'Port "65536" is out of range.',
+                               'Port "65536" is not a valid port.',
                                utils.validate_network_port,
                                '65536')
         self.assertRaisesRegex(exception.InvalidParameterValue,
-                               'fake_port "-1" is out of range.',
+                               'fake_port "-1" is not a valid port.',
                                utils.validate_network_port,
                                '-1',
                                'fake_port')
         self.assertRaisesRegex(exception.InvalidParameterValue,
-                               'Port "invalid" is not a valid integer.',
+                               'Port "invalid" is not a valid port.',
                                utils.validate_network_port,
                                'invalid')
+
+
+class JinjaTemplatingTestCase(base.TestCase):
+
+    def setUp(self):
+        super(JinjaTemplatingTestCase, self).setUp()
+        self.template = '{{ foo }} {{ bar }}'
+        self.params = {'foo': 'spam', 'bar': 'ham'}
+        self.expected = 'spam ham'
+
+    def test_render_string(self):
+        self.assertEqual(self.expected,
+                         utils.render_template(self.template,
+                                               self.params,
+                                               is_file=False))
+
+    @mock.patch('ironic.common.utils.jinja2.FileSystemLoader', autospec=True)
+    def test_render_file(self, jinja_fsl_mock):
+        path = '/path/to/template.j2'
+        jinja_fsl_mock.return_value = jinja2.DictLoader(
+            {'template.j2': self.template})
+        self.assertEqual(self.expected,
+                         utils.render_template(path,
+                                               self.params))
+        jinja_fsl_mock.assert_called_once_with('/path/to')
+
+
+class ValidateConductorGroupTestCase(base.TestCase):
+    def test_validate_conductor_group_success(self):
+        self.assertIsNone(utils.validate_conductor_group('foo'))
+        self.assertIsNone(utils.validate_conductor_group('group1'))
+        self.assertIsNone(utils.validate_conductor_group('group1.with.dot'))
+        self.assertIsNone(utils.validate_conductor_group('group1_with_under'))
+        self.assertIsNone(utils.validate_conductor_group('group1-with-dash'))
+
+    def test_validate_conductor_group_fail(self):
+        self.assertRaises(exception.InvalidConductorGroup,
+                          utils.validate_conductor_group, 'foo:bar')
+        self.assertRaises(exception.InvalidConductorGroup,
+                          utils.validate_conductor_group, 'foo*bar')
+        self.assertRaises(exception.InvalidConductorGroup,
+                          utils.validate_conductor_group, 'foo$bar')
+        self.assertRaises(exception.InvalidConductorGroup,
+                          utils.validate_conductor_group, object())
+        self.assertRaises(exception.InvalidConductorGroup,
+                          utils.validate_conductor_group, None)
