@@ -27,13 +27,14 @@ from oslo_messaging import _utils as messaging_utils
 
 from ironic.common import boot_devices
 from ironic.common import exception
+from ironic.common import release_mappings
 from ironic.common import states
 from ironic.conductor import manager as conductor_manager
 from ironic.conductor import rpcapi as conductor_rpcapi
 from ironic import objects
 from ironic.tests import base as tests_base
-from ironic.tests.unit.db import base
-from ironic.tests.unit.db import utils as dbutils
+from ironic.tests.unit.db import base as db_base
+from ironic.tests.unit.db import utils as db_utils
 
 CONF = cfg.CONF
 
@@ -45,23 +46,40 @@ class ConductorRPCAPITestCase(tests_base.TestCase):
             conductor_manager.ConductorManager.RPC_API_VERSION,
             conductor_rpcapi.ConductorAPI.RPC_API_VERSION)
 
+    @mock.patch('ironic.common.rpc.get_client')
+    def test_version_cap(self, mock_get_client):
+        conductor_rpcapi.ConductorAPI()
+        self.assertEqual(conductor_rpcapi.ConductorAPI.RPC_API_VERSION,
+                         mock_get_client.call_args[1]['version_cap'])
 
-class RPCAPITestCase(base.DbTestCase):
+    @mock.patch('ironic.common.release_mappings.RELEASE_MAPPING')
+    @mock.patch('ironic.common.rpc.get_client')
+    def test_version_capped(self, mock_get_client, mock_release_mapping):
+        CONF.set_override('pin_release_version',
+                          release_mappings.RELEASE_VERSIONS[0])
+        mock_release_mapping.get.return_value = {'rpc': '3'}
+        conductor_rpcapi.ConductorAPI()
+        self.assertEqual('3', mock_get_client.call_args[1]['version_cap'])
+
+
+class RPCAPITestCase(db_base.DbTestCase):
 
     def setUp(self):
         super(RPCAPITestCase, self).setUp()
-        self.fake_node = dbutils.get_test_node(driver='fake-driver')
+        self.fake_node = db_utils.get_test_node(driver='fake-driver')
         self.fake_node_obj = objects.Node._from_db_object(
-            objects.Node(self.context), self.fake_node)
-        self.fake_portgroup = dbutils.get_test_portgroup()
+            self.context, objects.Node(), self.fake_node)
+        self.fake_portgroup = db_utils.get_test_portgroup()
 
     def test_serialized_instance_has_uuid(self):
         self.assertIn('uuid', self.fake_node)
 
     def test_get_topic_for_known_driver(self):
         CONF.set_override('host', 'fake-host')
-        self.dbapi.register_conductor({'hostname': 'fake-host',
-                                       'drivers': ['fake-driver']})
+        c = self.dbapi.register_conductor({'hostname': 'fake-host',
+                                           'drivers': []})
+        self.dbapi.register_conductor_hardware_interfaces(
+            c.id, 'fake-driver', 'deploy', ['iscsi', 'direct'], 'iscsi')
 
         rpcapi = conductor_rpcapi.ConductorAPI(topic='fake-topic')
         expected_topic = 'fake-topic.fake-host'
@@ -70,8 +88,10 @@ class RPCAPITestCase(base.DbTestCase):
 
     def test_get_topic_for_unknown_driver(self):
         CONF.set_override('host', 'fake-host')
-        self.dbapi.register_conductor({'hostname': 'fake-host',
-                                       'drivers': ['other-driver']})
+        c = self.dbapi.register_conductor({'hostname': 'fake-host',
+                                           'drivers': []})
+        self.dbapi.register_conductor_hardware_interfaces(
+            c.id, 'other-driver', 'deploy', ['iscsi', 'direct'], 'iscsi')
 
         rpcapi = conductor_rpcapi.ConductorAPI(topic='fake-topic')
         self.assertRaises(exception.NoValidHost,
@@ -82,12 +102,14 @@ class RPCAPITestCase(base.DbTestCase):
         CONF.set_override('host', 'fake-host')
 
         rpcapi = conductor_rpcapi.ConductorAPI(topic='fake-topic')
-        self.assertRaises(exception.NoValidHost,
+        self.assertRaises(exception.TemporaryFailure,
                           rpcapi.get_topic_for,
                           self.fake_node_obj)
 
-        self.dbapi.register_conductor({'hostname': 'fake-host',
-                                       'drivers': ['fake-driver']})
+        c = self.dbapi.register_conductor({'hostname': 'fake-host',
+                                           'drivers': []})
+        self.dbapi.register_conductor_hardware_interfaces(
+            c.id, 'fake-driver', 'deploy', ['iscsi', 'direct'], 'iscsi')
 
         rpcapi = conductor_rpcapi.ConductorAPI(topic='fake-topic')
         expected_topic = 'fake-topic.fake-host'
@@ -96,10 +118,12 @@ class RPCAPITestCase(base.DbTestCase):
 
     def test_get_topic_for_driver_known_driver(self):
         CONF.set_override('host', 'fake-host')
-        self.dbapi.register_conductor({
+        c = self.dbapi.register_conductor({
             'hostname': 'fake-host',
-            'drivers': ['fake-driver'],
+            'drivers': [],
         })
+        self.dbapi.register_conductor_hardware_interfaces(
+            c.id, 'fake-driver', 'deploy', ['iscsi', 'direct'], 'iscsi')
         rpcapi = conductor_rpcapi.ConductorAPI(topic='fake-topic')
         self.assertEqual('fake-topic.fake-host',
                          rpcapi.get_topic_for_driver('fake-driver'))
@@ -108,7 +132,7 @@ class RPCAPITestCase(base.DbTestCase):
         CONF.set_override('host', 'fake-host')
         self.dbapi.register_conductor({
             'hostname': 'fake-host',
-            'drivers': ['other-driver'],
+            'drivers': [],
         })
         rpcapi = conductor_rpcapi.ConductorAPI(topic='fake-topic')
         self.assertRaises(exception.DriverNotFound,
@@ -122,14 +146,31 @@ class RPCAPITestCase(base.DbTestCase):
                           rpcapi.get_topic_for_driver,
                           'fake-driver')
 
-        self.dbapi.register_conductor({
+        c = self.dbapi.register_conductor({
             'hostname': 'fake-host',
-            'drivers': ['fake-driver'],
+            'drivers': [],
         })
+        self.dbapi.register_conductor_hardware_interfaces(
+            c.id, 'fake-driver', 'deploy', ['iscsi', 'direct'], 'iscsi')
         rpcapi = conductor_rpcapi.ConductorAPI(topic='fake-topic')
         self.assertEqual('fake-topic.fake-host',
                          rpcapi.get_topic_for_driver('fake-driver'))
 
+    def _test_can_send_create_port(self, can_send):
+        rpcapi = conductor_rpcapi.ConductorAPI(topic='fake-topic')
+        with mock.patch.object(rpcapi.client,
+                               "can_send_version") as mock_can_send_version:
+            mock_can_send_version.return_value = can_send
+            result = rpcapi.can_send_create_port()
+            self.assertEqual(can_send, result)
+            mock_can_send_version.assert_called_once_with("1.41")
+
+    def test_can_send_create_port_True(self):
+        self._test_can_send_create_port(True)
+
+    def test_can_send_create_port_False(self):
+        self._test_can_send_create_port(False)
+
     def _test_rpcapi(self, method, rpc_method, **kwargs):
         rpcapi = conductor_rpcapi.ConductorAPI(topic='fake-topic')
 
@@ -188,7 +229,7 @@ class RPCAPITestCase(base.DbTestCase):
     def test_change_node_power_state(self):
         self._test_rpcapi('change_node_power_state',
                           'call',
-                          version='1.6',
+                          version='1.39',
                           node_id=self.fake_node['uuid'],
                           new_state=states.POWER_ON)
 
@@ -249,8 +290,15 @@ class RPCAPITestCase(base.DbTestCase):
                           node_id=self.fake_node['uuid'],
                           enabled=True)
 
+    def test_create_port(self):
+        fake_port = db_utils.get_test_port()
+        self._test_rpcapi('create_port',
+                          'call',
+                          version='1.41',
+                          port_obj=fake_port)
+
     def test_update_port(self):
-        fake_port = dbutils.get_test_port()
+        fake_port = db_utils.get_test_port()
         self._test_rpcapi('update_port',
                           'call',
                           version='1.13',
@@ -276,6 +324,12 @@ class RPCAPITestCase(base.DbTestCase):
                           version='1.17',
                           node_id=self.fake_node['uuid'])
 
+    def test_inject_nmi(self):
+        self._test_rpcapi('inject_nmi',
+                          'call',
+                          version='1.40',
+                          node_id=self.fake_node['uuid'])
+
     def test_get_supported_boot_devices(self):
         self._test_rpcapi('get_supported_boot_devices',
                           'call',
@@ -306,6 +360,12 @@ class RPCAPITestCase(base.DbTestCase):
                           version='1.27',
                           node_id=self.fake_node['uuid'])
 
+    def test_continue_node_deploy(self):
+        self._test_rpcapi('continue_node_deploy',
+                          'cast',
+                          version='1.45',
+                          node_id=self.fake_node['uuid'])
+
     def test_get_raid_logical_disk_properties(self):
         self._test_rpcapi('get_raid_logical_disk_properties',
                           'call',
@@ -399,4 +459,116 @@ class RPCAPITestCase(base.DbTestCase):
                           'call',
                           node_id='fake-node',
                           callback_url='http://ramdisk.url:port',
-                          version='1.34')
+                          agent_version=None,
+                          version='1.42')
+
+    def test_destroy_volume_connector(self):
+        fake_volume_connector = db_utils.get_test_volume_connector()
+        self._test_rpcapi('destroy_volume_connector',
+                          'call',
+                          version='1.35',
+                          connector=fake_volume_connector)
+
+    def test_update_volume_connector(self):
+        fake_volume_connector = db_utils.get_test_volume_connector()
+        self._test_rpcapi('update_volume_connector',
+                          'call',
+                          version='1.35',
+                          connector=fake_volume_connector)
+
+    def test_create_node(self):
+        self._test_rpcapi('create_node',
+                          'call',
+                          version='1.36',
+                          node_obj=self.fake_node)
+
+    def test_destroy_volume_target(self):
+        fake_volume_target = db_utils.get_test_volume_target()
+        self._test_rpcapi('destroy_volume_target',
+                          'call',
+                          version='1.37',
+                          target=fake_volume_target)
+
+    def test_update_volume_target(self):
+        fake_volume_target = db_utils.get_test_volume_target()
+        self._test_rpcapi('update_volume_target',
+                          'call',
+                          version='1.37',
+                          target=fake_volume_target)
+
+    def test_vif_attach(self):
+        self._test_rpcapi('vif_attach',
+                          'call',
+                          node_id='fake-node',
+                          vif_info={"id": "vif"},
+                          version='1.38')
+
+    def test_vif_detach(self):
+        self._test_rpcapi('vif_detach',
+                          'call',
+                          node_id='fake-node',
+                          vif_id="vif",
+                          version='1.38')
+
+    def test_vif_list(self):
+        self._test_rpcapi('vif_list',
+                          'call',
+                          node_id='fake-node',
+                          version='1.38')
+
+    def test_do_node_rescue(self):
+        self._test_rpcapi('do_node_rescue',
+                          'call',
+                          version='1.43',
+                          node_id=self.fake_node['uuid'],
+                          rescue_password="password")
+
+    def test_do_node_unrescue(self):
+        self._test_rpcapi('do_node_unrescue',
+                          'call',
+                          version='1.43',
+                          node_id=self.fake_node['uuid'])
+
+    def _test_can_send_rescue(self, can_send):
+        rpcapi = conductor_rpcapi.ConductorAPI(topic='fake-topic')
+        with mock.patch.object(rpcapi.client,
+                               "can_send_version") as mock_can_send_version:
+            mock_can_send_version.return_value = can_send
+            result = rpcapi.can_send_rescue()
+            self.assertEqual(can_send, result)
+            mock_can_send_version.assert_called_once_with("1.43")
+
+    def test_can_send_rescue_true(self):
+        self._test_can_send_rescue(True)
+
+    def test_can_send_rescue_false(self):
+        self._test_can_send_rescue(False)
+
+    def test_add_node_traits(self):
+        self._test_rpcapi('add_node_traits',
+                          'call',
+                          node_id='fake-node',
+                          traits=['trait1'],
+                          version='1.44')
+
+    def test_add_node_traits_replace(self):
+        self._test_rpcapi('add_node_traits',
+                          'call',
+                          node_id='fake-node',
+                          traits=['trait1'],
+                          replace=True,
+                          version='1.44')
+
+    def test_remove_node_traits(self):
+        self._test_rpcapi('remove_node_traits',
+                          'call',
+                          node_id='fake-node',
+                          traits=['trait1'],
+                          version='1.44')
+
+    def test_remove_node_traits_all(self):
+        self._test_rpcapi('remove_node_traits',
+                          'call',
+                          node_id='fake-node',
+                          traits=None,
+                          version='1.44')
