@@ -25,10 +25,12 @@ from ironic.common import driver_factory
 from ironic.common import exception
 from ironic.common import fsm
 from ironic.common import states
+from ironic.conductor import notification_utils
 from ironic.conductor import task_manager
 from ironic import objects
+from ironic.objects import fields
 from ironic.tests import base as tests_base
-from ironic.tests.unit.db import base as tests_db_base
+from ironic.tests.unit.db import base as db_base
 from ironic.tests.unit.objects import utils as obj_utils
 
 
@@ -38,7 +40,9 @@ from ironic.tests.unit.objects import utils as obj_utils
 @mock.patch.object(driver_factory, 'build_driver_for_task')
 @mock.patch.object(objects.Port, 'list_by_node_id')
 @mock.patch.object(objects.Portgroup, 'list_by_node_id')
-class TaskManagerTestCase(tests_db_base.DbTestCase):
+@mock.patch.object(objects.VolumeConnector, 'list_by_node_id')
+@mock.patch.object(objects.VolumeTarget, 'list_by_node_id')
+class TaskManagerTestCase(db_base.DbTestCase):
     def setUp(self):
         super(TaskManagerTestCase, self).setUp()
         self.host = 'test-host'
@@ -48,7 +52,8 @@ class TaskManagerTestCase(tests_db_base.DbTestCase):
         self.node = obj_utils.create_test_node(self.context)
         self.future_mock = mock.Mock(spec=['cancel', 'add_done_callback'])
 
-    def test_excl_lock(self, get_portgroups_mock, get_ports_mock,
+    def test_excl_lock(self, get_voltgt_mock, get_volconn_mock,
+                       get_portgroups_mock, get_ports_mock,
                        build_driver_mock, reserve_mock, release_mock,
                        node_get_mock):
         reserve_mock.return_value = self.node
@@ -57,76 +62,71 @@ class TaskManagerTestCase(tests_db_base.DbTestCase):
             self.assertEqual(self.node, task.node)
             self.assertEqual(get_ports_mock.return_value, task.ports)
             self.assertEqual(get_portgroups_mock.return_value, task.portgroups)
+            self.assertEqual(get_volconn_mock.return_value,
+                             task.volume_connectors)
+            self.assertEqual(get_voltgt_mock.return_value, task.volume_targets)
             self.assertEqual(build_driver_mock.return_value, task.driver)
             self.assertFalse(task.shared)
-            build_driver_mock.assert_called_once_with(task, driver_name=None)
+            build_driver_mock.assert_called_once_with(task)
 
+        node_get_mock.assert_called_once_with(self.context, 'fake-node-id')
         reserve_mock.assert_called_once_with(self.context, self.host,
                                              'fake-node-id')
         get_ports_mock.assert_called_once_with(self.context, self.node.id)
         get_portgroups_mock.assert_called_once_with(self.context, self.node.id)
+        get_volconn_mock.assert_called_once_with(self.context, self.node.id)
+        get_voltgt_mock.assert_called_once_with(self.context, self.node.id)
         release_mock.assert_called_once_with(self.context, self.host,
                                              self.node.id)
-        self.assertFalse(node_get_mock.called)
-
-    def test_excl_lock_with_driver(
-            self, get_portgroups_mock, get_ports_mock, build_driver_mock,
-            reserve_mock, release_mock, node_get_mock):
-        reserve_mock.return_value = self.node
-        with task_manager.TaskManager(self.context, 'fake-node-id',
-                                      driver_name='fake-driver') as task:
-            self.assertEqual(self.context, task.context)
-            self.assertEqual(self.node, task.node)
-            self.assertEqual(get_ports_mock.return_value, task.ports)
-            self.assertEqual(get_portgroups_mock.return_value, task.portgroups)
-            self.assertEqual(build_driver_mock.return_value, task.driver)
-            self.assertFalse(task.shared)
-            build_driver_mock.assert_called_once_with(
-                task, driver_name='fake-driver')
-
-        reserve_mock.assert_called_once_with(self.context, self.host,
-                                             'fake-node-id')
-        get_ports_mock.assert_called_once_with(self.context, self.node.id)
-        get_portgroups_mock.assert_called_once_with(self.context, self.node.id)
-        release_mock.assert_called_once_with(self.context, self.host,
-                                             self.node.id)
-        self.assertFalse(node_get_mock.called)
 
     def test_excl_nested_acquire(
-            self, get_portgroups_mock, get_ports_mock, build_driver_mock,
+            self, get_voltgt_mock, get_volconn_mock, get_portgroups_mock,
+            get_ports_mock, build_driver_mock,
             reserve_mock, release_mock, node_get_mock):
         node2 = obj_utils.create_test_node(self.context,
                                            uuid=uuidutils.generate_uuid(),
-                                           driver='fake')
+                                           driver='fake-hardware')
 
         reserve_mock.return_value = self.node
         get_ports_mock.return_value = mock.sentinel.ports1
         get_portgroups_mock.return_value = mock.sentinel.portgroups1
+        get_volconn_mock.return_value = mock.sentinel.volconn1
+        get_voltgt_mock.return_value = mock.sentinel.voltgt1
         build_driver_mock.return_value = mock.sentinel.driver1
 
         with task_manager.TaskManager(self.context, 'node-id1') as task:
             reserve_mock.return_value = node2
             get_ports_mock.return_value = mock.sentinel.ports2
             get_portgroups_mock.return_value = mock.sentinel.portgroups2
+            get_volconn_mock.return_value = mock.sentinel.volconn2
+            get_voltgt_mock.return_value = mock.sentinel.voltgt2
             build_driver_mock.return_value = mock.sentinel.driver2
             with task_manager.TaskManager(self.context, 'node-id2') as task2:
                 self.assertEqual(self.context, task.context)
                 self.assertEqual(self.node, task.node)
                 self.assertEqual(mock.sentinel.ports1, task.ports)
                 self.assertEqual(mock.sentinel.portgroups1, task.portgroups)
+                self.assertEqual(mock.sentinel.volconn1,
+                                 task.volume_connectors)
+                self.assertEqual(mock.sentinel.voltgt1, task.volume_targets)
                 self.assertEqual(mock.sentinel.driver1, task.driver)
                 self.assertFalse(task.shared)
                 self.assertEqual(self.context, task2.context)
                 self.assertEqual(node2, task2.node)
                 self.assertEqual(mock.sentinel.ports2, task2.ports)
                 self.assertEqual(mock.sentinel.portgroups2, task2.portgroups)
+                self.assertEqual(mock.sentinel.volconn2,
+                                 task2.volume_connectors)
+                self.assertEqual(mock.sentinel.voltgt2, task2.volume_targets)
                 self.assertEqual(mock.sentinel.driver2, task2.driver)
                 self.assertFalse(task2.shared)
 
-                self.assertEqual([mock.call(task, driver_name=None),
-                                  mock.call(task2, driver_name=None)],
+                self.assertEqual([mock.call(task), mock.call(task2)],
                                  build_driver_mock.call_args_list)
 
+        self.assertEqual([mock.call(self.context, 'node-id1'),
+                          mock.call(self.context, 'node-id2')],
+                         node_get_mock.call_args_list)
         self.assertEqual([mock.call(self.context, self.host, 'node-id1'),
                           mock.call(self.context, self.host, 'node-id2')],
                          reserve_mock.call_args_list)
@@ -137,10 +137,10 @@ class TaskManagerTestCase(tests_db_base.DbTestCase):
         self.assertEqual([mock.call(self.context, self.host, node2.id),
                           mock.call(self.context, self.host, self.node.id)],
                          release_mock.call_args_list)
-        self.assertFalse(node_get_mock.called)
 
     def test_excl_lock_exception_then_lock(
-            self, get_portgroups_mock, get_ports_mock, build_driver_mock,
+            self, get_voltgt_mock, get_volconn_mock, get_portgroups_mock,
+            get_ports_mock, build_driver_mock,
             reserve_mock, release_mock, node_get_mock):
         retry_attempts = 3
         self.config(node_locked_retry_attempts=retry_attempts,
@@ -160,7 +160,8 @@ class TaskManagerTestCase(tests_db_base.DbTestCase):
         self.assertEqual(2, reserve_mock.call_count)
 
     def test_excl_lock_reserve_exception(
-            self, get_portgroups_mock, get_ports_mock, build_driver_mock,
+            self, get_voltgt_mock, get_volconn_mock, get_portgroups_mock,
+            get_ports_mock, build_driver_mock,
             reserve_mock, release_mock, node_get_mock):
         retry_attempts = 3
         self.config(node_locked_retry_attempts=retry_attempts,
@@ -172,18 +173,20 @@ class TaskManagerTestCase(tests_db_base.DbTestCase):
                           task_manager.TaskManager,
                           self.context,
                           'fake-node-id')
-
+        node_get_mock.assert_called_with(self.context, 'fake-node-id')
         reserve_mock.assert_called_with(self.context, self.host,
                                         'fake-node-id')
         self.assertEqual(retry_attempts, reserve_mock.call_count)
         self.assertFalse(get_ports_mock.called)
         self.assertFalse(get_portgroups_mock.called)
+        self.assertFalse(get_volconn_mock.called)
+        self.assertFalse(get_voltgt_mock.called)
         self.assertFalse(build_driver_mock.called)
         self.assertFalse(release_mock.called)
-        self.assertFalse(node_get_mock.called)
 
     def test_excl_lock_get_ports_exception(
-            self, get_portgroups_mock, get_ports_mock, build_driver_mock,
+            self, get_voltgt_mock, get_volconn_mock, get_portgroups_mock,
+            get_ports_mock, build_driver_mock,
             reserve_mock, release_mock, node_get_mock):
         reserve_mock.return_value = self.node
         get_ports_mock.side_effect = exception.IronicException('foo')
@@ -193,16 +196,17 @@ class TaskManagerTestCase(tests_db_base.DbTestCase):
                           self.context,
                           'fake-node-id')
 
+        node_get_mock.assert_called_once_with(self.context, 'fake-node-id')
         reserve_mock.assert_called_once_with(self.context, self.host,
                                              'fake-node-id')
         get_ports_mock.assert_called_once_with(self.context, self.node.id)
         self.assertFalse(build_driver_mock.called)
         release_mock.assert_called_once_with(self.context, self.host,
                                              self.node.id)
-        self.assertFalse(node_get_mock.called)
 
     def test_excl_lock_get_portgroups_exception(
-            self, get_portgroups_mock, get_ports_mock, build_driver_mock,
+            self, get_voltgt_mock, get_volconn_mock, get_portgroups_mock,
+            get_ports_mock, build_driver_mock,
             reserve_mock, release_mock, node_get_mock):
         reserve_mock.return_value = self.node
         get_portgroups_mock.side_effect = exception.IronicException('foo')
@@ -212,16 +216,57 @@ class TaskManagerTestCase(tests_db_base.DbTestCase):
                           self.context,
                           'fake-node-id')
 
+        node_get_mock.assert_called_once_with(self.context, 'fake-node-id')
         reserve_mock.assert_called_once_with(self.context, self.host,
                                              'fake-node-id')
         get_portgroups_mock.assert_called_once_with(self.context, self.node.id)
         self.assertFalse(build_driver_mock.called)
         release_mock.assert_called_once_with(self.context, self.host,
                                              self.node.id)
-        self.assertFalse(node_get_mock.called)
+
+    def test_excl_lock_get_volconn_exception(
+            self, get_voltgt_mock, get_volconn_mock, get_portgroups_mock,
+            get_ports_mock, build_driver_mock,
+            reserve_mock, release_mock, node_get_mock):
+        reserve_mock.return_value = self.node
+        get_volconn_mock.side_effect = exception.IronicException('foo')
+
+        self.assertRaises(exception.IronicException,
+                          task_manager.TaskManager,
+                          self.context,
+                          'fake-node-id')
+
+        reserve_mock.assert_called_once_with(self.context, self.host,
+                                             'fake-node-id')
+        get_volconn_mock.assert_called_once_with(self.context, self.node.id)
+        self.assertFalse(get_voltgt_mock.called)
+        release_mock.assert_called_once_with(self.context, self.host,
+                                             self.node.id)
+        node_get_mock.assert_called_once_with(self.context, 'fake-node-id')
+
+    def test_excl_lock_get_voltgt_exception(
+            self, get_voltgt_mock, get_volconn_mock, get_portgroups_mock,
+            get_ports_mock, build_driver_mock,
+            reserve_mock, release_mock, node_get_mock):
+        reserve_mock.return_value = self.node
+        get_voltgt_mock.side_effect = exception.IronicException('foo')
+
+        self.assertRaises(exception.IronicException,
+                          task_manager.TaskManager,
+                          self.context,
+                          'fake-node-id')
+
+        reserve_mock.assert_called_once_with(self.context, self.host,
+                                             'fake-node-id')
+        get_voltgt_mock.assert_called_once_with(self.context, self.node.id)
+        self.assertFalse(build_driver_mock.called)
+        release_mock.assert_called_once_with(self.context, self.host,
+                                             self.node.id)
+        node_get_mock.assert_called_once_with(self.context, 'fake-node-id')
 
     def test_excl_lock_build_driver_exception(
-            self, get_portgroups_mock, get_ports_mock, build_driver_mock,
+            self, get_voltgt_mock, get_volconn_mock, get_portgroups_mock,
+            get_ports_mock, build_driver_mock,
             reserve_mock, release_mock, node_get_mock):
         reserve_mock.return_value = self.node
         build_driver_mock.side_effect = (
@@ -232,17 +277,18 @@ class TaskManagerTestCase(tests_db_base.DbTestCase):
                           self.context,
                           'fake-node-id')
 
+        node_get_mock.assert_called_once_with(self.context, 'fake-node-id')
         reserve_mock.assert_called_once_with(self.context, self.host,
                                              'fake-node-id')
         get_ports_mock.assert_called_once_with(self.context, self.node.id)
         get_portgroups_mock.assert_called_once_with(self.context, self.node.id)
-        build_driver_mock.assert_called_once_with(mock.ANY, driver_name=None)
+        build_driver_mock.assert_called_once_with(mock.ANY)
         release_mock.assert_called_once_with(self.context, self.host,
                                              self.node.id)
-        self.assertFalse(node_get_mock.called)
 
     def test_shared_lock(
-            self, get_portgroups_mock, get_ports_mock, build_driver_mock,
+            self, get_voltgt_mock, get_volconn_mock, get_portgroups_mock,
+            get_ports_mock, build_driver_mock,
             reserve_mock, release_mock, node_get_mock):
         node_get_mock.return_value = self.node
         with task_manager.TaskManager(self.context, 'fake-node-id',
@@ -251,43 +297,25 @@ class TaskManagerTestCase(tests_db_base.DbTestCase):
             self.assertEqual(self.node, task.node)
             self.assertEqual(get_ports_mock.return_value, task.ports)
             self.assertEqual(get_portgroups_mock.return_value, task.portgroups)
+            self.assertEqual(get_volconn_mock.return_value,
+                             task.volume_connectors)
+            self.assertEqual(get_voltgt_mock.return_value, task.volume_targets)
             self.assertEqual(build_driver_mock.return_value, task.driver)
             self.assertTrue(task.shared)
 
-            build_driver_mock.assert_called_once_with(task, driver_name=None)
-
-        self.assertFalse(reserve_mock.called)
-        self.assertFalse(release_mock.called)
-        node_get_mock.assert_called_once_with(self.context, 'fake-node-id')
-        get_ports_mock.assert_called_once_with(self.context, self.node.id)
-        get_portgroups_mock.assert_called_once_with(self.context, self.node.id)
-
-    def test_shared_lock_with_driver(
-            self, get_portgroups_mock, get_ports_mock, build_driver_mock,
-            reserve_mock, release_mock, node_get_mock):
-        node_get_mock.return_value = self.node
-        with task_manager.TaskManager(self.context,
-                                      'fake-node-id',
-                                      shared=True,
-                                      driver_name='fake-driver') as task:
-            self.assertEqual(self.context, task.context)
-            self.assertEqual(self.node, task.node)
-            self.assertEqual(get_ports_mock.return_value, task.ports)
-            self.assertEqual(get_portgroups_mock.return_value, task.portgroups)
-            self.assertEqual(build_driver_mock.return_value, task.driver)
-            self.assertTrue(task.shared)
-
-            build_driver_mock.assert_called_once_with(
-                task, driver_name='fake-driver')
+            build_driver_mock.assert_called_once_with(task)
 
         self.assertFalse(reserve_mock.called)
         self.assertFalse(release_mock.called)
         node_get_mock.assert_called_once_with(self.context, 'fake-node-id')
         get_ports_mock.assert_called_once_with(self.context, self.node.id)
         get_portgroups_mock.assert_called_once_with(self.context, self.node.id)
+        get_volconn_mock.assert_called_once_with(self.context, self.node.id)
+        get_voltgt_mock.assert_called_once_with(self.context, self.node.id)
 
     def test_shared_lock_node_get_exception(
-            self, get_portgroups_mock, get_ports_mock, build_driver_mock,
+            self, get_voltgt_mock, get_volconn_mock, get_portgroups_mock,
+            get_ports_mock, build_driver_mock,
             reserve_mock, release_mock, node_get_mock):
         node_get_mock.side_effect = exception.NodeNotFound(node='foo')
 
@@ -302,10 +330,13 @@ class TaskManagerTestCase(tests_db_base.DbTestCase):
         node_get_mock.assert_called_once_with(self.context, 'fake-node-id')
         self.assertFalse(get_ports_mock.called)
         self.assertFalse(get_portgroups_mock.called)
+        self.assertFalse(get_volconn_mock.called)
+        self.assertFalse(get_voltgt_mock.called)
         self.assertFalse(build_driver_mock.called)
 
     def test_shared_lock_get_ports_exception(
-            self, get_portgroups_mock, get_ports_mock, build_driver_mock,
+            self, get_voltgt_mock, get_volconn_mock, get_portgroups_mock,
+            get_ports_mock, build_driver_mock,
             reserve_mock, release_mock, node_get_mock):
         node_get_mock.return_value = self.node
         get_ports_mock.side_effect = exception.IronicException('foo')
@@ -323,7 +354,8 @@ class TaskManagerTestCase(tests_db_base.DbTestCase):
         self.assertFalse(build_driver_mock.called)
 
     def test_shared_lock_get_portgroups_exception(
-            self, get_portgroups_mock, get_ports_mock, build_driver_mock,
+            self, get_voltgt_mock, get_volconn_mock, get_portgroups_mock,
+            get_ports_mock, build_driver_mock,
             reserve_mock, release_mock, node_get_mock):
         node_get_mock.return_value = self.node
         get_portgroups_mock.side_effect = exception.IronicException('foo')
@@ -340,8 +372,47 @@ class TaskManagerTestCase(tests_db_base.DbTestCase):
         get_portgroups_mock.assert_called_once_with(self.context, self.node.id)
         self.assertFalse(build_driver_mock.called)
 
+    def test_shared_lock_get_volconn_exception(
+            self, get_voltgt_mock, get_volconn_mock, get_portgroups_mock,
+            get_ports_mock, build_driver_mock,
+            reserve_mock, release_mock, node_get_mock):
+        node_get_mock.return_value = self.node
+        get_volconn_mock.side_effect = exception.IronicException('foo')
+
+        self.assertRaises(exception.IronicException,
+                          task_manager.TaskManager,
+                          self.context,
+                          'fake-node-id',
+                          shared=True)
+
+        self.assertFalse(reserve_mock.called)
+        self.assertFalse(release_mock.called)
+        node_get_mock.assert_called_once_with(self.context, 'fake-node-id')
+        get_volconn_mock.assert_called_once_with(self.context, self.node.id)
+        self.assertFalse(get_voltgt_mock.called)
+
+    def test_shared_lock_get_voltgt_exception(
+            self, get_voltgt_mock, get_volconn_mock, get_portgroups_mock,
+            get_ports_mock, build_driver_mock,
+            reserve_mock, release_mock, node_get_mock):
+        node_get_mock.return_value = self.node
+        get_voltgt_mock.side_effect = exception.IronicException('foo')
+
+        self.assertRaises(exception.IronicException,
+                          task_manager.TaskManager,
+                          self.context,
+                          'fake-node-id',
+                          shared=True)
+
+        self.assertFalse(reserve_mock.called)
+        self.assertFalse(release_mock.called)
+        node_get_mock.assert_called_once_with(self.context, 'fake-node-id')
+        get_voltgt_mock.assert_called_once_with(self.context, self.node.id)
+        self.assertFalse(build_driver_mock.called)
+
     def test_shared_lock_build_driver_exception(
-            self, get_portgroups_mock, get_ports_mock, build_driver_mock,
+            self, get_voltgt_mock, get_volconn_mock, get_portgroups_mock,
+            get_ports_mock, build_driver_mock,
             reserve_mock, release_mock, node_get_mock):
         node_get_mock.return_value = self.node
         build_driver_mock.side_effect = (
@@ -358,31 +429,38 @@ class TaskManagerTestCase(tests_db_base.DbTestCase):
         node_get_mock.assert_called_once_with(self.context, 'fake-node-id')
         get_ports_mock.assert_called_once_with(self.context, self.node.id)
         get_portgroups_mock.assert_called_once_with(self.context, self.node.id)
-        build_driver_mock.assert_called_once_with(mock.ANY, driver_name=None)
+        get_volconn_mock.assert_called_once_with(self.context, self.node.id)
+        get_voltgt_mock.assert_called_once_with(self.context, self.node.id)
+        build_driver_mock.assert_called_once_with(mock.ANY)
 
     def test_upgrade_lock(
-            self, get_portgroups_mock, get_ports_mock, build_driver_mock,
+            self, get_voltgt_mock, get_volconn_mock, get_portgroups_mock,
+            get_ports_mock, build_driver_mock,
             reserve_mock, release_mock, node_get_mock):
         node_get_mock.return_value = self.node
         reserve_mock.return_value = self.node
         with task_manager.TaskManager(self.context, 'fake-node-id',
-                                      shared=True) as task:
+                                      shared=True, purpose='ham') as task:
             self.assertEqual(self.context, task.context)
             self.assertEqual(self.node, task.node)
             self.assertEqual(get_ports_mock.return_value, task.ports)
             self.assertEqual(get_portgroups_mock.return_value, task.portgroups)
+            self.assertEqual(get_volconn_mock.return_value,
+                             task.volume_connectors)
+            self.assertEqual(get_voltgt_mock.return_value, task.volume_targets)
             self.assertEqual(build_driver_mock.return_value, task.driver)
             self.assertTrue(task.shared)
             self.assertFalse(reserve_mock.called)
 
             task.upgrade_lock()
             self.assertFalse(task.shared)
-            # second upgrade does nothing
-            task.upgrade_lock()
+            self.assertEqual('ham', task._purpose)
+            # second upgrade does nothing except changes the purpose
+            task.upgrade_lock(purpose='spam')
             self.assertFalse(task.shared)
+            self.assertEqual('spam', task._purpose)
 
-            build_driver_mock.assert_called_once_with(mock.ANY,
-                                                      driver_name=None)
+            build_driver_mock.assert_called_once_with(mock.ANY)
 
         # make sure reserve() was called only once
         reserve_mock.assert_called_once_with(self.context, self.host,
@@ -392,9 +470,37 @@ class TaskManagerTestCase(tests_db_base.DbTestCase):
         node_get_mock.assert_called_once_with(self.context, 'fake-node-id')
         get_ports_mock.assert_called_once_with(self.context, self.node.id)
         get_portgroups_mock.assert_called_once_with(self.context, self.node.id)
+        get_volconn_mock.assert_called_once_with(self.context, self.node.id)
+        get_voltgt_mock.assert_called_once_with(self.context, self.node.id)
 
+    def test_upgrade_lock_refreshes_fsm(self, get_voltgt_mock,
+                                        get_volconn_mock, get_portgroups_mock,
+                                        get_ports_mock, build_driver_mock,
+                                        reserve_mock, release_mock,
+                                        node_get_mock):
+        reserve_mock.return_value = self.node
+        node_get_mock.return_value = self.node
+        with task_manager.acquire(self.context, 'fake-node-id',
+                                  shared=True) as task1:
+            self.assertEqual(states.AVAILABLE, task1.node.provision_state)
+
+            with task_manager.acquire(self.context, 'fake-node-id',
+                                      shared=False) as task2:
+                # move the node to manageable
+                task2.process_event('manage')
+                self.assertEqual(states.MANAGEABLE, task1.node.provision_state)
+
+            # now upgrade our shared task and try to go to cleaning
+            # this will explode if task1's FSM doesn't get refreshed
+            task1.upgrade_lock()
+            task1.process_event('provide')
+            self.assertEqual(states.CLEANING, task1.node.provision_state)
+
+    @mock.patch.object(task_manager.TaskManager,
+                       '_notify_provision_state_change', autospec=True)
     def test_spawn_after(
-            self, get_portgroups_mock, get_ports_mock, build_driver_mock,
+            self, notify_mock, get_voltgt_mock, get_volconn_mock,
+            get_portgroups_mock, get_ports_mock, build_driver_mock,
             reserve_mock, release_mock, node_get_mock):
         spawn_mock = mock.Mock(return_value=self.future_mock)
         task_release_mock = mock.Mock()
@@ -412,9 +518,11 @@ class TaskManagerTestCase(tests_db_base.DbTestCase):
         # release resources pending the finishing of the background
         # thread
         self.assertFalse(task_release_mock.called)
+        notify_mock.assert_called_once_with(task)
 
     def test_spawn_after_exception_while_yielded(
-            self, get_portgroups_mock, get_ports_mock, build_driver_mock,
+            self, get_voltgt_mock, get_volconn_mock, get_portgroups_mock,
+            get_ports_mock, build_driver_mock,
             reserve_mock, release_mock, node_get_mock):
         spawn_mock = mock.Mock()
         task_release_mock = mock.Mock()
@@ -430,8 +538,11 @@ class TaskManagerTestCase(tests_db_base.DbTestCase):
         self.assertFalse(spawn_mock.called)
         task_release_mock.assert_called_once_with()
 
+    @mock.patch.object(task_manager.TaskManager,
+                       '_notify_provision_state_change', autospec=True)
     def test_spawn_after_spawn_fails(
-            self, get_portgroups_mock, get_ports_mock, build_driver_mock,
+            self, notify_mock, get_voltgt_mock, get_volconn_mock,
+            get_portgroups_mock, get_ports_mock, build_driver_mock,
             reserve_mock, release_mock, node_get_mock):
         spawn_mock = mock.Mock(side_effect=exception.IronicException('foo'))
         task_release_mock = mock.Mock()
@@ -446,9 +557,11 @@ class TaskManagerTestCase(tests_db_base.DbTestCase):
 
         spawn_mock.assert_called_once_with(1, 2, foo='bar', cat='meow')
         task_release_mock.assert_called_once_with()
+        self.assertFalse(notify_mock.called)
 
     def test_spawn_after_link_fails(
-            self, get_portgroups_mock, get_ports_mock, build_driver_mock,
+            self, get_voltgt_mock, get_volconn_mock, get_portgroups_mock,
+            get_ports_mock, build_driver_mock,
             reserve_mock, release_mock, node_get_mock):
         self.future_mock.add_done_callback.side_effect = (
             exception.IronicException('foo'))
@@ -471,7 +584,8 @@ class TaskManagerTestCase(tests_db_base.DbTestCase):
         task_release_mock.assert_called_once_with()
 
     def test_spawn_after_on_error_hook(
-            self, get_portgroups_mock, get_ports_mock, build_driver_mock,
+            self, get_voltgt_mock, get_volconn_mock, get_portgroups_mock,
+            get_ports_mock, build_driver_mock,
             reserve_mock, release_mock, node_get_mock):
         expected_exception = exception.IronicException('foo')
         spawn_mock = mock.Mock(side_effect=expected_exception)
@@ -493,7 +607,8 @@ class TaskManagerTestCase(tests_db_base.DbTestCase):
                                                  'fake-argument')
 
     def test_spawn_after_on_error_hook_exception(
-            self, get_portgroups_mock, get_ports_mock, build_driver_mock,
+            self, get_voltgt_mock, get_volconn_mock, get_portgroups_mock,
+            get_ports_mock, build_driver_mock,
             reserve_mock, release_mock, node_get_mock):
         expected_exception = exception.IronicException('foo')
         spawn_mock = mock.Mock(side_effect=expected_exception)
@@ -519,7 +634,8 @@ class TaskManagerTestCase(tests_db_base.DbTestCase):
 
     @mock.patch.object(states.machine, 'copy')
     def test_init_prepares_fsm(
-            self, copy_mock, get_portgroups_mock, get_ports_mock,
+            self, copy_mock, get_volconn_mock, get_voltgt_mock,
+            get_portgroups_mock, get_ports_mock,
             build_driver_mock, reserve_mock, release_mock, node_get_mock):
         m = mock.Mock(spec=fsm.FSM)
         reserve_mock.return_value = self.node
@@ -547,15 +663,20 @@ class TaskManagerStateModelTestCases(tests_base.TestCase):
         t.driver = mock.Mock()
         t.ports = mock.Mock()
         t.portgroups = mock.Mock()
+        t.volume_connectors = mock.Mock()
+        t.volume_targets = mock.Mock()
         t.shared = True
         t._purpose = 'purpose'
         t._debug_timer = mock.Mock()
+        t._debug_timer.elapsed.return_value = 3.14
 
         t.release_resources(t)
         self.assertIsNone(t.node)
         self.assertIsNone(t.driver)
         self.assertIsNone(t.ports)
         self.assertIsNone(t.portgroups)
+        self.assertIsNone(t.volume_connectors)
+        self.assertIsNone(t.volume_targets)
         self.assertIsNone(t.fsm)
 
     def test_process_event_fsm_raises(self):
@@ -647,6 +768,11 @@ class TaskManagerStateModelTestCases(tests_base.TestCase):
             self.assertEqual(states.NOSTATE,
                              self.task.node.target_provision_state)
 
+    def test_process_event_no_callback_notify(self):
+        self.task.process_event = task_manager.TaskManager.process_event
+        self.task.process_event(self.task, 'fake')
+        self.task._notify_provision_state_change.assert_called_once_with()
+
 
 @task_manager.require_exclusive_lock
 def _req_excl_lock_method(*args, **kwargs):
@@ -737,3 +863,98 @@ class ThreadExceptionTestCase(tests_base.TestCase):
         self.future_mock.exception.assert_called_once_with()
         self.assertIsNone(self.node.last_error)
         self.assertTrue(log_mock.called)
+
+
+@mock.patch.object(notification_utils, 'emit_provision_set_notification',
+                   autospec=True)
+class ProvisionNotifyTestCase(tests_base.TestCase):
+    def setUp(self):
+        super(ProvisionNotifyTestCase, self).setUp()
+        self.node = mock.Mock(spec=objects.Node)
+        self.task = mock.Mock(spec=task_manager.TaskManager)
+        self.task.node = self.node
+        notifier = task_manager.TaskManager._notify_provision_state_change
+        self.task.notifier = notifier
+        self.task._prev_target_provision_state = 'oldtarget'
+        self.task._event = 'event'
+
+    def test_notify_no_state_change(self, emit_mock):
+        self.task._event = None
+        self.task.notifier(self.task)
+        self.assertFalse(emit_mock.called)
+
+    def test_notify_error_state(self, emit_mock):
+        self.task._event = 'fail'
+        self.task._prev_provision_state = 'fake'
+        self.task.notifier(self.task)
+        emit_mock.assert_called_once_with(self.task,
+                                          fields.NotificationLevel.ERROR,
+                                          fields.NotificationStatus.ERROR,
+                                          'fake', 'oldtarget', 'fail')
+        self.assertIsNone(self.task._event)
+
+    def test_notify_unstable_to_unstable(self, emit_mock):
+        self.node.provision_state = states.DEPLOYING
+        self.task._prev_provision_state = states.DEPLOYWAIT
+        self.task.notifier(self.task)
+        emit_mock.assert_called_once_with(self.task,
+                                          fields.NotificationLevel.INFO,
+                                          fields.NotificationStatus.SUCCESS,
+                                          states.DEPLOYWAIT,
+                                          'oldtarget', 'event')
+
+    def test_notify_stable_to_unstable(self, emit_mock):
+        self.node.provision_state = states.DEPLOYING
+        self.task._prev_provision_state = states.AVAILABLE
+        self.task.notifier(self.task)
+        emit_mock.assert_called_once_with(self.task,
+                                          fields.NotificationLevel.INFO,
+                                          fields.NotificationStatus.START,
+                                          states.AVAILABLE,
+                                          'oldtarget', 'event')
+
+    def test_notify_unstable_to_stable(self, emit_mock):
+        self.node.provision_state = states.ACTIVE
+        self.task._prev_provision_state = states.DEPLOYING
+        self.task.notifier(self.task)
+        emit_mock.assert_called_once_with(self.task,
+                                          fields.NotificationLevel.INFO,
+                                          fields.NotificationStatus.END,
+                                          states.DEPLOYING,
+                                          'oldtarget', 'event')
+
+    def test_notify_stable_to_stable(self, emit_mock):
+        self.node.provision_state = states.MANAGEABLE
+        self.task._prev_provision_state = states.AVAILABLE
+        self.task.notifier(self.task)
+        emit_mock.assert_called_once_with(self.task,
+                                          fields.NotificationLevel.INFO,
+                                          fields.NotificationStatus.SUCCESS,
+                                          states.AVAILABLE,
+                                          'oldtarget', 'event')
+
+    def test_notify_resource_released(self, emit_mock):
+        node = mock.Mock(spec=objects.Node)
+        node.provision_state = states.DEPLOYING
+        node.target_provision_state = states.ACTIVE
+        task = mock.Mock(spec=task_manager.TaskManager)
+        task._prev_provision_state = states.AVAILABLE
+        task._prev_target_provision_state = states.NOSTATE
+        task._event = 'event'
+        task.node = None
+        task._saved_node = node
+        notifier = task_manager.TaskManager._notify_provision_state_change
+        task.notifier = notifier
+        task.notifier(task)
+        task_arg = emit_mock.call_args[0][0]
+        self.assertEqual(node, task_arg.node)
+        self.assertIsNot(task, task_arg)
+
+    def test_notify_only_once(self, emit_mock):
+        self.node.provision_state = states.DEPLOYING
+        self.task._prev_provision_state = states.AVAILABLE
+        self.task.notifier(self.task)
+        self.assertIsNone(self.task._event)
+        self.task.notifier(self.task)
+        self.assertEqual(1, emit_mock.call_count)
+        self.assertIsNone(self.task._event)
