@@ -38,61 +38,34 @@ import collections
 import contextlib
 
 from alembic import script
+import fixtures
 import mock
 from oslo_db import exception as db_exc
 from oslo_db.sqlalchemy import enginefacade
-from oslo_db.sqlalchemy import test_base
+from oslo_db.sqlalchemy import test_fixtures
 from oslo_db.sqlalchemy import test_migrations
 from oslo_db.sqlalchemy import utils as db_utils
 from oslo_log import log as logging
 from oslo_utils import uuidutils
+from oslotest import base as test_base
 import sqlalchemy
 import sqlalchemy.exc
 
-from ironic.common.i18n import _LE
+from ironic.conf import CONF
 from ironic.db.sqlalchemy import migration
 from ironic.db.sqlalchemy import models
 from ironic.tests import base
 
 LOG = logging.getLogger(__name__)
 
-
-def _get_connect_string(backend, user, passwd, database):
-    """Get database connection
-
-    Try to get a connection with a very specific set of values, if we get
-    these then we'll run the tests, otherwise they are skipped
-    """
-    if backend == "postgres":
-        backend = "postgresql+psycopg2"
-    elif backend == "mysql":
-        backend = "mysql+mysqldb"
-    else:
-        raise Exception("Unrecognized backend: '%s'" % backend)
-
-    return ("%(backend)s://%(user)s:%(passwd)s@localhost/%(database)s"
-            % {'backend': backend, 'user': user, 'passwd': passwd,
-               'database': database})
-
-
-def _is_backend_avail(backend, user, passwd, database):
-    try:
-        connect_uri = _get_connect_string(backend, user, passwd, database)
-        engine = sqlalchemy.create_engine(connect_uri)
-        connection = engine.connect()
-    except Exception:
-        # intentionally catch all to handle exceptions even if we don't
-        # have any backend code loaded.
-        return False
-    else:
-        connection.close()
-        engine.dispose()
-        return True
+# NOTE(vdrok): This was introduced after migration tests started taking more
+# time in gate. Timeout value in seconds for tests performing migrations.
+MIGRATIONS_TIMEOUT = 300
 
 
 @contextlib.contextmanager
 def patch_with_engine(engine):
-    with mock.patch.object(enginefacade.get_legacy_facade(),
+    with mock.patch.object(enginefacade.writer,
                            'get_engine') as patch_engine:
         patch_engine.return_value = engine
         yield
@@ -142,8 +115,8 @@ class WalkVersionsMixin(object):
                 if check:
                     check(engine, data)
         except Exception:
-            LOG.error(_LE("Failed to migrate to version %(version)s on engine "
-                          "%(engine)s"),
+            LOG.error("Failed to migrate to version %(version)s on engine "
+                      "%(engine)s",
                       {'version': version, 'engine': engine})
             raise
 
@@ -210,22 +183,15 @@ class MigrationCheckersMixin(object):
 
     def setUp(self):
         super(MigrationCheckersMixin, self).setUp()
+        self.engine = enginefacade.writer.get_engine()
         self.config = migration._alembic_config()
         self.migration_api = migration
+        self.useFixture(fixtures.Timeout(MIGRATIONS_TIMEOUT,
+                                         gentle=True))
 
     def test_walk_versions(self):
         self._walk_versions(self.engine, self.config)
 
-    def test_connect_fail(self):
-        """Test that we can trigger a database connection failure
-
-        Test that we can fail gracefully to ensure we don't break people
-        without specific database backend
-        """
-        if _is_backend_avail(self.FIXTURE.DRIVER, "openstack_cifail",
-                             self.FIXTURE.USERNAME, self.FIXTURE.DBNAME):
-            self.fail("Shouldn't have connected")
-
     def _check_21b331f883ef(self, engine, data):
         nodes = db_utils.get_table(engine, 'nodes')
         col_names = [column.name for column in nodes.c]
@@ -278,8 +244,8 @@ class MigrationCheckersMixin(object):
         data_conductor = {'hostname': 'test_host'}
         conductors.insert().execute(data_conductor)
         conductor = conductors.select(
-            conductors.c.hostname ==
-            data_conductor['hostname']).execute().first()
+            conductors.c.hostname
+            == data_conductor['hostname']).execute().first()
 
         data_node = {'uuid': uuidutils.generate_uuid(),
                      'conductor_affinity': conductor['id']}
@@ -530,6 +496,270 @@ class MigrationCheckersMixin(object):
                               (sqlalchemy.types.Boolean,
                                sqlalchemy.types.Integer))
 
+    def _check_bcdd431ba0bf(self, engine, data):
+        nodes = db_utils.get_table(engine, 'nodes')
+        col_names = [column.name for column in nodes.c]
+        added_ifaces = ['boot', 'console', 'deploy', 'inspect',
+                        'management', 'power', 'raid', 'vendor']
+        for iface in added_ifaces:
+            name = '%s_interface' % iface
+            self.assertIn(name, col_names)
+            self.assertIsInstance(getattr(nodes.c, name).type,
+                                  sqlalchemy.types.String)
+
+    def _check_daa1ba02d98(self, engine, data):
+        connectors = db_utils.get_table(engine, 'volume_connectors')
+        col_names = [column.name for column in connectors.c]
+        expected_names = ['created_at', 'updated_at', 'id', 'uuid', 'node_id',
+                          'type', 'connector_id', 'extra']
+        self.assertEqual(sorted(expected_names), sorted(col_names))
+
+        self.assertIsInstance(connectors.c.created_at.type,
+                              sqlalchemy.types.DateTime)
+        self.assertIsInstance(connectors.c.updated_at.type,
+                              sqlalchemy.types.DateTime)
+        self.assertIsInstance(connectors.c.id.type,
+                              sqlalchemy.types.Integer)
+        self.assertIsInstance(connectors.c.uuid.type,
+                              sqlalchemy.types.String)
+        self.assertIsInstance(connectors.c.node_id.type,
+                              sqlalchemy.types.Integer)
+        self.assertIsInstance(connectors.c.type.type,
+                              sqlalchemy.types.String)
+        self.assertIsInstance(connectors.c.connector_id.type,
+                              sqlalchemy.types.String)
+        self.assertIsInstance(connectors.c.extra.type,
+                              sqlalchemy.types.TEXT)
+
+    def _check_1a59178ebdf6(self, engine, data):
+        targets = db_utils.get_table(engine, 'volume_targets')
+        col_names = [column.name for column in targets.c]
+        expected_names = ['created_at', 'updated_at', 'id', 'uuid', 'node_id',
+                          'boot_index', 'extra', 'properties', 'volume_type',
+                          'volume_id']
+        self.assertEqual(sorted(expected_names), sorted(col_names))
+
+        self.assertIsInstance(targets.c.created_at.type,
+                              sqlalchemy.types.DateTime)
+        self.assertIsInstance(targets.c.updated_at.type,
+                              sqlalchemy.types.DateTime)
+        self.assertIsInstance(targets.c.id.type,
+                              sqlalchemy.types.Integer)
+        self.assertIsInstance(targets.c.uuid.type,
+                              sqlalchemy.types.String)
+        self.assertIsInstance(targets.c.node_id.type,
+                              sqlalchemy.types.Integer)
+        self.assertIsInstance(targets.c.boot_index.type,
+                              sqlalchemy.types.Integer)
+        self.assertIsInstance(targets.c.extra.type,
+                              sqlalchemy.types.TEXT)
+        self.assertIsInstance(targets.c.properties.type,
+                              sqlalchemy.types.TEXT)
+        self.assertIsInstance(targets.c.volume_type.type,
+                              sqlalchemy.types.String)
+        self.assertIsInstance(targets.c.volume_id.type,
+                              sqlalchemy.types.String)
+
+    def _pre_upgrade_493d8f27f235(self, engine):
+        portgroups = db_utils.get_table(engine, 'portgroups')
+        data = [{'uuid': uuidutils.generate_uuid()},
+                {'uuid': uuidutils.generate_uuid()}]
+        portgroups.insert().values(data).execute()
+        return data
+
+    def _check_493d8f27f235(self, engine, data):
+        portgroups = db_utils.get_table(engine, 'portgroups')
+        col_names = [column.name for column in portgroups.c]
+        self.assertIn('properties', col_names)
+        self.assertIsInstance(portgroups.c.properties.type,
+                              sqlalchemy.types.TEXT)
+        self.assertIn('mode', col_names)
+        self.assertIsInstance(portgroups.c.mode.type,
+                              sqlalchemy.types.String)
+
+        result = engine.execute(portgroups.select())
+        for row in result:
+            self.assertEqual(CONF.default_portgroup_mode, row['mode'])
+
+    def _check_1d6951876d68(self, engine, data):
+        nodes = db_utils.get_table(engine, 'nodes')
+        col_names = [column.name for column in nodes.c]
+        self.assertIn('storage_interface', col_names)
+        self.assertIsInstance(nodes.c.storage_interface.type,
+                              sqlalchemy.types.String)
+
+    def _check_2353895ecfae(self, engine, data):
+        ifaces = db_utils.get_table(engine, 'conductor_hardware_interfaces')
+        col_names = [column.name for column in ifaces.c]
+        expected_names = ['created_at', 'updated_at', 'id', 'conductor_id',
+                          'hardware_type', 'interface_type', 'interface_name']
+        self.assertEqual(sorted(expected_names), sorted(col_names))
+
+        self.assertIsInstance(ifaces.c.created_at.type,
+                              sqlalchemy.types.DateTime)
+        self.assertIsInstance(ifaces.c.updated_at.type,
+                              sqlalchemy.types.DateTime)
+        self.assertIsInstance(ifaces.c.id.type,
+                              sqlalchemy.types.Integer)
+        self.assertIsInstance(ifaces.c.conductor_id.type,
+                              sqlalchemy.types.Integer)
+        self.assertIsInstance(ifaces.c.hardware_type.type,
+                              sqlalchemy.types.String)
+        self.assertIsInstance(ifaces.c.interface_type.type,
+                              sqlalchemy.types.String)
+        self.assertIsInstance(ifaces.c.interface_name.type,
+                              sqlalchemy.types.String)
+
+    def _check_dbefd6bdaa2c(self, engine, data):
+        ifaces = db_utils.get_table(engine, 'conductor_hardware_interfaces')
+        col_names = [column.name for column in ifaces.c]
+        self.assertIn('default', col_names)
+        self.assertIsInstance(ifaces.c.default.type,
+                              (sqlalchemy.types.Boolean,
+                               sqlalchemy.types.Integer))
+
+    def _check_3d86a077a3f2(self, engine, data):
+        ports = db_utils.get_table(engine, 'ports')
+        col_names = [column.name for column in ports.c]
+        self.assertIn('physical_network', col_names)
+        self.assertIsInstance(ports.c.physical_network.type,
+                              sqlalchemy.types.String)
+
+    def _check_868cb606a74a(self, engine, data):
+        for table in ['chassis', 'conductors', 'node_tags', 'nodes',
+                      'portgroups', 'ports', 'volume_connectors',
+                      'volume_targets', 'conductor_hardware_interfaces']:
+            table = db_utils.get_table(engine, table)
+            col_names = [column.name for column in table.c]
+            self.assertIn('version', col_names)
+            self.assertIsInstance(table.c.version.type,
+                                  sqlalchemy.types.String)
+
+    def _check_405cfe08f18d(self, engine, data):
+        nodes = db_utils.get_table(engine, 'nodes')
+        col_names = [column.name for column in nodes.c]
+        self.assertIn('rescue_interface', col_names)
+        self.assertIsInstance(nodes.c.rescue_interface.type,
+                              sqlalchemy.types.String)
+
+    def _pre_upgrade_b4130a7fc904(self, engine):
+        # Create a node to which traits can be added.
+        data = {'uuid': uuidutils.generate_uuid()}
+        nodes = db_utils.get_table(engine, 'nodes')
+        nodes.insert().execute(data)
+        node = nodes.select(nodes.c.uuid == data['uuid']).execute().first()
+        data['id'] = node['id']
+        return data
+
+    def _check_b4130a7fc904(self, engine, data):
+        node_traits = db_utils.get_table(engine, 'node_traits')
+        col_names = [column.name for column in node_traits.c]
+        self.assertIn('node_id', col_names)
+        self.assertIsInstance(node_traits.c.node_id.type,
+                              sqlalchemy.types.Integer)
+        self.assertIn('trait', col_names)
+        self.assertIsInstance(node_traits.c.trait.type,
+                              sqlalchemy.types.String)
+
+        trait = {'node_id': data['id'], 'trait': 'trait1'}
+        node_traits.insert().execute(trait)
+        trait = node_traits.select(
+            node_traits.c.node_id == data['id']).execute().first()
+        self.assertEqual('trait1', trait['trait'])
+
+    def _pre_upgrade_82c315d60161(self, engine):
+        # Create a node to which bios setting can be added.
+        data = {'uuid': uuidutils.generate_uuid()}
+        nodes = db_utils.get_table(engine, 'nodes')
+        nodes.insert().execute(data)
+        node = nodes.select(nodes.c.uuid == data['uuid']).execute().first()
+        data['id'] = node['id']
+        return data
+
+    def _check_82c315d60161(self, engine, data):
+        bios_settings = db_utils.get_table(engine, 'bios_settings')
+        col_names = [column.name for column in bios_settings.c]
+        expected_names = ['node_id', 'created_at', 'updated_at',
+                          'name', 'value', 'version']
+        self.assertEqual(sorted(expected_names), sorted(col_names))
+        self.assertIsInstance(bios_settings.c.node_id.type,
+                              sqlalchemy.types.Integer)
+        self.assertIsInstance(bios_settings.c.created_at.type,
+                              sqlalchemy.types.DateTime)
+        self.assertIsInstance(bios_settings.c.updated_at.type,
+                              sqlalchemy.types.DateTime)
+        self.assertIsInstance(bios_settings.c.name.type,
+                              sqlalchemy.types.String)
+        self.assertIsInstance(bios_settings.c.version.type,
+                              sqlalchemy.types.String)
+        self.assertIsInstance(bios_settings.c.value.type,
+                              sqlalchemy.types.Text)
+
+        setting = {'node_id': data['id'],
+                   'name': 'virtualization',
+                   'value': 'on'}
+        bios_settings.insert().execute(setting)
+        setting = bios_settings.select(
+            sqlalchemy.sql.and_(
+                bios_settings.c.node_id == data['id'],
+                bios_settings.c.name == setting['name'])).execute().first()
+        self.assertEqual('on', setting['value'])
+
+    def _check_2d13bc3d6bba(self, engine, data):
+        nodes = db_utils.get_table(engine, 'nodes')
+        col_names = [column.name for column in nodes.c]
+        self.assertIn('bios_interface', col_names)
+        self.assertIsInstance(nodes.c.bios_interface.type,
+                              sqlalchemy.types.String)
+
+    def _check_fb3f10dd262e(self, engine, data):
+        nodes_tbl = db_utils.get_table(engine, 'nodes')
+        col_names = [column.name for column in nodes_tbl.c]
+        self.assertIn('fault', col_names)
+        self.assertIsInstance(nodes_tbl.c.fault.type,
+                              sqlalchemy.types.String)
+
+    def _check_b9117ac17882(self, engine, data):
+        nodes = db_utils.get_table(engine, 'nodes')
+        col_names = [column.name for column in nodes.c]
+        self.assertIn('deploy_step', col_names)
+        self.assertIsInstance(nodes.c.deploy_step.type,
+                              sqlalchemy.types.String)
+
+    def _pre_upgrade_664f85c2f622(self, engine):
+        # Create a node and a conductor to verify existing records
+        # get a conductor_group of ""
+        data = {
+            'conductor_id': 98765432,
+            'node_uuid': uuidutils.generate_uuid(),
+        }
+
+        nodes = db_utils.get_table(engine, 'nodes')
+        nodes.insert().execute({'uuid': data['node_uuid']})
+
+        conductors = db_utils.get_table(engine, 'conductors')
+        conductors.insert().execute({'id': data['conductor_id'],
+                                     'hostname': uuidutils.generate_uuid()})
+
+        return data
+
+    def _check_664f85c2f622(self, engine, data):
+        nodes_tbl = db_utils.get_table(engine, 'nodes')
+        conductors_tbl = db_utils.get_table(engine, 'conductors')
+        for tbl in (nodes_tbl, conductors_tbl):
+            col_names = [column.name for column in tbl.c]
+            self.assertIn('conductor_group', col_names)
+            self.assertIsInstance(tbl.c.conductor_group.type,
+                                  sqlalchemy.types.String)
+
+        node = nodes_tbl.select(
+            nodes_tbl.c.uuid == data['node_uuid']).execute().first()
+        self.assertEqual(node['conductor_group'], "")
+
+        conductor = conductors_tbl.select(
+            conductors_tbl.c.id == data['conductor_id']).execute().first()
+        self.assertEqual(conductor['conductor_group'], "")
+
     def test_upgrade_and_version(self):
         with patch_with_engine(self.engine):
             self.migration_api.upgrade('head')
@@ -543,7 +773,7 @@ class MigrationCheckersMixin(object):
     def test_upgrade_and_create_schema(self):
         with patch_with_engine(self.engine):
             self.migration_api.upgrade('31baaf680d2b')
-            self.assertRaises(db_exc.DbMigrationError,
+            self.assertRaises(db_exc.DBMigrationError,
                               self.migration_api.create_schema)
 
     def test_upgrade_twice(self):
@@ -557,18 +787,64 @@ class MigrationCheckersMixin(object):
 
 class TestMigrationsMySQL(MigrationCheckersMixin,
                           WalkVersionsMixin,
-                          test_base.MySQLOpportunisticTestCase):
-    pass
+                          test_fixtures.OpportunisticDBTestMixin,
+                          test_base.BaseTestCase):
+    FIXTURE = test_fixtures.MySQLOpportunisticFixture
+
+    def _pre_upgrade_e918ff30eb42(self, engine):
+
+        nodes = db_utils.get_table(engine, 'nodes')
+
+        # this should always fail pre-upgrade
+        mediumtext = 'a' * (pow(2, 16) + 1)
+        uuid = uuidutils.generate_uuid()
+        expected_to_fail_data = {'uuid': uuid, 'instance_info': mediumtext}
+        self.assertRaises(db_exc.DBError,
+                          nodes.insert().execute, expected_to_fail_data)
+
+        # this should always work pre-upgrade
+        text = 'a' * (pow(2, 16) - 1)
+        uuid = uuidutils.generate_uuid()
+        valid_pre_upgrade_data = {'uuid': uuid, 'instance_info': text}
+        nodes.insert().execute(valid_pre_upgrade_data)
+
+        return valid_pre_upgrade_data
+
+    def _check_e918ff30eb42(self, engine, data):
+
+        nodes = db_utils.get_table(engine, 'nodes')
+
+        # check that the data for the successful pre-upgrade
+        # entry didn't change
+        node = nodes.select(nodes.c.uuid == data['uuid']).execute().first()
+        self.assertIsNotNone(node)
+        self.assertEqual(data['instance_info'], node['instance_info'])
+
+        # now this should pass post-upgrade
+        test = 'b' * (pow(2, 16) + 1)
+        uuid = uuidutils.generate_uuid()
+        data = {'uuid': uuid, 'instance_info': test}
+        nodes.insert().execute(data)
+
+        node = nodes.select(nodes.c.uuid == uuid).execute().first()
+        self.assertEqual(test, node['instance_info'])
 
 
 class TestMigrationsPostgreSQL(MigrationCheckersMixin,
                                WalkVersionsMixin,
-                               test_base.PostgreSQLOpportunisticTestCase):
-    pass
+                               test_fixtures.OpportunisticDBTestMixin,
+                               test_base.BaseTestCase):
+    FIXTURE = test_fixtures.PostgresqlOpportunisticFixture
 
 
 class ModelsMigrationSyncMixin(object):
 
+    def setUp(self):
+        super(ModelsMigrationSyncMixin, self).setUp()
+        self.engine = enginefacade.writer.get_engine()
+        self.useFixture(fixtures.Timeout(MIGRATIONS_TIMEOUT,
+                                         gentle=True))
+
     def get_metadata(self):
         return models.Base.metadata
 
@@ -582,11 +858,13 @@ class ModelsMigrationSyncMixin(object):
 
 class ModelsMigrationsSyncMysql(ModelsMigrationSyncMixin,
                                 test_migrations.ModelsMigrationsSync,
-                                test_base.MySQLOpportunisticTestCase):
-    pass
+                                test_fixtures.OpportunisticDBTestMixin,
+                                test_base.BaseTestCase):
+    FIXTURE = test_fixtures.MySQLOpportunisticFixture
 
 
 class ModelsMigrationsSyncPostgres(ModelsMigrationSyncMixin,
                                    test_migrations.ModelsMigrationsSync,
-                                   test_base.PostgreSQLOpportunisticTestCase):
-    pass
+                                   test_fixtures.OpportunisticDBTestMixin,
+                                   test_base.BaseTestCase):
+    FIXTURE = test_fixtures.PostgresqlOpportunisticFixture
