@@ -31,13 +31,20 @@ from ironic.tests.unit.db import utils
 class DbNodeTestCase(base.DbTestCase):
 
     def test_create_node(self):
-        utils.create_test_node()
+        node = utils.create_test_node()
+        self.assertEqual([], node.tags)
+        self.assertEqual([], node.traits)
 
     def test_create_node_with_tags(self):
         self.assertRaises(exception.InvalidParameterValue,
                           utils.create_test_node,
                           tags=['tag1', 'tag2'])
 
+    def test_create_node_with_traits(self):
+        self.assertRaises(exception.InvalidParameterValue,
+                          utils.create_test_node,
+                          traits=['trait1', 'trait2'])
+
     def test_create_node_already_exists(self):
         utils.create_test_node()
         self.assertRaises(exception.NodeAlreadyExists,
@@ -61,27 +68,39 @@ class DbNodeTestCase(base.DbTestCase):
     def test_get_node_by_id(self):
         node = utils.create_test_node()
         self.dbapi.set_node_tags(node.id, ['tag1', 'tag2'])
+        utils.create_test_node_traits(node_id=node.id,
+                                      traits=['trait1', 'trait2'])
         res = self.dbapi.get_node_by_id(node.id)
         self.assertEqual(node.id, res.id)
         self.assertEqual(node.uuid, res.uuid)
         self.assertItemsEqual(['tag1', 'tag2'], [tag.tag for tag in res.tags])
+        self.assertItemsEqual(['trait1', 'trait2'],
+                              [trait.trait for trait in res.traits])
 
     def test_get_node_by_uuid(self):
         node = utils.create_test_node()
         self.dbapi.set_node_tags(node.id, ['tag1', 'tag2'])
+        utils.create_test_node_traits(node_id=node.id,
+                                      traits=['trait1', 'trait2'])
         res = self.dbapi.get_node_by_uuid(node.uuid)
         self.assertEqual(node.id, res.id)
         self.assertEqual(node.uuid, res.uuid)
         self.assertItemsEqual(['tag1', 'tag2'], [tag.tag for tag in res.tags])
+        self.assertItemsEqual(['trait1', 'trait2'],
+                              [trait.trait for trait in res.traits])
 
     def test_get_node_by_name(self):
         node = utils.create_test_node()
         self.dbapi.set_node_tags(node.id, ['tag1', 'tag2'])
+        utils.create_test_node_traits(node_id=node.id,
+                                      traits=['trait1', 'trait2'])
         res = self.dbapi.get_node_by_name(node.name)
         self.assertEqual(node.id, res.id)
         self.assertEqual(node.uuid, res.uuid)
         self.assertEqual(node.name, res.name)
         self.assertItemsEqual(['tag1', 'tag2'], [tag.tag for tag in res.tags])
+        self.assertItemsEqual(['trait1', 'trait2'],
+                              [trait.trait for trait in res.traits])
 
     def test_get_node_that_does_not_exist(self):
         self.assertRaises(exception.NodeNotFound,
@@ -124,7 +143,9 @@ class DbNodeTestCase(base.DbTestCase):
             driver='driver-two',
             uuid=uuidutils.generate_uuid(),
             maintenance=True,
-            resource_class='foo')
+            fault='boom',
+            resource_class='foo',
+            conductor_group='group1')
         node3 = utils.create_test_node(
             driver='driver-one',
             uuid=uuidutils.generate_uuid(),
@@ -151,22 +172,56 @@ class DbNodeTestCase(base.DbTestCase):
         res = self.dbapi.get_nodeinfo_list(filters={'reserved': False})
         self.assertEqual([node2.id], [r[0] for r in res])
 
-        res = self.dbapi.get_node_list(filters={'maintenance': True})
+        res = self.dbapi.get_nodeinfo_list(filters={'maintenance': True})
         self.assertEqual([node2.id], [r.id for r in res])
 
-        res = self.dbapi.get_node_list(filters={'maintenance': False})
+        res = self.dbapi.get_nodeinfo_list(filters={'maintenance': False})
         self.assertEqual(sorted([node1.id, node3.id]),
                          sorted([r.id for r in res]))
 
-        res = self.dbapi.get_node_list(filters={'resource_class': 'foo'})
+        res = self.dbapi.get_nodeinfo_list(filters={'fault': 'boom'})
         self.assertEqual([node2.id], [r.id for r in res])
 
-        res = self.dbapi.get_node_list(
+        res = self.dbapi.get_nodeinfo_list(filters={'fault': 'moob'})
+        self.assertEqual([], [r.id for r in res])
+
+        res = self.dbapi.get_nodeinfo_list(filters={'resource_class': 'foo'})
+        self.assertEqual([node2.id], [r.id for r in res])
+
+        res = self.dbapi.get_nodeinfo_list(
+            filters={'conductor_group': 'group1'})
+        self.assertEqual([node2.id], [r.id for r in res])
+
+        res = self.dbapi.get_nodeinfo_list(
+            filters={'conductor_group': 'group2'})
+        self.assertEqual([], [r.id for r in res])
+
+        res = self.dbapi.get_nodeinfo_list(
             filters={'reserved_by_any_of': ['fake-host',
                                             'another-fake-host']})
         self.assertEqual(sorted([node1.id, node3.id]),
                          sorted([r.id for r in res]))
 
+        res = self.dbapi.get_nodeinfo_list(filters={'id': node1.id})
+        self.assertEqual([node1.id], [r.id for r in res])
+
+        res = self.dbapi.get_nodeinfo_list(filters={'uuid': node1.uuid})
+        self.assertEqual([node1.id], [r.id for r in res])
+
+        # ensure unknown filters explode
+        filters = {'bad_filter': 'foo'}
+        self.assertRaisesRegex(ValueError,
+                               'bad_filter',
+                               self.dbapi.get_nodeinfo_list,
+                               filters=filters)
+
+        # even with good filters present
+        filters = {'bad_filter': 'foo', 'id': node1.id}
+        self.assertRaisesRegex(ValueError,
+                               'bad_filter',
+                               self.dbapi.get_nodeinfo_list,
+                               filters=filters)
+
     @mock.patch.object(timeutils, 'utcnow', autospec=True)
     def test_get_nodeinfo_list_provision(self, mock_utcnow):
         past = datetime.datetime(2000, 1, 1, 0, 0)
@@ -228,6 +283,7 @@ class DbNodeTestCase(base.DbTestCase):
         six.assertCountEqual(self, uuids, res_uuids)
         for r in res:
             self.assertEqual([], r.tags)
+            self.assertEqual([], r.traits)
 
     def test_get_node_list_with_filters(self):
         ch1 = utils.create_test_chassis(uuid=uuidutils.generate_uuid())
@@ -243,7 +299,10 @@ class DbNodeTestCase(base.DbTestCase):
             driver='driver-two',
             uuid=uuidutils.generate_uuid(),
             chassis_id=ch2['id'],
-            maintenance=True)
+            maintenance=True,
+            fault='boom',
+            resource_class='foo',
+            conductor_group='group1')
 
         res = self.dbapi.get_node_list(filters={'chassis_uuid': ch1['uuid']})
         self.assertEqual([node1.id], [r.id for r in res])
@@ -275,6 +334,41 @@ class DbNodeTestCase(base.DbTestCase):
         res = self.dbapi.get_node_list(filters={'maintenance': False})
         self.assertEqual([node1.id], [r.id for r in res])
 
+        res = self.dbapi.get_nodeinfo_list(filters={'fault': 'boom'})
+        self.assertEqual([node2.id], [r.id for r in res])
+
+        res = self.dbapi.get_nodeinfo_list(filters={'fault': 'moob'})
+        self.assertEqual([], [r.id for r in res])
+
+        res = self.dbapi.get_node_list(filters={'resource_class': 'foo'})
+        self.assertEqual([node2.id], [r.id for r in res])
+
+        res = self.dbapi.get_node_list(filters={'conductor_group': 'group1'})
+        self.assertEqual([node2.id], [r.id for r in res])
+
+        res = self.dbapi.get_node_list(filters={'conductor_group': 'group2'})
+        self.assertEqual([], [r.id for r in res])
+
+        res = self.dbapi.get_node_list(filters={'id': node1.id})
+        self.assertEqual([node1.id], [r.id for r in res])
+
+        res = self.dbapi.get_node_list(filters={'uuid': node1.uuid})
+        self.assertEqual([node1.id], [r.id for r in res])
+
+        # ensure unknown filters explode
+        filters = {'bad_filter': 'foo'}
+        self.assertRaisesRegex(ValueError,
+                               'bad_filter',
+                               self.dbapi.get_node_list,
+                               filters=filters)
+
+        # even with good filters present
+        filters = {'bad_filter': 'foo', 'id': node1.id}
+        self.assertRaisesRegex(ValueError,
+                               'bad_filter',
+                               self.dbapi.get_node_list,
+                               filters=filters)
+
     def test_get_node_list_chassis_not_found(self):
         self.assertRaises(exception.ChassisNotFound,
                           self.dbapi.get_node_list,
@@ -284,10 +378,14 @@ class DbNodeTestCase(base.DbTestCase):
         node = utils.create_test_node(
             instance_uuid='12345678-9999-0000-aaaa-123456789012')
         self.dbapi.set_node_tags(node.id, ['tag1', 'tag2'])
+        utils.create_test_node_traits(node_id=node.id,
+                                      traits=['trait1', 'trait2'])
 
         res = self.dbapi.get_node_by_instance(node.instance_uuid)
         self.assertEqual(node.uuid, res.uuid)
         self.assertItemsEqual(['tag1', 'tag2'], [tag.tag for tag in res.tags])
+        self.assertItemsEqual(['trait1', 'trait2'],
+                              [trait.trait for trait in res.traits])
 
     def test_get_node_by_instance_wrong_uuid(self):
         utils.create_test_node(
@@ -348,7 +446,8 @@ class DbNodeTestCase(base.DbTestCase):
 
         self.assertTrue(self.dbapi.node_tag_exists(node.id, tag.tag))
         self.dbapi.destroy_node(node.id)
-        self.assertFalse(self.dbapi.node_tag_exists(node.id, tag.tag))
+        self.assertRaises(exception.NodeNotFound,
+                          self.dbapi.node_tag_exists, node.id, tag.tag)
 
     def test_tags_get_destroyed_after_destroying_a_node_by_uuid(self):
         node = utils.create_test_node()
@@ -357,7 +456,68 @@ class DbNodeTestCase(base.DbTestCase):
 
         self.assertTrue(self.dbapi.node_tag_exists(node.id, tag.tag))
         self.dbapi.destroy_node(node.uuid)
-        self.assertFalse(self.dbapi.node_tag_exists(node.id, tag.tag))
+        self.assertRaises(exception.NodeNotFound,
+                          self.dbapi.node_tag_exists, node.id, tag.tag)
+
+    def test_volume_connector_get_destroyed_after_destroying_a_node(self):
+        node = utils.create_test_node()
+
+        connector = utils.create_test_volume_connector(node_id=node.id)
+
+        self.dbapi.destroy_node(node.id)
+
+        self.assertRaises(exception.VolumeConnectorNotFound,
+                          self.dbapi.get_volume_connector_by_id, connector.id)
+
+    def test_volume_connector_get_destroyed_after_destroying_a_node_uuid(self):
+        node = utils.create_test_node()
+
+        connector = utils.create_test_volume_connector(node_id=node.id)
+
+        self.dbapi.destroy_node(node.uuid)
+
+        self.assertRaises(exception.VolumeConnectorNotFound,
+                          self.dbapi.get_volume_connector_by_id, connector.id)
+
+    def test_volume_target_gets_destroyed_after_destroying_a_node(self):
+        node = utils.create_test_node()
+
+        target = utils.create_test_volume_target(node_id=node.id)
+
+        self.dbapi.destroy_node(node.id)
+
+        self.assertRaises(exception.VolumeTargetNotFound,
+                          self.dbapi.get_volume_target_by_id, target.id)
+
+    def test_volume_target_gets_destroyed_after_destroying_a_node_uuid(self):
+        node = utils.create_test_node()
+
+        target = utils.create_test_volume_target(node_id=node.id)
+
+        self.dbapi.destroy_node(node.uuid)
+
+        self.assertRaises(exception.VolumeTargetNotFound,
+                          self.dbapi.get_volume_target_by_id, target.id)
+
+    def test_traits_get_destroyed_after_destroying_a_node(self):
+        node = utils.create_test_node()
+
+        trait = utils.create_test_node_trait(node_id=node.id)
+
+        self.assertTrue(self.dbapi.node_trait_exists(node.id, trait.trait))
+        self.dbapi.destroy_node(node.id)
+        self.assertRaises(exception.NodeNotFound,
+                          self.dbapi.node_trait_exists, node.id, trait.trait)
+
+    def test_traits_get_destroyed_after_destroying_a_node_by_uuid(self):
+        node = utils.create_test_node()
+
+        trait = utils.create_test_node_trait(node_id=node.id)
+
+        self.assertTrue(self.dbapi.node_trait_exists(node.id, trait.trait))
+        self.dbapi.destroy_node(node.uuid)
+        self.assertRaises(exception.NodeNotFound,
+                          self.dbapi.node_trait_exists, node.id, trait.trait)
 
     def test_update_node(self):
         node = utils.create_test_node()
@@ -368,6 +528,30 @@ class DbNodeTestCase(base.DbTestCase):
 
         res = self.dbapi.update_node(node.id, {'extra': new_extra})
         self.assertEqual(new_extra, res.extra)
+        self.assertEqual([], res.tags)
+        self.assertEqual([], res.traits)
+
+    def test_update_node_with_tags(self):
+        node = utils.create_test_node()
+        tag = utils.create_test_node_tag(node_id=node.id)
+
+        old_extra = node.extra
+        new_extra = {'foo': 'bar'}
+        self.assertNotEqual(old_extra, new_extra)
+
+        res = self.dbapi.update_node(node.id, {'extra': new_extra})
+        self.assertEqual([tag.tag], [t.tag for t in res.tags])
+
+    def test_update_node_with_traits(self):
+        node = utils.create_test_node()
+        trait = utils.create_test_node_trait(node_id=node.id)
+
+        old_extra = node.extra
+        new_extra = {'foo': 'bar'}
+        self.assertNotEqual(old_extra, new_extra)
+
+        res = self.dbapi.update_node(node.id, {'extra': new_extra})
+        self.assertEqual([trait.trait], [t.trait for t in res.traits])
 
     def test_update_node_not_found(self):
         node_uuid = uuidutils.generate_uuid()
@@ -389,16 +573,6 @@ class DbNodeTestCase(base.DbTestCase):
         res = self.dbapi.update_node(node.id, {'instance_uuid': None})
         self.assertIsNone(res.instance_uuid)
 
-    def test_update_node_already_associated(self):
-        node = utils.create_test_node()
-        new_i_uuid_one = uuidutils.generate_uuid()
-        self.dbapi.update_node(node.id, {'instance_uuid': new_i_uuid_one})
-        new_i_uuid_two = uuidutils.generate_uuid()
-        self.assertRaises(exception.NodeAssociated,
-                          self.dbapi.update_node,
-                          node.id,
-                          {'instance_uuid': new_i_uuid_two})
-
     def test_update_node_instance_already_associated(self):
         node1 = utils.create_test_node(uuid=uuidutils.generate_uuid())
         new_i_uuid = uuidutils.generate_uuid()
@@ -460,6 +634,8 @@ class DbNodeTestCase(base.DbTestCase):
     def test_reserve_node(self):
         node = utils.create_test_node()
         self.dbapi.set_node_tags(node.id, ['tag1', 'tag2'])
+        utils.create_test_node_traits(node_id=node.id,
+                                      traits=['trait1', 'trait2'])
         uuid = node.uuid
 
         r1 = 'fake-reservation'
@@ -467,6 +643,8 @@ class DbNodeTestCase(base.DbTestCase):
         # reserve the node
         res = self.dbapi.reserve_node(r1, uuid)
         self.assertItemsEqual(['tag1', 'tag2'], [tag.tag for tag in res.tags])
+        self.assertItemsEqual(['trait1', 'trait2'],
+                              [trait.trait for trait in res.traits])
 
         # check reservation
         res = self.dbapi.get_node_by_uuid(uuid)
@@ -523,10 +701,9 @@ class DbNodeTestCase(base.DbTestCase):
 
         r = 'fake-reservation'
         self.dbapi.reserve_node(r, uuid)
-        try:
-            self.dbapi.reserve_node('another', uuid)
-        except exception.NodeLocked as e:
-            self.assertIn(r, str(e))
+        exc = self.assertRaises(exception.NodeLocked, self.dbapi.reserve_node,
+                                'another', uuid)
+        self.assertIn(r, str(exc))
 
     def test_reservation_non_existent_node(self):
         node = utils.create_test_node()
@@ -594,6 +771,7 @@ class DbNodeTestCase(base.DbTestCase):
 
         res = self.dbapi.get_node_by_port_addresses(addresses)
         self.assertEqual(node.uuid, res.uuid)
+        self.assertEqual([], res.traits)
 
     def test_get_node_by_port_addresses_not_found(self):
         node = utils.create_test_node(
@@ -603,10 +781,10 @@ class DbNodeTestCase(base.DbTestCase):
                                node_id=node.id,
                                address='aa:bb:cc:dd:ee:ff')
 
-        self.assertRaisesRegexp(exception.NodeNotFound,
-                                'was not found',
-                                self.dbapi.get_node_by_port_addresses,
-                                ['11:22:33:44:55:66'])
+        self.assertRaisesRegex(exception.NodeNotFound,
+                               'was not found',
+                               self.dbapi.get_node_by_port_addresses,
+                               ['11:22:33:44:55:66'])
 
     def test_get_node_by_port_addresses_multiple_found(self):
         node1 = utils.create_test_node(
@@ -623,7 +801,7 @@ class DbNodeTestCase(base.DbTestCase):
                                node_id=node2.id,
                                address=addresses[1])
 
-        self.assertRaisesRegexp(exception.NodeNotFound,
-                                'Multiple nodes',
-                                self.dbapi.get_node_by_port_addresses,
-                                addresses)
+        self.assertRaisesRegex(exception.NodeNotFound,
+                               'Multiple nodes',
+                               self.dbapi.get_node_by_port_addresses,
+                               addresses)
