@@ -15,9 +15,7 @@
 #    under the License.
 
 import mock
-
 from neutronclient.common import exceptions as neutron_client_exc
-from neutronclient.v2_0 import client
 from oslo_utils import uuidutils
 
 from ironic.common import dhcp_factory
@@ -25,8 +23,6 @@ from ironic.common import exception
 from ironic.common import pxe_utils
 from ironic.conductor import task_manager
 from ironic.dhcp import neutron
-from ironic.drivers.modules import ssh
-from ironic.tests.unit.conductor import mgr_utils
 from ironic.tests.unit.db import base as db_base
 from ironic.tests.unit.objects import utils as object_utils
 
@@ -35,24 +31,11 @@ class TestNeutron(db_base.DbTestCase):
 
     def setUp(self):
         super(TestNeutron, self).setUp()
-        mgr_utils.mock_the_extension_manager(driver='fake')
         self.config(
-            cleaning_network_uuid='00000000-0000-0000-0000-000000000000',
+            cleaning_network='00000000-0000-0000-0000-000000000000',
             group='neutron')
-        self.config(enabled_drivers=['fake'])
         self.config(dhcp_provider='neutron',
                     group='dhcp')
-        self.config(url='test-url',
-                    url_timeout=30,
-                    retries=2,
-                    group='neutron')
-        self.config(insecure=False,
-                    certfile='test-file',
-                    admin_user='test-admin-user',
-                    admin_tenant_name='test-admin-tenant',
-                    admin_password='test-admin-password',
-                    auth_uri='test-auth-uri',
-                    group='keystone_authtoken')
         self.node = object_utils.create_test_node(self.context)
         self.ports = [
             object_utils.create_test_port(
@@ -65,9 +48,8 @@ class TestNeutron(db_base.DbTestCase):
 
         dhcp_factory.DHCPFactory._dhcp_provider = None
 
-    @mock.patch.object(client.Client, 'update_port')
-    @mock.patch.object(client.Client, "__init__")
-    def test_update_port_dhcp_opts(self, mock_client_init, mock_update_port):
+    @mock.patch('ironic.common.neutron.get_client', autospec=True)
+    def test_update_port_dhcp_opts(self, client_mock):
         opts = [{'opt_name': 'bootfile-name',
                  'opt_value': 'pxelinux.0'},
                 {'opt_name': 'tftp-server',
@@ -77,98 +59,31 @@ class TestNeutron(db_base.DbTestCase):
         port_id = 'fake-port-id'
         expected = {'port': {'extra_dhcp_opts': opts}}
 
-        mock_client_init.return_value = None
         api = dhcp_factory.DHCPFactory()
-        api.provider.update_port_dhcp_opts(port_id, opts)
-        mock_update_port.assert_called_once_with(port_id, expected)
+        with task_manager.acquire(self.context, self.node.uuid) as task:
+            api.provider.update_port_dhcp_opts(port_id, opts,
+                                               context=task.context)
+        client_mock.return_value.update_port.assert_called_once_with(
+            port_id, expected)
 
-    @mock.patch.object(client.Client, 'update_port')
-    @mock.patch.object(client.Client, "__init__")
-    def test_update_port_dhcp_opts_with_exception(self, mock_client_init,
-                                                  mock_update_port):
+    @mock.patch('ironic.common.neutron.get_client', autospec=True)
+    def test_update_port_dhcp_opts_with_exception(self, client_mock):
         opts = [{}]
         port_id = 'fake-port-id'
-        mock_client_init.return_value = None
-        mock_update_port.side_effect = (
+        client_mock.return_value.update_port.side_effect = (
             neutron_client_exc.NeutronClientException())
 
         api = dhcp_factory.DHCPFactory()
-        self.assertRaises(
-            exception.FailedToUpdateDHCPOptOnPort,
-            api.provider.update_port_dhcp_opts,
-            port_id, opts)
+        with task_manager.acquire(self.context, self.node.uuid) as task:
+            self.assertRaises(
+                exception.FailedToUpdateDHCPOptOnPort,
+                api.provider.update_port_dhcp_opts,
+                port_id, opts, context=task.context)
 
-    @mock.patch.object(client.Client, 'show_port')
-    @mock.patch.object(client.Client, 'update_port')
-    @mock.patch.object(client.Client, '__init__')
-    def test_update_port_address(self, mock_client_init, mock_update_port,
-                                 mock_show_port):
-        address = 'fe:54:00:77:07:d9'
-        port_id = 'fake-port-id'
-        expected = {'port': {'mac_address': address}}
-        mock_client_init.return_value = None
-        mock_show_port.return_value = {}
-
-        api = dhcp_factory.DHCPFactory()
-        api.provider.update_port_address(port_id, address)
-        mock_update_port.assert_called_once_with(port_id, expected)
-
-    @mock.patch.object(client.Client, 'show_port')
-    @mock.patch.object(client.Client, 'update_port')
-    @mock.patch.object(client.Client, '__init__')
-    def test_update_port_address_with_binding(self, mock_client_init,
-                                              mock_update_port,
-                                              mock_show_port):
-        address = 'fe:54:00:77:07:d9'
-        port_id = 'fake-port-id'
-        expected = {'port': {'mac_address': address,
-                             'binding:host_id': 'host'}}
-        mock_client_init.return_value = None
-        mock_show_port.return_value = {'port': {'binding:host_id': 'host'}}
-
-        api = dhcp_factory.DHCPFactory()
-        api.provider.update_port_address(port_id, address)
-        mock_update_port.assert_any_call(port_id,
-                                         {'port': {'binding:host_id': ''}})
-        mock_update_port.assert_any_call(port_id, expected)
-
-    @mock.patch.object(client.Client, 'show_port')
-    @mock.patch.object(client.Client, 'update_port')
-    @mock.patch.object(client.Client, '__init__')
-    def test_update_port_address_show_failed(self, mock_client_init,
-                                             mock_update_port,
-                                             mock_show_port):
-        address = 'fe:54:00:77:07:d9'
-        port_id = 'fake-port-id'
-        mock_client_init.return_value = None
-        mock_show_port.side_effect = (
-            neutron_client_exc.NeutronClientException())
-
-        api = dhcp_factory.DHCPFactory()
-        self.assertRaises(exception.FailedToUpdateMacOnPort,
-                          api.provider.update_port_address, port_id, address)
-        self.assertFalse(mock_update_port.called)
-
-    @mock.patch.object(client.Client, 'show_port')
-    @mock.patch.object(client.Client, 'update_port')
-    @mock.patch.object(client.Client, '__init__')
-    def test_update_port_address_with_exception(self, mock_client_init,
-                                                mock_update_port,
-                                                mock_show_port):
-        address = 'fe:54:00:77:07:d9'
-        port_id = 'fake-port-id'
-        mock_client_init.return_value = None
-        mock_show_port.return_value = {}
-
-        api = dhcp_factory.DHCPFactory()
-        mock_update_port.side_effect = (
-            neutron_client_exc.NeutronClientException())
-        self.assertRaises(exception.FailedToUpdateMacOnPort,
-                          api.provider.update_port_address,
-                          port_id, address)
-
-    @mock.patch('ironic.dhcp.neutron.NeutronDHCPApi.update_port_dhcp_opts')
-    @mock.patch('ironic.common.network.get_node_vif_ids')
+    @mock.patch('ironic.dhcp.neutron.NeutronDHCPApi.update_port_dhcp_opts',
+                autospec=True)
+    @mock.patch('ironic.common.network.get_node_vif_ids',
+                autospec=True)
     def test_update_dhcp(self, mock_gnvi, mock_updo):
         mock_gnvi.return_value = {'ports': {'port-uuid': 'vif-uuid'},
                                   'portgroups': {}}
@@ -177,11 +92,13 @@ class TestNeutron(db_base.DbTestCase):
             opts = pxe_utils.dhcp_options_for_instance(task)
             api = dhcp_factory.DHCPFactory()
             api.update_dhcp(task, opts)
-        mock_updo.assert_called_once_with('vif-uuid', opts,
-                                          token=self.context.auth_token)
+            mock_updo.assert_called_once_with(mock.ANY, 'vif-uuid', opts,
+                                              context=task.context)
 
-    @mock.patch('ironic.dhcp.neutron.NeutronDHCPApi.update_port_dhcp_opts')
-    @mock.patch('ironic.common.network.get_node_vif_ids')
+    @mock.patch('ironic.dhcp.neutron.NeutronDHCPApi.update_port_dhcp_opts',
+                autospec=True)
+    @mock.patch('ironic.common.network.get_node_vif_ids',
+                autospec=True)
     def test_update_dhcp_no_vif_data(self, mock_gnvi, mock_updo):
         mock_gnvi.return_value = {'portgroups': {}, 'ports': {}}
         with task_manager.acquire(self.context,
@@ -191,8 +108,10 @@ class TestNeutron(db_base.DbTestCase):
                               api.update_dhcp, task, self.node)
         self.assertFalse(mock_updo.called)
 
-    @mock.patch('ironic.dhcp.neutron.NeutronDHCPApi.update_port_dhcp_opts')
-    @mock.patch('ironic.common.network.get_node_vif_ids')
+    @mock.patch('ironic.dhcp.neutron.NeutronDHCPApi.update_port_dhcp_opts',
+                autospec=True)
+    @mock.patch('ironic.common.network.get_node_vif_ids',
+                autospec=True)
     def test_update_dhcp_some_failures(self, mock_gnvi, mock_updo):
         # confirm update is called twice, one fails, but no exception raised
         mock_gnvi.return_value = {'ports': {'p1': 'v1', 'p2': 'v2'},
@@ -206,8 +125,9 @@ class TestNeutron(db_base.DbTestCase):
             mock_gnvi.assert_called_once_with(task)
         self.assertEqual(2, mock_updo.call_count)
 
-    @mock.patch('ironic.dhcp.neutron.NeutronDHCPApi.update_port_dhcp_opts')
-    @mock.patch('ironic.common.network.get_node_vif_ids')
+    @mock.patch('ironic.dhcp.neutron.NeutronDHCPApi.update_port_dhcp_opts',
+                autospec=True)
+    @mock.patch('ironic.common.network.get_node_vif_ids', autospec=True)
     def test_update_dhcp_fails(self, mock_gnvi, mock_updo):
         # confirm update is called twice, both fail, and exception is raised
         mock_gnvi.return_value = {'ports': {'p1': 'v1', 'p2': 'v2'},
@@ -223,53 +143,10 @@ class TestNeutron(db_base.DbTestCase):
             mock_gnvi.assert_called_once_with(task)
         self.assertEqual(2, mock_updo.call_count)
 
-    @mock.patch('time.sleep', autospec=True)
-    @mock.patch.object(neutron.NeutronDHCPApi, 'update_port_dhcp_opts',
-                       autospec=True)
-    @mock.patch('ironic.common.network.get_node_vif_ids', autospec=True)
-    def test_update_dhcp_set_sleep_and_ssh(self, mock_gnvi, mock_updo,
-                                           mock_ts):
-        mock_gnvi.return_value = {'ports': {'port-uuid': 'vif-uuid'},
-                                  'portgroups': {}}
-        self.config(port_setup_delay=30, group='neutron')
-        with task_manager.acquire(self.context,
-                                  self.node.uuid) as task:
-            task.driver.power = ssh.SSHPower()
-            opts = pxe_utils.dhcp_options_for_instance(task)
-            api = dhcp_factory.DHCPFactory()
-            api.update_dhcp(task, opts)
-            mock_ts.assert_called_with(30)
-        mock_updo.assert_called_once_with(mock.ANY, 'vif-uuid', opts,
-                                          token=self.context.auth_token)
-
     @mock.patch.object(neutron, 'LOG', autospec=True)
     @mock.patch('time.sleep', autospec=True)
-    @mock.patch.object(neutron.NeutronDHCPApi, 'update_port_dhcp_opts',
-                       autospec=True)
     @mock.patch('ironic.common.network.get_node_vif_ids', autospec=True)
-    def test_update_dhcp_unset_sleep_and_ssh(self, mock_gnvi, mock_updo,
-                                             mock_ts, mock_log):
-        mock_gnvi.return_value = {'ports': {'port-uuid': 'vif-uuid'},
-                                  'portgroups': {}}
-        with task_manager.acquire(self.context,
-                                  self.node.uuid) as task:
-            opts = pxe_utils.dhcp_options_for_instance(task)
-            task.driver.power = ssh.SSHPower()
-            api = dhcp_factory.DHCPFactory()
-            api.update_dhcp(task, opts)
-            self.assertTrue(mock_log.warning.called)
-            self.assertIn('Setting the port delay to 15 for SSH',
-                          mock_log.warning.call_args[0][0])
-            mock_ts.assert_called_with(15)
-        mock_updo.assert_called_once_with(mock.ANY, 'vif-uuid', opts,
-                                          token=self.context.auth_token)
-
-    @mock.patch.object(neutron, 'LOG', autospec=True)
-    @mock.patch('time.sleep', autospec=True)
-    @mock.patch.object(neutron.NeutronDHCPApi, 'update_port_dhcp_opts',
-                       autospec=True)
-    @mock.patch('ironic.common.network.get_node_vif_ids', autospec=True)
-    def test_update_dhcp_set_sleep_and_fake(self, mock_gnvi, mock_updo,
+    def test_update_dhcp_set_sleep_and_fake(self, mock_gnvi,
                                             mock_ts, mock_log):
         mock_gnvi.return_value = {'ports': {'port-uuid': 'vif-uuid'},
                                   'portgroups': {}}
@@ -278,31 +155,30 @@ class TestNeutron(db_base.DbTestCase):
                                   self.node.uuid) as task:
             opts = pxe_utils.dhcp_options_for_instance(task)
             api = dhcp_factory.DHCPFactory()
-            api.update_dhcp(task, opts)
-            mock_log.debug.assert_called_once_with(
-                "Waiting %d seconds for Neutron.", 30)
-            mock_log.warning.assert_not_called()
-            mock_ts.assert_called_with(30)
-        mock_updo.assert_called_once_with(mock.ANY, 'vif-uuid', opts,
-                                          token=self.context.auth_token)
+            with mock.patch.object(api.provider, 'update_port_dhcp_opts',
+                                   autospec=True) as mock_updo:
+                api.update_dhcp(task, opts)
+                mock_log.debug.assert_called_once_with(
+                    "Waiting %d seconds for Neutron.", 30)
+                mock_ts.assert_called_with(30)
+                mock_updo.assert_called_once_with('vif-uuid', opts,
+                                                  context=task.context)
 
     @mock.patch.object(neutron, 'LOG', autospec=True)
-    @mock.patch.object(neutron.NeutronDHCPApi, 'update_port_dhcp_opts',
-                       autospec=True)
     @mock.patch('ironic.common.network.get_node_vif_ids', autospec=True)
-    def test_update_dhcp_unset_sleep_and_fake(self, mock_gnvi, mock_updo,
-                                              mock_log):
+    def test_update_dhcp_unset_sleep_and_fake(self, mock_gnvi, mock_log):
         mock_gnvi.return_value = {'ports': {'port-uuid': 'vif-uuid'},
                                   'portgroups': {}}
         with task_manager.acquire(self.context,
                                   self.node.uuid) as task:
             opts = pxe_utils.dhcp_options_for_instance(task)
             api = dhcp_factory.DHCPFactory()
-            api.update_dhcp(task, opts)
-            mock_log.debug.assert_not_called()
-            mock_log.warning.assert_not_called()
-        mock_updo.assert_called_once_with(mock.ANY, 'vif-uuid', opts,
-                                          token=self.context.auth_token)
+            with mock.patch.object(api.provider, 'update_port_dhcp_opts',
+                                   autospec=True) as mock_updo:
+                api.update_dhcp(task, opts)
+                mock_log.debug.assert_not_called()
+                mock_updo.assert_called_once_with('vif-uuid', opts,
+                                                  context=task.context)
 
     def test__get_fixed_ip_address(self):
         port_id = 'fake-port-id'
@@ -359,24 +235,26 @@ class TestNeutron(db_base.DbTestCase):
         fake_client = mock.Mock()
         fake_client.show_port.side_effect = (
             neutron_client_exc.NeutronClientException())
-        self.assertRaises(exception.FailedToGetIPAddressOnPort,
+        self.assertRaises(exception.NetworkError,
                           api._get_fixed_ip_address, port_id, fake_client)
         fake_client.show_port.assert_called_once_with(port_id)
 
-    @mock.patch('ironic.dhcp.neutron.NeutronDHCPApi._get_fixed_ip_address')
+    @mock.patch('ironic.dhcp.neutron.NeutronDHCPApi._get_fixed_ip_address',
+                autospec=True)
     def _test__get_port_ip_address(self, mock_gfia, network):
         expected = "192.168.1.3"
         fake_vif = 'test-vif-%s' % network
         port = object_utils.create_test_port(
             self.context, node_id=self.node.id, address='aa:bb:cc:dd:ee:ff',
             uuid=uuidutils.generate_uuid(),
-            extra={'vif_port_id': fake_vif} if network == 'tenant' else {},
             internal_info={
                 'cleaning_vif_port_id': (fake_vif if network == 'cleaning'
                                          else None),
                 'provisioning_vif_port_id': (fake_vif
                                              if network == 'provisioning'
                                              else None),
+                'tenant_vif_port_id': (fake_vif if network == 'tenant'
+                                       else None),
             }
         )
         mock_gfia.return_value = expected
@@ -386,7 +264,8 @@ class TestNeutron(db_base.DbTestCase):
             result = api._get_port_ip_address(task, port,
                                               mock.sentinel.client)
         self.assertEqual(expected, result)
-        mock_gfia.assert_called_once_with(fake_vif, mock.sentinel.client)
+        mock_gfia.assert_called_once_with(mock.ANY, fake_vif,
+                                          mock.sentinel.client)
 
     def test__get_port_ip_address_tenant(self):
         self._test__get_port_ip_address(network='tenant')
@@ -397,15 +276,14 @@ class TestNeutron(db_base.DbTestCase):
     def test__get_port_ip_address_provisioning(self):
         self._test__get_port_ip_address(network='provisioning')
 
-    @mock.patch('ironic.dhcp.neutron.NeutronDHCPApi._get_fixed_ip_address')
+    @mock.patch('ironic.dhcp.neutron.NeutronDHCPApi._get_fixed_ip_address',
+                autospec=True)
     def test__get_port_ip_address_for_portgroup(self, mock_gfia):
         expected = "192.168.1.3"
-        pg = object_utils.create_test_portgroup(self.context,
-                                                node_id=self.node.id,
-                                                address='aa:bb:cc:dd:ee:ff',
-                                                uuid=uuidutils.generate_uuid(),
-                                                extra={'vif_port_id':
-                                                       'test-vif-A'})
+        pg = object_utils.create_test_portgroup(
+            self.context, node_id=self.node.id, address='aa:bb:cc:dd:ee:ff',
+            uuid=uuidutils.generate_uuid(),
+            internal_info={'tenant_vif_port_id': 'test-vif-A'})
         mock_gfia.return_value = expected
         with task_manager.acquire(self.context,
                                   self.node.uuid) as task:
@@ -413,9 +291,11 @@ class TestNeutron(db_base.DbTestCase):
             result = api._get_port_ip_address(task, pg,
                                               mock.sentinel.client)
         self.assertEqual(expected, result)
-        mock_gfia.assert_called_once_with('test-vif-A', mock.sentinel.client)
+        mock_gfia.assert_called_once_with(mock.ANY, 'test-vif-A',
+                                          mock.sentinel.client)
 
-    @mock.patch('ironic.dhcp.neutron.NeutronDHCPApi._get_fixed_ip_address')
+    @mock.patch('ironic.dhcp.neutron.NeutronDHCPApi._get_fixed_ip_address',
+                autospec=True)
     def test__get_port_ip_address_with_exception(self, mock_gfia):
         expected = "192.168.1.3"
         port = object_utils.create_test_port(self.context,
@@ -430,7 +310,8 @@ class TestNeutron(db_base.DbTestCase):
                               api._get_port_ip_address, task, port,
                               mock.sentinel.client)
 
-    @mock.patch('ironic.dhcp.neutron.NeutronDHCPApi._get_fixed_ip_address')
+    @mock.patch('ironic.dhcp.neutron.NeutronDHCPApi._get_fixed_ip_address',
+                autospec=True)
     def test__get_port_ip_address_for_portgroup_with_exception(
             self, mock_gfia):
         expected = "192.168.1.3"
@@ -446,16 +327,20 @@ class TestNeutron(db_base.DbTestCase):
                               api._get_port_ip_address, task, pg,
                               mock.sentinel.client)
 
-    @mock.patch('ironic.dhcp.neutron.NeutronDHCPApi._get_fixed_ip_address')
-    def test__get_ip_addresses_ports(self, mock_gfia):
+    @mock.patch('ironic.dhcp.neutron.NeutronDHCPApi._get_fixed_ip_address',
+                autospec=True)
+    def _test__get_ip_addresses_ports(self, key, mock_gfia):
+        if key == "extra":
+            kwargs1 = {key: {'vif_port_id': 'test-vif-A'}}
+        else:
+            kwargs1 = {key: {'tenant_vif_port_id': 'test-vif-A'}}
         ip_address = '10.10.0.1'
         expected = [ip_address]
         port = object_utils.create_test_port(self.context,
                                              node_id=self.node.id,
                                              address='aa:bb:cc:dd:ee:ff',
                                              uuid=uuidutils.generate_uuid(),
-                                             extra={'vif_port_id':
-                                                    'test-vif-A'})
+                                             **kwargs1)
         mock_gfia.return_value = ip_address
         with task_manager.acquire(self.context, self.node.uuid) as task:
             api = dhcp_factory.DHCPFactory().provider
@@ -463,24 +348,41 @@ class TestNeutron(db_base.DbTestCase):
                                            mock.sentinel.client)
         self.assertEqual(expected, result)
 
-    @mock.patch('ironic.dhcp.neutron.NeutronDHCPApi._get_fixed_ip_address')
-    def test__get_ip_addresses_portgroup(self, mock_gfia):
+    def test__get_ip_addresses_ports_extra(self):
+        self._test__get_ip_addresses_ports('extra')
+
+    def test__get_ip_addresses_ports_int_info(self):
+        self._test__get_ip_addresses_ports('internal_info')
+
+    @mock.patch('ironic.dhcp.neutron.NeutronDHCPApi._get_fixed_ip_address',
+                autospec=True)
+    def _test__get_ip_addresses_portgroup(self, key, mock_gfia):
+        if key == "extra":
+            kwargs1 = {key: {'vif_port_id': 'test-vif-A'}}
+        else:
+            kwargs1 = {key: {'tenant_vif_port_id': 'test-vif-A'}}
         ip_address = '10.10.0.1'
         expected = [ip_address]
-        pg = object_utils.create_test_portgroup(self.context,
-                                                node_id=self.node.id,
-                                                address='aa:bb:cc:dd:ee:ff',
-                                                uuid=uuidutils.generate_uuid(),
-                                                extra={'vif_port_id':
-                                                       'test-vif-A'})
+        pg = object_utils.create_test_portgroup(
+            self.context, node_id=self.node.id,
+            address='aa:bb:cc:dd:ee:ff', uuid=uuidutils.generate_uuid(),
+            **kwargs1)
         mock_gfia.return_value = ip_address
         with task_manager.acquire(self.context, self.node.uuid) as task:
             api = dhcp_factory.DHCPFactory().provider
             result = api._get_ip_addresses(task, [pg], mock.sentinel.client)
         self.assertEqual(expected, result)
 
-    @mock.patch('ironic.dhcp.neutron.NeutronDHCPApi._get_port_ip_address')
-    def test_get_ip_addresses(self, get_ip_mock):
+    def test__get_ip_addresses_portgroup_extra(self):
+        self._test__get_ip_addresses_portgroup('extra')
+
+    def test__get_ip_addresses_portgroup_int_info(self):
+        self._test__get_ip_addresses_portgroup('internal_info')
+
+    @mock.patch('ironic.common.neutron.get_client', autospec=True)
+    @mock.patch('ironic.dhcp.neutron.NeutronDHCPApi._get_port_ip_address',
+                autospec=True)
+    def test_get_ip_addresses(self, get_ip_mock, client_mock):
         ip_address = '10.10.0.1'
         expected = [ip_address]
 
@@ -489,53 +391,26 @@ class TestNeutron(db_base.DbTestCase):
         with task_manager.acquire(self.context, self.node.uuid) as task:
             api = dhcp_factory.DHCPFactory().provider
             result = api.get_ip_addresses(task)
-            get_ip_mock.assert_called_once_with(task, task.ports[0],
-                                                mock.ANY)
+            get_ip_mock.assert_called_once_with(mock.ANY, task, task.ports[0],
+                                                client_mock.return_value)
         self.assertEqual(expected, result)
 
-    @mock.patch('ironic.dhcp.neutron.NeutronDHCPApi._get_port_ip_address')
-    def test_get_ip_addresses_for_port_and_portgroup(self, get_ip_mock):
-        object_utils.create_test_portgroup(self.context,
-                                           node_id=self.node.id,
-                                           address='aa:bb:cc:dd:ee:ff',
-                                           uuid=uuidutils.generate_uuid(),
-                                           extra={'vif_port_id':
-                                                  'test-vif-A'})
+    @mock.patch('ironic.common.neutron.get_client', autospec=True)
+    @mock.patch('ironic.dhcp.neutron.NeutronDHCPApi._get_port_ip_address',
+                autospec=True)
+    def test_get_ip_addresses_for_port_and_portgroup(self, get_ip_mock,
+                                                     client_mock):
+        object_utils.create_test_portgroup(
+            self.context, node_id=self.node.id, address='aa:bb:cc:dd:ee:ff',
+            uuid=uuidutils.generate_uuid(),
+            internal_info={'tenant_vif_port_id': 'test-vif-A'})
+
         with task_manager.acquire(self.context, self.node.uuid) as task:
             api = dhcp_factory.DHCPFactory().provider
             api.get_ip_addresses(task)
             get_ip_mock.assert_has_calls(
-                [mock.call(task, task.ports[0], mock.ANY),
-                 mock.call(task, task.portgroups[0], mock.ANY)])
-
-    @mock.patch.object(neutron, 'create_cleaning_ports_deprecation', False)
-    @mock.patch.object(neutron, 'LOG', autospec=True)
-    def test_create_cleaning_ports(self, log_mock):
-        self.config(cleaning_network_uuid=uuidutils.generate_uuid(),
-                    group='neutron')
-        api = dhcp_factory.DHCPFactory().provider
-
-        with task_manager.acquire(self.context, self.node.uuid) as task:
-            with mock.patch.object(
-                    task.driver.network, 'add_cleaning_network',
-                    autospec=True) as add_net_mock:
-                api.create_cleaning_ports(task)
-                add_net_mock.assert_called_once_with(task)
-
-                api.create_cleaning_ports(task)
-                self.assertEqual(1, log_mock.warning.call_count)
-
-    @mock.patch.object(neutron, 'delete_cleaning_ports_deprecation', False)
-    @mock.patch.object(neutron, 'LOG', autospec=True)
-    def test_delete_cleaning_ports(self, log_mock):
-        api = dhcp_factory.DHCPFactory().provider
-
-        with task_manager.acquire(self.context, self.node.uuid) as task:
-            with mock.patch.object(
-                    task.driver.network, 'remove_cleaning_network',
-                    autospec=True) as rm_net_mock:
-                api.delete_cleaning_ports(task)
-                rm_net_mock.assert_called_once_with(task)
-
-                api.delete_cleaning_ports(task)
-                self.assertEqual(1, log_mock.warning.call_count)
+                [mock.call(mock.ANY, task, task.ports[0],
+                           client_mock.return_value),
+                 mock.call(mock.ANY, task, task.portgroups[0],
+                           client_mock.return_value)]
+            )
