@@ -19,7 +19,6 @@
 
 import errno
 import os
-import psutil
 import random
 import signal
 import string
@@ -29,7 +28,9 @@ import tempfile
 from ironic_lib import utils as ironic_utils
 import mock
 from oslo_config import cfg
+from oslo_service import loopingcall
 from oslo_utils import netutils
+import psutil
 
 from ironic.common import exception
 from ironic.drivers.modules import console_utils
@@ -50,7 +51,6 @@ class ConsoleUtilsTestCase(db_base.DbTestCase):
         super(ConsoleUtilsTestCase, self).setUp()
         self.node = obj_utils.get_test_node(
             self.context,
-            driver='fake_ipmitool',
             driver_info=INFO_DICT)
         self.info = ipmi._parse_driver_info(self.node)
 
@@ -401,6 +401,61 @@ class ConsoleUtilsTestCase(db_base.DbTestCase):
         url = console_utils.get_socat_console_url(self.info['port'])
         self.assertEqual("tcp://[::1]:%s" % self.info['port'], url)
 
+    def test_get_socat_console_url_tcp_with_address_conf(self):
+        self.config(socat_address="10.0.0.1", group='console')
+        url = console_utils.get_socat_console_url(self.info['port'])
+        self.assertEqual("tcp://10.0.0.1:%s" % self.info['port'], url)
+
+    @mock.patch.object(subprocess, 'Popen', autospec=True)
+    @mock.patch.object(console_utils, '_get_console_pid_file', autospec=True)
+    @mock.patch.object(console_utils, '_ensure_console_pid_dir_exists',
+                       autospec=True)
+    @mock.patch.object(console_utils, '_stop_console', autospec=True)
+    @mock.patch.object(loopingcall.FixedIntervalLoopingCall, 'start',
+                       autospec=True)
+    def _test_start_socat_console_check_arg(self, mock_timer_start,
+                                            mock_stop, mock_dir_exists,
+                                            mock_get_pid, mock_popen):
+        mock_timer_start.return_value = mock.Mock()
+        mock_get_pid.return_value = '/tmp/%s.pid' % self.info['uuid']
+
+        console_utils.start_socat_console(self.info['uuid'],
+                                          self.info['port'],
+                                          'ls&')
+
+        mock_stop.assert_called_once_with(self.info['uuid'])
+        mock_dir_exists.assert_called_once_with()
+        mock_get_pid.assert_called_once_with(self.info['uuid'])
+        mock_timer_start.assert_called_once_with(mock.ANY, interval=mock.ANY)
+        mock_popen.assert_called_once_with(mock.ANY, stderr=subprocess.PIPE)
+        return mock_popen.call_args[0][0]
+
+    def test_start_socat_console_check_arg_default_timeout(self):
+        args = self._test_start_socat_console_check_arg()
+        self.assertIn('-T600', args)
+
+    def test_start_socat_console_check_arg_timeout(self):
+        self.config(terminal_timeout=1, group='console')
+        args = self._test_start_socat_console_check_arg()
+        self.assertIn('-T1', args)
+
+    def test_start_socat_console_check_arg_timeout_disabled(self):
+        self.config(terminal_timeout=0, group='console')
+        args = self._test_start_socat_console_check_arg()
+        self.assertNotIn('-T0', args)
+
+    def test_start_socat_console_check_arg_bind_addr_default_ipv4(self):
+        self.config(my_ip='10.0.0.1')
+        args = self._test_start_socat_console_check_arg()
+        self.assertIn('TCP4-LISTEN:%s,bind=10.0.0.1,reuseaddr' %
+                      self.info['port'], args)
+
+    def test_start_socat_console_check_arg_bind_addr_ipv4(self):
+        self.config(socat_address='10.0.0.1', group='console')
+        args = self._test_start_socat_console_check_arg()
+        self.assertIn('TCP4-LISTEN:%s,bind=10.0.0.1,reuseaddr' %
+                      self.info['port'], args)
+
     @mock.patch.object(os.path, 'exists', autospec=True)
     @mock.patch.object(subprocess, 'Popen', autospec=True)
     @mock.patch.object(psutil, 'pid_exists', autospec=True)
@@ -501,7 +556,7 @@ class ConsoleUtilsTestCase(db_base.DbTestCase):
 
         mock_stop.assert_called_once_with(self.info['uuid'])
         mock_dir_exists.assert_called_once_with()
-        mock_popen.assert_not_called()
+        self.assertEqual(0, mock_popen.call_count)
 
     @mock.patch.object(console_utils, '_stop_console', autospec=True)
     def test_stop_socat_console(self, mock_stop):
