@@ -24,8 +24,8 @@ import mock
 from oslo_config import cfg
 from oslo_utils import fileutils
 
+from ironic.common import boot_devices
 from ironic.common import dhcp_factory
-from ironic.common import driver_factory
 from ironic.common import exception
 from ironic.common import pxe_utils
 from ironic.common import states
@@ -35,10 +35,12 @@ from ironic.conductor import utils as manager_utils
 from ironic.drivers.modules import agent_base_vendor
 from ironic.drivers.modules import agent_client
 from ironic.drivers.modules import deploy_utils
+from ironic.drivers.modules import fake
 from ironic.drivers.modules import iscsi_deploy
+from ironic.drivers.modules.network import flat as flat_network
 from ironic.drivers.modules import pxe
+from ironic.drivers.modules.storage import noop as noop_storage
 from ironic.drivers import utils as driver_utils
-from ironic.tests.unit.conductor import mgr_utils
 from ironic.tests.unit.db import base as db_base
 from ironic.tests.unit.db import utils as db_utils
 from ironic.tests.unit.objects import utils as obj_utils
@@ -55,12 +57,12 @@ class IscsiDeployPrivateMethodsTestCase(db_base.DbTestCase):
     def setUp(self):
         super(IscsiDeployPrivateMethodsTestCase, self).setUp()
         n = {
-            'driver': 'fake_pxe',
+            'boot_interface': 'pxe',
+            'deploy_interface': 'iscsi',
             'instance_info': INST_INFO_DICT,
             'driver_info': DRV_INFO_DICT,
             'driver_internal_info': DRV_INTERNAL_INFO_DICT,
         }
-        mgr_utils.mock_the_extension_manager(driver="fake_pxe")
         self.node = obj_utils.create_test_node(self.context, **n)
 
     def test__save_disk_layout(self):
@@ -97,12 +99,12 @@ class IscsiDeployMethodsTestCase(db_base.DbTestCase):
         instance_info = dict(INST_INFO_DICT)
         instance_info['deploy_key'] = 'fake-56789'
         n = {
-            'driver': 'fake_pxe',
+            'boot_interface': 'pxe',
+            'deploy_interface': 'iscsi',
             'instance_info': instance_info,
             'driver_info': DRV_INFO_DICT,
             'driver_internal_info': DRV_INTERNAL_INFO_DICT,
         }
-        mgr_utils.mock_the_extension_manager(driver="fake_pxe")
         self.node = obj_utils.create_test_node(self.context, **n)
 
     @mock.patch.object(disk_utils, 'get_image_mb', autospec=True)
@@ -115,6 +117,17 @@ class IscsiDeployMethodsTestCase(db_base.DbTestCase):
             get_image_mb_mock.assert_called_once_with(
                 iscsi_deploy._get_image_file_path(task.node.uuid))
 
+    @mock.patch.object(disk_utils, 'get_image_mb', autospec=True)
+    def test_check_image_size_whole_disk_image(self, get_image_mb_mock):
+        get_image_mb_mock.return_value = 1025
+        with task_manager.acquire(self.context, self.node.uuid,
+                                  shared=False) as task:
+            task.node.instance_info['root_gb'] = 1
+            task.node.driver_internal_info['is_whole_disk_image'] = True
+            # No error for whole disk images
+            iscsi_deploy.check_image_size(task)
+            self.assertFalse(get_image_mb_mock.called)
+
     @mock.patch.object(disk_utils, 'get_image_mb', autospec=True)
     def test_check_image_size_fails(self, get_image_mb_mock):
         get_image_mb_mock.return_value = 1025
@@ -176,9 +189,42 @@ class IscsiDeployMethodsTestCase(db_base.DbTestCase):
         with task_manager.acquire(self.context, self.node.uuid,
                                   shared=False) as task:
             params = iscsi_deploy.get_deploy_info(task.node, **kwargs)
-            self.assertRaises(exception.InstanceDeployFailure,
-                              iscsi_deploy.continue_deploy,
-                              task, **kwargs)
+            # Ironic exceptions are preserved as they are
+            self.assertRaisesRegex(exception.InstanceDeployFailure,
+                                   '^test deploy error$',
+                                   iscsi_deploy.continue_deploy,
+                                   task, **kwargs)
+            self.assertEqual(states.DEPLOYFAIL, task.node.provision_state)
+            self.assertEqual(states.ACTIVE, task.node.target_provision_state)
+            self.assertIsNotNone(task.node.last_error)
+            deploy_mock.assert_called_once_with(**params)
+            power_mock.assert_called_once_with(task, states.POWER_OFF)
+            mock_image_cache.assert_called_once_with()
+            mock_image_cache.return_value.clean_up.assert_called_once_with()
+            self.assertFalse(mock_disk_layout.called)
+            mock_collect_logs.assert_called_once_with(task.node)
+
+    @mock.patch.object(driver_utils, 'collect_ramdisk_logs', autospec=True)
+    @mock.patch.object(iscsi_deploy, '_save_disk_layout', autospec=True)
+    @mock.patch.object(iscsi_deploy, 'InstanceImageCache', autospec=True)
+    @mock.patch.object(manager_utils, 'node_power_action', autospec=True)
+    @mock.patch.object(deploy_utils, 'deploy_partition_image', autospec=True)
+    def test_continue_deploy_unexpected_fail(
+            self, deploy_mock, power_mock, mock_image_cache, mock_disk_layout,
+            mock_collect_logs):
+        kwargs = {'address': '123456', 'iqn': 'aaa-bbb'}
+        deploy_mock.side_effect = KeyError('boom')
+        self.node.provision_state = states.DEPLOYWAIT
+        self.node.target_provision_state = states.ACTIVE
+        self.node.save()
+
+        with task_manager.acquire(self.context, self.node.uuid,
+                                  shared=False) as task:
+            params = iscsi_deploy.get_deploy_info(task.node, **kwargs)
+            self.assertRaisesRegex(exception.InstanceDeployFailure,
+                                   "Deploy failed.*Error: 'boom'",
+                                   iscsi_deploy.continue_deploy,
+                                   task, **kwargs)
             self.assertEqual(states.DEPLOYFAIL, task.node.provision_state)
             self.assertEqual(states.ACTIVE, task.node.target_provision_state)
             self.assertIsNotNone(task.node.last_error)
@@ -386,6 +432,17 @@ class IscsiDeployMethodsTestCase(db_base.DbTestCase):
         ret_val = self._test_get_deploy_info()
         self.assertEqual(3266, ret_val['port'])
 
+    def test_get_deploy_info_whole_disk_image(self):
+        instance_info = self.node.instance_info
+        instance_info['configdrive'] = 'My configdrive'
+        self.node.instance_info = instance_info
+        self.node.driver_internal_info['is_whole_disk_image'] = True
+        kwargs = {'address': '1.1.1.1', 'iqn': 'target-iqn'}
+        ret_val = iscsi_deploy.get_deploy_info(self.node, **kwargs)
+        self.assertEqual('1.1.1.1', ret_val['address'])
+        self.assertEqual('target-iqn', ret_val['iqn'])
+        self.assertEqual('My configdrive', ret_val['configdrive'])
+
     @mock.patch.object(iscsi_deploy, 'continue_deploy', autospec=True)
     def test_do_agent_iscsi_deploy_okay(self, continue_deploy_mock):
         agent_client_mock = mock.MagicMock(spec_set=agent_client.AgentClient)
@@ -475,7 +532,9 @@ class IscsiDeployMethodsTestCase(db_base.DbTestCase):
                               iscsi_deploy.validate, task)
         mock_get_url.assert_called_once_with()
 
-    def test_validate_invalid_root_device_hints(self):
+    @mock.patch('ironic.drivers.modules.deploy_utils.get_ironic_api_url')
+    def test_validate_invalid_root_device_hints(self, mock_get_url):
+        mock_get_url.return_value = 'http://spam.ham/baremetal'
         with task_manager.acquire(self.context, self.node.uuid,
                                   shared=True) as task:
             task.node.properties['root_device'] = {'size': 'not-int'}
@@ -487,27 +546,25 @@ class ISCSIDeployTestCase(db_base.DbTestCase):
 
     def setUp(self):
         super(ISCSIDeployTestCase, self).setUp()
-        mgr_utils.mock_the_extension_manager(driver="fake_pxe")
-        self.driver = driver_factory.get_driver("fake_pxe")
-        self.driver.vendor = iscsi_deploy.VendorPassthru()
+        # NOTE(TheJulia): We explicitly set the noop storage interface as the
+        # default below for deployment tests in order to raise any change
+        # in the default which could be a breaking behavior change
+        # as the storage interface is explicitly an "opt-in" interface.
         self.node = obj_utils.create_test_node(
-            self.context, driver='fake_pxe',
+            self.context, boot_interface='pxe', deploy_interface='iscsi',
             instance_info=INST_INFO_DICT,
             driver_info=DRV_INFO_DICT,
             driver_internal_info=DRV_INTERNAL_INFO_DICT,
+            storage_interface='noop',
         )
         self.node.driver_internal_info['agent_url'] = 'http://1.2.3.4:1234'
-        self.task = mock.MagicMock(spec=task_manager.TaskManager)
-        self.task.shared = False
-        self.task.node = self.node
-        self.task.driver = self.driver
-        self.task.context = self.context
         dhcp_factory.DHCPFactory._dhcp_provider = None
 
     def test_get_properties(self):
         with task_manager.acquire(self.context, self.node.uuid,
                                   shared=True) as task:
-            self.assertEqual({}, task.driver.deploy.get_properties())
+            props = task.driver.deploy.get_properties()
+            self.assertEqual(['deploy_forces_oob_reboot'], list(props))
 
     @mock.patch.object(iscsi_deploy, 'validate', autospec=True)
     @mock.patch.object(deploy_utils, 'validate_capabilities', autospec=True)
@@ -523,11 +580,38 @@ class ISCSIDeployTestCase(db_base.DbTestCase):
             validate_capabilities_mock.assert_called_once_with(task.node)
             validate_mock.assert_called_once_with(task)
 
-    @mock.patch('ironic.drivers.modules.network.flat.FlatNetwork.'
-                'add_provisioning_network', spec_set=True, autospec=True)
+    @mock.patch.object(noop_storage.NoopStorage, 'should_write_image',
+                       autospec=True)
+    @mock.patch.object(iscsi_deploy, 'validate', autospec=True)
+    @mock.patch.object(deploy_utils, 'validate_capabilities', autospec=True)
+    @mock.patch.object(pxe.PXEBoot, 'validate', autospec=True)
+    def test_validate_storage_should_write_image_false(
+            self, pxe_validate_mock,
+            validate_capabilities_mock, validate_mock,
+            should_write_image_mock):
+        should_write_image_mock.return_value = False
+        with task_manager.acquire(self.context, self.node.uuid,
+                                  shared=True) as task:
+
+            task.driver.deploy.validate(task)
+
+            pxe_validate_mock.assert_called_once_with(task.driver.boot, task)
+            validate_capabilities_mock.assert_called_once_with(task.node)
+            self.assertFalse(validate_mock.called)
+            should_write_image_mock.assert_called_once_with(
+                task.driver.storage, task)
+
+    @mock.patch.object(noop_storage.NoopStorage, 'attach_volumes',
+                       autospec=True)
+    @mock.patch.object(deploy_utils, 'populate_storage_driver_internal_info',
+                       autospec=True)
+    @mock.patch.object(flat_network.FlatNetwork, 'add_provisioning_network',
+                       spec_set=True, autospec=True)
     @mock.patch.object(pxe.PXEBoot, 'prepare_instance', autospec=True)
     def test_prepare_node_active(self, prepare_instance_mock,
-                                 add_provisioning_net_mock):
+                                 add_provisioning_net_mock,
+                                 storage_driver_info_mock,
+                                 storage_attach_volumes_mock):
         with task_manager.acquire(self.context, self.node.uuid) as task:
             task.node.provision_state = states.ACTIVE
 
@@ -536,9 +620,11 @@ class ISCSIDeployTestCase(db_base.DbTestCase):
             prepare_instance_mock.assert_called_once_with(
                 task.driver.boot, task)
             self.assertEqual(0, add_provisioning_net_mock.call_count)
+            storage_driver_info_mock.assert_called_once_with(task)
+        self.assertFalse(storage_attach_volumes_mock.called)
 
-    @mock.patch('ironic.drivers.modules.network.flat.FlatNetwork.'
-                'add_provisioning_network', spec_set=True, autospec=True)
+    @mock.patch.object(flat_network.FlatNetwork, 'add_provisioning_network',
+                       spec_set=True, autospec=True)
     @mock.patch.object(pxe.PXEBoot, 'prepare_instance', autospec=True)
     def test_prepare_node_adopting(self, prepare_instance_mock,
                                    add_provisioning_net_mock):
@@ -551,13 +637,21 @@ class ISCSIDeployTestCase(db_base.DbTestCase):
                 task.driver.boot, task)
             self.assertEqual(0, add_provisioning_net_mock.call_count)
 
+    @mock.patch.object(noop_storage.NoopStorage, 'attach_volumes',
+                       autospec=True)
+    @mock.patch.object(deploy_utils, 'populate_storage_driver_internal_info',
+                       autospec=True)
     @mock.patch.object(deploy_utils, 'build_agent_options', autospec=True)
     @mock.patch.object(pxe.PXEBoot, 'prepare_ramdisk', autospec=True)
-    @mock.patch('ironic.drivers.modules.network.flat.FlatNetwork.'
-                'add_provisioning_network', spec_set=True, autospec=True)
-    def test_prepare_node_deploying(self, add_provisioning_net_mock,
-                                    mock_prepare_ramdisk,
-                                    mock_agent_options):
+    @mock.patch.object(flat_network.FlatNetwork, 'add_provisioning_network',
+                       spec_set=True, autospec=True)
+    @mock.patch.object(flat_network.FlatNetwork,
+                       'unconfigure_tenant_networks',
+                       spec_set=True, autospec=True)
+    def test_prepare_node_deploying(
+            self, unconfigure_tenant_net_mock, add_provisioning_net_mock,
+            mock_prepare_ramdisk, mock_agent_options,
+            storage_driver_info_mock, storage_attach_volumes_mock):
         mock_agent_options.return_value = {'c': 'd'}
         with task_manager.acquire(self.context, self.node.uuid) as task:
             task.node.provision_state = states.DEPLOYING
@@ -568,6 +662,44 @@ class ISCSIDeployTestCase(db_base.DbTestCase):
             mock_prepare_ramdisk.assert_called_once_with(
                 task.driver.boot, task, {'c': 'd'})
             add_provisioning_net_mock.assert_called_once_with(mock.ANY, task)
+            unconfigure_tenant_net_mock.assert_called_once_with(mock.ANY, task)
+            storage_driver_info_mock.assert_called_once_with(task)
+            storage_attach_volumes_mock.assert_called_once_with(
+                task.driver.storage, task)
+
+    @mock.patch.object(noop_storage.NoopStorage, 'should_write_image',
+                       autospec=True)
+    @mock.patch.object(noop_storage.NoopStorage, 'attach_volumes',
+                       autospec=True)
+    @mock.patch.object(deploy_utils, 'populate_storage_driver_internal_info',
+                       autospec=True)
+    @mock.patch.object(deploy_utils, 'build_agent_options', autospec=True)
+    @mock.patch.object(pxe.PXEBoot, 'prepare_ramdisk', autospec=True)
+    @mock.patch.object(flat_network.FlatNetwork, 'add_provisioning_network',
+                       spec_set=True, autospec=True)
+    @mock.patch.object(flat_network.FlatNetwork,
+                       'unconfigure_tenant_networks',
+                       spec_set=True, autospec=True)
+    def test_prepare_node_deploying_storage_should_write_false(
+            self, unconfigure_tenant_net_mock, add_provisioning_net_mock,
+            mock_prepare_ramdisk, mock_agent_options,
+            storage_driver_info_mock, storage_attach_volumes_mock,
+            storage_should_write_mock):
+        storage_should_write_mock.return_value = False
+        mock_agent_options.return_value = {'c': 'd'}
+        with task_manager.acquire(self.context, self.node.uuid) as task:
+            task.node.provision_state = states.DEPLOYING
+
+            task.driver.deploy.prepare(task)
+
+            self.assertFalse(mock_agent_options.called)
+            self.assertFalse(mock_prepare_ramdisk.called)
+            self.assertFalse(add_provisioning_net_mock.called)
+            self.assertFalse(unconfigure_tenant_net_mock.called)
+            storage_driver_info_mock.assert_called_once_with(task)
+            storage_attach_volumes_mock.assert_called_once_with(
+                task.driver.storage, task)
+            self.assertEqual(2, storage_should_write_mock.call_count)
 
     @mock.patch.object(manager_utils, 'node_power_action', autospec=True)
     @mock.patch.object(iscsi_deploy, 'check_image_size', autospec=True)
@@ -583,11 +715,60 @@ class ISCSIDeployTestCase(db_base.DbTestCase):
             mock_check_image_size.assert_called_once_with(task)
             mock_node_power_action.assert_called_once_with(task, states.REBOOT)
 
-    @mock.patch('ironic.drivers.modules.network.flat.FlatNetwork.'
-                'unconfigure_tenant_networks', autospec=True)
+    @mock.patch.object(noop_storage.NoopStorage, 'should_write_image',
+                       autospec=True)
+    @mock.patch.object(flat_network.FlatNetwork,
+                       'configure_tenant_networks',
+                       spec_set=True, autospec=True)
+    @mock.patch.object(flat_network.FlatNetwork,
+                       'remove_provisioning_network',
+                       spec_set=True, autospec=True)
+    @mock.patch.object(pxe.PXEBoot,
+                       'prepare_instance',
+                       spec_set=True, autospec=True)
+    @mock.patch.object(manager_utils, 'node_power_action', autospec=True)
+    @mock.patch.object(iscsi_deploy, 'check_image_size', autospec=True)
+    @mock.patch.object(iscsi_deploy, 'cache_instance_image', autospec=True)
+    def test_deploy_storage_check_write_image_false(self,
+                                                    mock_cache_instance_image,
+                                                    mock_check_image_size,
+                                                    mock_node_power_action,
+                                                    mock_prepare_instance,
+                                                    mock_remove_network,
+                                                    mock_tenant_network,
+                                                    mock_write):
+        mock_write.return_value = False
+        self.node.provision_state = states.DEPLOYING
+        self.node.deploy_step = {
+            'step': 'deploy', 'priority': 50, 'interface': 'deploy'}
+        self.node.save()
+        with task_manager.acquire(self.context,
+                                  self.node.uuid, shared=False) as task:
+            ret = task.driver.deploy.deploy(task)
+            self.assertIsNone(ret)
+            self.assertFalse(mock_cache_instance_image.called)
+            self.assertFalse(mock_check_image_size.called)
+            mock_remove_network.assert_called_once_with(mock.ANY, task)
+            mock_tenant_network.assert_called_once_with(mock.ANY, task)
+            mock_prepare_instance.assert_called_once_with(mock.ANY, task)
+            self.assertEqual(2, mock_node_power_action.call_count)
+            self.assertEqual(states.DEPLOYING, task.node.provision_state)
+
+    @mock.patch.object(noop_storage.NoopStorage, 'detach_volumes',
+                       autospec=True)
+    @mock.patch.object(flat_network.FlatNetwork,
+                       'remove_provisioning_network',
+                       spec_set=True, autospec=True)
+    @mock.patch.object(flat_network.FlatNetwork,
+                       'unconfigure_tenant_networks',
+                       spec_set=True, autospec=True)
     @mock.patch.object(manager_utils, 'node_power_action', autospec=True)
     def test_tear_down(self, node_power_action_mock,
-                       unconfigure_tenant_nets_mock):
+                       unconfigure_tenant_nets_mock,
+                       remove_provisioning_net_mock,
+                       storage_detach_volumes_mock):
+        obj_utils.create_test_volume_target(
+            self.context, node_id=self.node.id)
         with task_manager.acquire(self.context,
                                   self.node.uuid, shared=False) as task:
             state = task.driver.deploy.tear_down(task)
@@ -596,6 +777,14 @@ class ISCSIDeployTestCase(db_base.DbTestCase):
                                                            states.POWER_OFF)
             unconfigure_tenant_nets_mock.assert_called_once_with(mock.ANY,
                                                                  task)
+            remove_provisioning_net_mock.assert_called_once_with(mock.ANY,
+                                                                 task)
+            storage_detach_volumes_mock.assert_called_once_with(
+                task.driver.storage, task)
+        # Verify no volumes exist for new task instances.
+        with task_manager.acquire(self.context,
+                                  self.node.uuid, shared=False) as task:
+            self.assertEqual(0, len(task.volume_targets))
 
     @mock.patch('ironic.common.dhcp_factory.DHCPFactory._set_dhcp_provider')
     @mock.patch('ironic.common.dhcp_factory.DHCPFactory.clean_dhcp')
@@ -638,6 +827,7 @@ class ISCSIDeployTestCase(db_base.DbTestCase):
     def test_get_clean_steps(self, mock_get_clean_steps):
         # Test getting clean steps
         self.config(group='deploy', erase_devices_priority=10)
+        self.config(group='deploy', erase_devices_metadata_priority=5)
         mock_steps = [{'priority': 10, 'interface': 'deploy',
                        'step': 'erase_devices'}]
         self.node.driver_internal_info = {'agent_url': 'foo'}
@@ -648,7 +838,8 @@ class ISCSIDeployTestCase(db_base.DbTestCase):
             mock_get_clean_steps.assert_called_once_with(
                 task, interface='deploy',
                 override_priorities={
-                    'erase_devices': 10})
+                    'erase_devices': 10,
+                    'erase_devices_metadata': 5})
         self.assertEqual(mock_steps, steps)
 
     @mock.patch.object(deploy_utils, 'agent_execute_clean_step', autospec=True)
@@ -659,111 +850,89 @@ class ISCSIDeployTestCase(db_base.DbTestCase):
             agent_execute_clean_step_mock.assert_called_once_with(
                 task, {'some-step': 'step-info'})
 
-    def test_heartbeat(self):
-        with task_manager.acquire(self.context, self.node.uuid,
-                                  shared=True) as task:
-            self.driver.deploy.heartbeat(task, 'url')
-            self.assertFalse(task.shared)
-
-
-class TestVendorPassthru(db_base.DbTestCase):
-
-    def setUp(self):
-        super(TestVendorPassthru, self).setUp()
-        mgr_utils.mock_the_extension_manager()
-        self.driver = driver_factory.get_driver("fake")
-        self.driver.vendor = iscsi_deploy.VendorPassthru()
-        self.node = obj_utils.create_test_node(
-            self.context, driver='fake',
-            instance_info=INST_INFO_DICT,
-            driver_info=DRV_INFO_DICT,
-            driver_internal_info=DRV_INTERNAL_INFO_DICT,
-        )
-        self.node.driver_internal_info['agent_url'] = 'http://1.2.3.4:1234'
-        self.task = mock.MagicMock(spec=task_manager.TaskManager)
-        self.task.shared = False
-        self.task.node = self.node
-        self.task.driver = self.driver
-        self.task.context = self.context
-
-    def test_vendor_routes(self):
-        expected = ['heartbeat']
-        with task_manager.acquire(self.context, self.node.uuid,
-                                  shared=True) as task:
-            vendor_routes = task.driver.vendor.vendor_routes
-            self.assertIsInstance(vendor_routes, dict)
-            self.assertEqual(sorted(expected), sorted(list(vendor_routes)))
-
-    def test_driver_routes(self):
-        expected = ['lookup']
-        with task_manager.acquire(self.context, self.node.uuid,
-                                  shared=True) as task:
-            driver_routes = task.driver.vendor.driver_routes
-            self.assertIsInstance(driver_routes, dict)
-            self.assertEqual(sorted(expected), sorted(list(driver_routes)))
-
-    @mock.patch.object(agent_base_vendor.BaseAgentVendor,
+    @mock.patch.object(agent_base_vendor.AgentDeployMixin,
                        'reboot_and_finish_deploy', autospec=True)
     @mock.patch.object(iscsi_deploy, 'do_agent_iscsi_deploy', autospec=True)
     def test_continue_deploy_netboot(self, do_agent_iscsi_deploy_mock,
                                      reboot_and_finish_deploy_mock):
 
+        self.node.provision_state = states.DEPLOYWAIT
+        self.node.target_provision_state = states.ACTIVE
+        self.node.save()
         uuid_dict_returned = {'root uuid': 'some-root-uuid'}
         do_agent_iscsi_deploy_mock.return_value = uuid_dict_returned
-        self.driver.vendor.continue_deploy(self.task)
-        do_agent_iscsi_deploy_mock.assert_called_once_with(
-            self.task, self.driver.vendor._client)
-        reboot_and_finish_deploy_mock.assert_called_once_with(
-            mock.ANY, self.task)
+        with task_manager.acquire(self.context, self.node.uuid) as task:
+            with mock.patch.object(
+                    task.driver.boot, 'prepare_instance') as m_prep_instance:
+                task.driver.deploy.continue_deploy(task)
+                do_agent_iscsi_deploy_mock.assert_called_once_with(
+                    task, task.driver.deploy._client)
+                reboot_and_finish_deploy_mock.assert_called_once_with(
+                    mock.ANY, task)
+                m_prep_instance.assert_called_once_with(task)
 
-    @mock.patch.object(agent_base_vendor.BaseAgentVendor,
+    @mock.patch.object(fake.FakeManagement, 'set_boot_device', autospec=True)
+    @mock.patch.object(agent_base_vendor.AgentDeployMixin,
                        'reboot_and_finish_deploy', autospec=True)
-    @mock.patch.object(agent_base_vendor.BaseAgentVendor,
+    @mock.patch.object(agent_base_vendor.AgentDeployMixin,
                        'configure_local_boot', autospec=True)
     @mock.patch.object(iscsi_deploy, 'do_agent_iscsi_deploy', autospec=True)
     def test_continue_deploy_localboot(self, do_agent_iscsi_deploy_mock,
                                        configure_local_boot_mock,
-                                       reboot_and_finish_deploy_mock):
+                                       reboot_and_finish_deploy_mock,
+                                       set_boot_device_mock):
 
         self.node.instance_info = {
             'capabilities': {'boot_option': 'local'}}
+        self.node.provision_state = states.DEPLOYWAIT
+        self.node.target_provision_state = states.ACTIVE
         self.node.save()
         uuid_dict_returned = {'root uuid': 'some-root-uuid'}
         do_agent_iscsi_deploy_mock.return_value = uuid_dict_returned
 
-        self.driver.vendor.continue_deploy(self.task)
-        do_agent_iscsi_deploy_mock.assert_called_once_with(
-            self.task, self.driver.vendor._client)
-        configure_local_boot_mock.assert_called_once_with(
-            self.task.driver.vendor, self.task, root_uuid='some-root-uuid',
-            efi_system_part_uuid=None)
-        reboot_and_finish_deploy_mock.assert_called_once_with(
-            self.task.driver.vendor, self.task)
+        with task_manager.acquire(self.context, self.node.uuid) as task:
+            task.driver.deploy.continue_deploy(task)
+            do_agent_iscsi_deploy_mock.assert_called_once_with(
+                task, task.driver.deploy._client)
+            configure_local_boot_mock.assert_called_once_with(
+                task.driver.deploy, task, root_uuid='some-root-uuid',
+                efi_system_part_uuid=None, prep_boot_part_uuid=None)
+            reboot_and_finish_deploy_mock.assert_called_once_with(
+                task.driver.deploy, task)
+            set_boot_device_mock.assert_called_once_with(
+                mock.ANY, task, device=boot_devices.DISK, persistent=True)
 
-    @mock.patch.object(agent_base_vendor.BaseAgentVendor,
+    @mock.patch.object(fake.FakeManagement, 'set_boot_device', autospec=True)
+    @mock.patch.object(agent_base_vendor.AgentDeployMixin,
                        'reboot_and_finish_deploy', autospec=True)
-    @mock.patch.object(agent_base_vendor.BaseAgentVendor,
+    @mock.patch.object(agent_base_vendor.AgentDeployMixin,
                        'configure_local_boot', autospec=True)
     @mock.patch.object(iscsi_deploy, 'do_agent_iscsi_deploy', autospec=True)
     def test_continue_deploy_localboot_uefi(self, do_agent_iscsi_deploy_mock,
                                             configure_local_boot_mock,
-                                            reboot_and_finish_deploy_mock):
+                                            reboot_and_finish_deploy_mock,
+                                            set_boot_device_mock):
 
         self.node.instance_info = {
             'capabilities': {'boot_option': 'local'}}
+        self.node.provision_state = states.DEPLOYWAIT
+        self.node.target_provision_state = states.ACTIVE
         self.node.save()
         uuid_dict_returned = {'root uuid': 'some-root-uuid',
                               'efi system partition uuid': 'efi-part-uuid'}
         do_agent_iscsi_deploy_mock.return_value = uuid_dict_returned
 
-        self.driver.vendor.continue_deploy(self.task)
-        do_agent_iscsi_deploy_mock.assert_called_once_with(
-            self.task, self.driver.vendor._client)
-        configure_local_boot_mock.assert_called_once_with(
-            self.task.driver.vendor, self.task, root_uuid='some-root-uuid',
-            efi_system_part_uuid='efi-part-uuid')
-        reboot_and_finish_deploy_mock.assert_called_once_with(
-            self.task.driver.vendor, self.task)
+        with task_manager.acquire(self.context, self.node.uuid) as task:
+            task.driver.deploy.continue_deploy(task)
+            do_agent_iscsi_deploy_mock.assert_called_once_with(
+                task, task.driver.deploy._client)
+            configure_local_boot_mock.assert_called_once_with(
+                task.driver.deploy, task, root_uuid='some-root-uuid',
+                efi_system_part_uuid='efi-part-uuid', prep_boot_part_uuid=None)
+            reboot_and_finish_deploy_mock.assert_called_once_with(
+                task.driver.deploy, task)
+            set_boot_device_mock.assert_called_once_with(
+                mock.ANY, task, device=boot_devices.DISK, persistent=True)
 
 
 # Cleanup of iscsi_deploy with pxe boot interface
@@ -773,11 +942,10 @@ class CleanUpFullFlowTestCase(db_base.DbTestCase):
         self.config(image_cache_size=0, group='pxe')
 
         # Configure node
-        mgr_utils.mock_the_extension_manager(driver="fake_pxe")
         instance_info = INST_INFO_DICT
         instance_info['deploy_key'] = 'fake-56789'
         self.node = obj_utils.create_test_node(
-            self.context, driver='fake_pxe',
+            self.context, boot_interface='pxe', deploy_interface='iscsi',
             instance_info=instance_info,
             driver_info=DRV_INFO_DICT,
             driver_internal_info=DRV_INTERNAL_INFO_DICT,
@@ -836,7 +1004,7 @@ class CleanUpFullFlowTestCase(db_base.DbTestCase):
     @mock.patch('ironic.common.dhcp_factory.DHCPFactory._set_dhcp_provider')
     @mock.patch('ironic.common.dhcp_factory.DHCPFactory.clean_dhcp')
     @mock.patch.object(pxe, '_get_instance_image_info', autospec=True)
-    @mock.patch.object(pxe, '_get_deploy_image_info', autospec=True)
+    @mock.patch.object(pxe, '_get_image_info', autospec=True)
     def test_clean_up_with_master(self, mock_get_deploy_image_info,
                                   mock_get_instance_image_info,
                                   clean_dhcp_mock, set_dhcp_provider_mock):
@@ -850,7 +1018,8 @@ class CleanUpFullFlowTestCase(db_base.DbTestCase):
             task.driver.deploy.clean_up(task)
             mock_get_instance_image_info.assert_called_with(task.node,
                                                             task.context)
-            mock_get_deploy_image_info.assert_called_with(task.node)
+            mock_get_deploy_image_info.assert_called_with(task.node,
+                                                          mode='deploy')
             set_dhcp_provider_mock.assert_called_once_with()
             clean_dhcp_mock.assert_called_once_with(task)
         for path in ([self.kernel_path, self.image_path, self.config_path]
