@@ -100,6 +100,16 @@ class PassthruDecoratorTestCase(base.TestCase):
         self.assertNotEqual(inst1.driver_routes['driver_noexception']['func'],
                             inst2.driver_routes['driver_noexception']['func'])
 
+    @mock.patch.object(driver_base.LOG, 'warning')
+    def test_old_async_warning(self, mock_log_warning):
+        driver_base.passthru(['POST'], async=True)
+        mock_log_warning.assert_called_once()
+
+    @mock.patch.object(driver_base.LOG, 'warning')
+    def test_new_async_call_without_warning(self, mock_log_warning):
+        driver_base.passthru(['POST'], async_call=True)
+        mock_log_warning.assert_not_called()
+
 
 class CleanStepDecoratorTestCase(base.TestCase):
 
@@ -226,7 +236,14 @@ class CleanStepTestCase(base.TestCase):
         method_args_mock = mock.MagicMock(spec_set=[])
         task_mock = mock.MagicMock(spec_set=[])
 
-        class TestClass(driver_base.BaseInterface):
+        class BaseTestClass(driver_base.BaseInterface):
+            def get_properties(self):
+                return {}
+
+            def validate(self, task):
+                pass
+
+        class TestClass(BaseTestClass):
             interface_type = 'test'
 
             @driver_base.clean_step(priority=0)
@@ -240,7 +257,7 @@ class CleanStepTestCase(base.TestCase):
             def not_clean_method(self, task):
                 pass
 
-        class TestClass2(driver_base.BaseInterface):
+        class TestClass2(BaseTestClass):
             interface_type = 'test2'
 
             @driver_base.clean_step(priority=0)
@@ -254,7 +271,7 @@ class CleanStepTestCase(base.TestCase):
             def not_clean_method2(self, task):
                 pass
 
-        class TestClass3(driver_base.BaseInterface):
+        class TestClass3(BaseTestClass):
             interface_type = 'test3'
 
             @driver_base.clean_step(priority=0, abortable=True, argsinfo={
@@ -337,6 +354,127 @@ class CleanStepTestCase(base.TestCase):
         method_args_mock.assert_called_once_with(task_mock, **args)
 
 
+class DeployStepTestCase(base.TestCase):
+    def test_get_and_execute_deploy_steps(self):
+        # Create a fake Driver class, create some deploy steps, make sure
+        # they are listed correctly, and attempt to execute one of them
+
+        method_mock = mock.MagicMock(spec_set=[])
+        method_args_mock = mock.MagicMock(spec_set=[])
+        task_mock = mock.MagicMock(spec_set=[])
+
+        class BaseTestClass(driver_base.BaseInterface):
+            def get_properties(self):
+                return {}
+
+            def validate(self, task):
+                pass
+
+        class TestClass(BaseTestClass):
+            interface_type = 'test'
+
+            @driver_base.deploy_step(priority=0)
+            def deploy_zero(self, task):
+                pass
+
+            @driver_base.deploy_step(priority=10)
+            def deploy_ten(self, task):
+                method_mock(task)
+
+            def not_deploy_method(self, task):
+                pass
+
+        class TestClass2(BaseTestClass):
+            interface_type = 'test2'
+
+            @driver_base.deploy_step(priority=0)
+            def deploy_zero2(self, task):
+                pass
+
+            @driver_base.deploy_step(priority=20)
+            def deploy_twenty(self, task):
+                method_mock(task)
+
+            def not_deploy_method2(self, task):
+                pass
+
+        class TestClass3(BaseTestClass):
+            interface_type = 'test3'
+
+            @driver_base.deploy_step(priority=0, argsinfo={
+                                     'arg1': {'description': 'desc1',
+                                              'required': True}})
+            def deploy_zero3(self, task, **kwargs):
+                method_args_mock(task, **kwargs)
+
+            @driver_base.deploy_step(priority=15, argsinfo={
+                                     'arg10': {'description': 'desc10'}})
+            def deploy_fifteen(self, task, **kwargs):
+                pass
+
+            def not_deploy_method3(self, task):
+                pass
+
+        obj = TestClass()
+        obj2 = TestClass2()
+        obj3 = TestClass3()
+
+        self.assertEqual(2, len(obj.get_deploy_steps(task_mock)))
+        # Ensure the steps look correct
+        self.assertEqual(10, obj.get_deploy_steps(task_mock)[0]['priority'])
+        self.assertEqual('test', obj.get_deploy_steps(
+            task_mock)[0]['interface'])
+        self.assertEqual('deploy_ten', obj.get_deploy_steps(
+            task_mock)[0]['step'])
+        self.assertEqual(0, obj.get_deploy_steps(task_mock)[1]['priority'])
+        self.assertEqual('test', obj.get_deploy_steps(
+            task_mock)[1]['interface'])
+        self.assertEqual('deploy_zero', obj.get_deploy_steps(
+            task_mock)[1]['step'])
+
+        # Ensure the second obj has different deploy steps
+        self.assertEqual(2, len(obj2.get_deploy_steps(task_mock)))
+        # Ensure the steps look correct
+        self.assertEqual(20, obj2.get_deploy_steps(task_mock)[0]['priority'])
+        self.assertEqual('test2', obj2.get_deploy_steps(
+            task_mock)[0]['interface'])
+        self.assertEqual('deploy_twenty', obj2.get_deploy_steps(
+            task_mock)[0]['step'])
+        self.assertEqual(0, obj2.get_deploy_steps(task_mock)[1]['priority'])
+        self.assertEqual('test2', obj2.get_deploy_steps(
+            task_mock)[1]['interface'])
+        self.assertEqual('deploy_zero2', obj2.get_deploy_steps(
+            task_mock)[1]['step'])
+        self.assertIsNone(obj2.get_deploy_steps(task_mock)[0]['argsinfo'])
+
+        # Ensure the third obj has different deploy steps
+        self.assertEqual(2, len(obj3.get_deploy_steps(task_mock)))
+        self.assertEqual(15, obj3.get_deploy_steps(task_mock)[0]['priority'])
+        self.assertEqual('test3', obj3.get_deploy_steps(
+            task_mock)[0]['interface'])
+        self.assertEqual('deploy_fifteen', obj3.get_deploy_steps(
+            task_mock)[0]['step'])
+        self.assertEqual({'arg10': {'description': 'desc10'}},
+                         obj3.get_deploy_steps(task_mock)[0]['argsinfo'])
+        self.assertEqual(0, obj3.get_deploy_steps(task_mock)[1]['priority'])
+        self.assertEqual(obj3.interface_type, obj3.get_deploy_steps(
+            task_mock)[1]['interface'])
+        self.assertEqual('deploy_zero3', obj3.get_deploy_steps(
+            task_mock)[1]['step'])
+        self.assertEqual({'arg1': {'description': 'desc1', 'required': True}},
+                         obj3.get_deploy_steps(task_mock)[1]['argsinfo'])
+
+        # Ensure we can execute the function.
+        obj.execute_deploy_step(task_mock, obj.get_deploy_steps(task_mock)[0])
+        method_mock.assert_called_once_with(task_mock)
+
+        args = {'arg1': 'val1'}
+        deploy_step = {'interface': 'test3', 'step': 'deploy_zero3',
+                       'args': args}
+        obj3.execute_deploy_step(task_mock, deploy_step)
+        method_args_mock.assert_called_once_with(task_mock, **args)
+
+
 class MyRAIDInterface(driver_base.RAIDInterface):
 
     def create_configuration(self, task):
@@ -399,5 +537,94 @@ class TestDeployInterface(base.TestCase):
         deploy = fake.FakeDeploy()
         deploy.heartbeat(mock.Mock(node=mock.Mock(uuid='uuid',
                                                   driver='driver')),
-                         'url')
+                         'url', '3.2.0')
         self.assertTrue(mock_log.called)
+
+
+class MyBIOSInterface(driver_base.BIOSInterface):
+
+    def get_properties(self):
+        pass
+
+    def validate(self, task):
+        pass
+
+    def apply_configuration(self, task, settings):
+        pass
+
+    def factory_reset(self, task):
+        pass
+
+    def cache_bios_settings(self, task):
+        pass
+
+
+class TestBIOSInterface(base.TestCase):
+
+    @mock.patch.object(MyBIOSInterface, 'cache_bios_settings', autospec=True)
+    def test_apply_configuration_wrapper(self, cache_bios_settings_mock):
+        bios = MyBIOSInterface()
+        task_mock = mock.MagicMock()
+
+        bios.apply_configuration(task_mock, "")
+        cache_bios_settings_mock.assert_called_once_with(bios, task_mock)
+
+    @mock.patch.object(MyBIOSInterface, 'cache_bios_settings', autospec=True)
+    def test_factory_reset_wrapper(self, cache_bios_settings_mock):
+        bios = MyBIOSInterface()
+        task_mock = mock.MagicMock()
+
+        bios.factory_reset(task_mock)
+        cache_bios_settings_mock.assert_called_once_with(bios, task_mock)
+
+
+class TestBootInterface(base.TestCase):
+
+    def test_validate_rescue_default_impl(self):
+        boot = fake.FakeBoot()
+        task_mock = mock.MagicMock(spec_set=['node'])
+
+        self.assertRaises(exception.UnsupportedDriverExtension,
+                          boot.validate_rescue, task_mock)
+
+
+class TestManagementInterface(base.TestCase):
+
+    def test_inject_nmi_default_impl(self):
+        management = fake.FakeManagement()
+        task_mock = mock.MagicMock(spec_set=['node'])
+
+        self.assertRaises(exception.UnsupportedDriverExtension,
+                          management.inject_nmi, task_mock)
+
+    def test_get_supported_boot_modes_default_impl(self):
+        management = fake.FakeManagement()
+        task_mock = mock.MagicMock(spec_set=['node'])
+
+        self.assertRaises(exception.UnsupportedDriverExtension,
+                          management.get_supported_boot_modes, task_mock)
+
+    def test_set_boot_mode_default_impl(self):
+        management = fake.FakeManagement()
+        task_mock = mock.MagicMock(spec_set=['node'])
+
+        self.assertRaises(exception.UnsupportedDriverExtension,
+                          management.set_boot_mode, task_mock, 'whatever')
+
+    def test_get_boot_mode_default_impl(self):
+        management = fake.FakeManagement()
+        task_mock = mock.MagicMock(spec_set=['node'])
+
+        self.assertRaises(exception.UnsupportedDriverExtension,
+                          management.get_boot_mode, task_mock)
+
+
+class TestBareDriver(base.TestCase):
+
+    def test_class_variables(self):
+        self.assertEqual(['boot', 'deploy', 'management', 'network', 'power'],
+                         driver_base.BareDriver().core_interfaces)
+        self.assertEqual(
+            ['bios', 'console', 'inspect', 'raid', 'rescue', 'storage'],
+            driver_base.BareDriver().optional_interfaces
+        )
