@@ -22,12 +22,10 @@ respective external libraries' actually being present.
 Any external library required by a third-party driver should be mocked here.
 Current list of mocked libraries:
 
-- seamicroclient
-- ipminative
 - proliantutils
 - pysnmp
 - scciclient
-- oneview_client
+- hpOneView
 - pywsman
 - python-dracclient
 """
@@ -43,24 +41,6 @@ from ironic.tests.unit.drivers import third_party_driver_mock_specs \
     as mock_specs
 
 
-# attempt to load the external 'seamicroclient' library, which is
-# required by the optional drivers.modules.seamicro module
-seamicroclient = importutils.try_import("seamicroclient")
-if not seamicroclient:
-    smc = mock.MagicMock(spec_set=mock_specs.SEAMICRO_SPEC)
-    smc.client = mock.MagicMock(spec_set=mock_specs.SEAMICRO_CLIENT_MOD_SPEC)
-    smc.exceptions = mock.MagicMock(spec_set=mock_specs.SEAMICRO_EXC_SPEC)
-    smc.exceptions.ClientException = Exception
-    smc.exceptions.UnsupportedVersion = Exception
-    sys.modules['seamicroclient'] = smc
-    sys.modules['seamicroclient.client'] = smc.client
-    sys.modules['seamicroclient.exceptions'] = smc.exceptions
-
-# if anything has loaded the seamicro driver yet, reload it now that
-# the external library has been mocked
-if 'ironic.drivers.modules.seamicro' in sys.modules:
-    six.moves.reload_module(sys.modules['ironic.drivers.modules.seamicro'])
-
 # IPMITool driver checks the system for presence of 'ipmitool' binary during
 # __init__. We bypass that check in order to run the unit tests, which do not
 # depend on 'ipmitool' being on the system.
@@ -68,26 +48,6 @@ ipmitool.TIMING_SUPPORT = False
 ipmitool.DUAL_BRIDGE_SUPPORT = False
 ipmitool.SINGLE_BRIDGE_SUPPORT = False
 
-pyghmi = importutils.try_import("pyghmi")
-if not pyghmi:
-    p = mock.MagicMock(spec_set=mock_specs.PYGHMI_SPEC)
-    p.exceptions = mock.MagicMock(spec_set=mock_specs.PYGHMI_EXC_SPEC)
-    p.exceptions.IpmiException = Exception
-    p.ipmi = mock.MagicMock(spec_set=mock_specs.PYGHMI_IPMI_SPEC)
-    p.ipmi.command = mock.MagicMock(spec_set=mock_specs.PYGHMI_IPMICMD_SPEC)
-    p.ipmi.command.Command = mock.MagicMock(spec_set=[])
-    sys.modules['pyghmi'] = p
-    sys.modules['pyghmi.exceptions'] = p.exceptions
-    sys.modules['pyghmi.ipmi'] = p.ipmi
-    sys.modules['pyghmi.ipmi.command'] = p.ipmi.command
-    # FIXME(deva): the next line is a hack, because several unit tests
-    #              actually depend on this particular string being present
-    #              in pyghmi.ipmi.command.boot_devices
-    p.ipmi.command.boot_devices = {'pxe': 4}
-
-if 'ironic.drivers.modules.ipminative' in sys.modules:
-    six.moves.reload_module(sys.modules['ironic.drivers.modules.ipminative'])
-
 proliantutils = importutils.try_import('proliantutils')
 if not proliantutils:
     proliantutils = mock.MagicMock(spec_set=mock_specs.PROLIANTUTILS_SPEC)
@@ -100,6 +60,8 @@ if not proliantutils:
     proliantutils.exception.IloError = type('IloError', (Exception,), {})
     command_exception = type('IloCommandNotSupportedError', (Exception,), {})
     proliantutils.exception.IloCommandNotSupportedError = command_exception
+    proliantutils.exception.IloCommandNotSupportedInBiosError = type(
+        'IloCommandNotSupportedInBiosError', (Exception,), {})
     proliantutils.exception.InvalidInputError = type(
         'InvalidInputError', (Exception,), {})
     proliantutils.exception.ImageExtractionFailed = type(
@@ -108,42 +70,29 @@ if not proliantutils:
         six.moves.reload_module(sys.modules['ironic.drivers.ilo'])
 
 
-oneview_client = importutils.try_import('oneview_client')
-if not oneview_client:
-    oneview_client = mock.MagicMock(spec_set=mock_specs.ONEVIEWCLIENT_SPEC)
-    sys.modules['oneview_client'] = oneview_client
-    sys.modules['oneview_client.client'] = oneview_client.client
-    sys.modules['oneview_client.client.Client'] = mock.MagicMock(
-        spec_set=mock_specs.ONEVIEWCLIENT_CLIENT_CLS_SPEC
-    )
-    states = mock.MagicMock(
-        spec_set=mock_specs.ONEVIEWCLIENT_STATES_SPEC,
-        ONEVIEW_POWER_OFF='Off',
-        ONEVIEW_POWERING_OFF='PoweringOff',
-        ONEVIEW_POWER_ON='On',
-        ONEVIEW_POWERING_ON='PoweringOn',
-        ONEVIEW_RESETTING='Resetting',
-        ONEVIEW_ERROR='error')
-    sys.modules['oneview_client.states'] = states
-    sys.modules['oneview_client.exceptions'] = oneview_client.exceptions
-    sys.modules['oneview_client.utils'] = oneview_client.utils
-    oneview_client.exceptions.OneViewException = type('OneViewException',
-                                                      (Exception,), {})
-    sys.modules['oneview_client.models'] = oneview_client.models
-    if 'ironic.drivers.oneview' in sys.modules:
-        six.moves.reload_module(sys.modules['ironic.drivers.modules.oneview'])
+hpOneView = importutils.try_import('hpOneView')
+if not hpOneView:
+    hpOneView = mock.MagicMock(spec_set=mock_specs.HPE_ONEVIEW_SPEC)
+    sys.modules['hpOneView'] = hpOneView
+    sys.modules['hpOneView.oneview_client'] = hpOneView.oneview_client
+    sys.modules['hpOneView.resources'] = hpOneView.resources
+    sys.modules['hpOneView.exceptions'] = hpOneView.exceptions
+    hpOneView.exceptions.HPOneViewException = type('HPOneViewException',
+                                                   (Exception,), {})
+sys.modules['hpOneView.oneview_client'].OneViewClient = mock.MagicMock(
+    spec_set=mock_specs.HPE_ONEVIEW_CLS_SPEC
+)
+if 'ironic.drivers.oneview' in sys.modules:
+    six.moves.reload_module(sys.modules['ironic.drivers.modules.oneview'])
 
 
-# attempt to load the external 'pywsman' library, which is required by
-# the optional drivers.modules.amt module
-pywsman = importutils.try_import('pywsman')
-if not pywsman:
-    pywsman = mock.MagicMock(spec_set=mock_specs.PYWSMAN_SPEC)
-    sys.modules['pywsman'] = pywsman
-    # Now that the external library has been mocked, if anything had already
-    # loaded any of the drivers, reload them.
-    if 'ironic.drivers.modules.amt' in sys.modules:
-        six.moves.reload_module(sys.modules['ironic.drivers.modules.amt'])
+redfish = importutils.try_import('redfish')
+if not redfish:
+    redfish = mock.MagicMock(spec_set=mock_specs.REDFISH_SPEC)
+    sys.modules['redfish'] = redfish
+if 'ironic.drivers.oneview' in sys.modules:
+    six.moves.reload_module(sys.modules['ironic.drivers.modules.oneview'])
+
 
 # attempt to load the external 'python-dracclient' library, which is required
 # by the optional drivers.modules.drac module
@@ -168,19 +117,6 @@ if not dracclient:
     if 'ironic.drivers.modules.drac' in sys.modules:
         six.moves.reload_module(sys.modules['ironic.drivers.modules.drac'])
 
-# attempt to load the external 'iboot' library, which is required by
-# the optional drivers.modules.iboot module
-iboot = importutils.try_import("iboot")
-if not iboot:
-    ib = mock.MagicMock(spec_set=mock_specs.IBOOT_SPEC)
-    ib.iBootInterface = mock.MagicMock(spec_set=[])
-    sys.modules['iboot'] = ib
-
-# if anything has loaded the iboot driver yet, reload it now that the
-# external library has been mocked
-if 'ironic.drivers.modules.iboot' in sys.modules:
-    six.moves.reload_module(sys.modules['ironic.drivers.modules.iboot'])
-
 
 # attempt to load the external 'pysnmp' library, which is required by
 # the optional drivers.modules.snmp module
@@ -188,18 +124,11 @@ pysnmp = importutils.try_import("pysnmp")
 if not pysnmp:
     pysnmp = mock.MagicMock(spec_set=mock_specs.PYWSNMP_SPEC)
     sys.modules["pysnmp"] = pysnmp
-    sys.modules["pysnmp.entity"] = pysnmp.entity
-    sys.modules["pysnmp.entity.rfc3413"] = pysnmp.entity.rfc3413
-    sys.modules["pysnmp.entity.rfc3413.oneliner"] = (
-        pysnmp.entity.rfc3413.oneliner)
-    sys.modules["pysnmp.entity.rfc3413.oneliner.cmdgen"] = (
-        pysnmp.entity.rfc3413.oneliner.cmdgen)
+    sys.modules["pysnmp.hlapi"] = pysnmp.hlapi
     sys.modules["pysnmp.error"] = pysnmp.error
     pysnmp.error.PySnmpError = Exception
-    sys.modules["pysnmp.proto"] = pysnmp.proto
-    sys.modules["pysnmp.proto.rfc1902"] = pysnmp.proto.rfc1902
     # Patch the RFC1902 integer class with a python int
-    pysnmp.proto.rfc1902.Integer = int
+    pysnmp.hlapi.Integer = int
 
 
 # if anything has loaded the snmp driver yet, reload it now that the
@@ -224,6 +153,8 @@ if not scciclient:
         UNMOUNT_CD=mock.sentinel.UNMOUNT_CD,
         MOUNT_FD=mock.sentinel.MOUNT_FD,
         UNMOUNT_FD=mock.sentinel.UNMOUNT_FD)
+    sys.modules['scciclient.irmc.elcm'] = mock.MagicMock(
+        spec_set=mock_specs.SCCICLIENT_IRMC_ELCM_SPEC)
 
 
 # if anything has loaded the iRMC driver yet, reload it now that the
@@ -232,7 +163,7 @@ if 'ironic.drivers.modules.irmc' in sys.modules:
     six.moves.reload_module(sys.modules['ironic.drivers.modules.irmc'])
 
 
-# install mock object to prevent 'iscsi_irmc' and 'agent_irmc' from
+# install mock object to prevent the irmc-virtual-media boot interface from
 # checking whether NFS/CIFS share file system is mounted or not.
 irmc_boot = importutils.import_module(
     'ironic.drivers.modules.irmc.boot')
@@ -242,21 +173,6 @@ irmc_boot.check_share_fs_mounted_patcher = mock.patch(
 irmc_boot.check_share_fs_mounted_patcher.return_value = None
 
 
-pyremotevbox = importutils.try_import('pyremotevbox')
-if not pyremotevbox:
-    pyremotevbox = mock.MagicMock(spec_set=mock_specs.PYREMOTEVBOX_SPEC)
-    pyremotevbox.exception = mock.MagicMock(
-        spec_set=mock_specs.PYREMOTEVBOX_EXC_SPEC)
-    pyremotevbox.exception.PyRemoteVBoxException = Exception
-    pyremotevbox.exception.VmInWrongPowerState = Exception
-    pyremotevbox.vbox = mock.MagicMock(
-        spec_set=mock_specs.PYREMOTEVBOX_VBOX_SPEC)
-    sys.modules['pyremotevbox'] = pyremotevbox
-    if 'ironic.drivers.modules.virtualbox' in sys.modules:
-        six.moves.reload_module(
-            sys.modules['ironic.drivers.modules.virtualbox'])
-
-
 ironic_inspector_client = importutils.try_import('ironic_inspector_client')
 if not ironic_inspector_client:
     ironic_inspector_client = mock.MagicMock(
@@ -298,3 +214,51 @@ if not imcsdk:
     if 'ironic.drivers.modules.cimc' in sys.modules:
         six.moves.reload_module(
             sys.modules['ironic.drivers.modules.cimc'])
+
+
+sushy = importutils.try_import('sushy')
+if not sushy:
+    sushy = mock.MagicMock(
+        spec_set=mock_specs.SUSHY_CONSTANTS_SPEC,
+        BOOT_SOURCE_TARGET_PXE='Pxe',
+        BOOT_SOURCE_TARGET_HDD='Hdd',
+        BOOT_SOURCE_TARGET_CD='Cd',
+        BOOT_SOURCE_TARGET_BIOS_SETUP='BiosSetup',
+        SYSTEM_POWER_STATE_ON='on',
+        SYSTEM_POWER_STATE_POWERING_ON='powering on',
+        SYSTEM_POWER_STATE_OFF='off',
+        SYSTEM_POWER_STATE_POWERING_OFF='powering off',
+        RESET_ON='on',
+        RESET_FORCE_OFF='force off',
+        RESET_GRACEFUL_SHUTDOWN='graceful shutdown',
+        RESET_GRACEFUL_RESTART='graceful restart',
+        RESET_FORCE_RESTART='force restart',
+        RESET_NMI='nmi',
+        BOOT_SOURCE_ENABLED_CONTINUOUS='continuous',
+        BOOT_SOURCE_ENABLED_ONCE='once',
+        BOOT_SOURCE_MODE_BIOS='bios',
+        BOOT_SOURCE_MODE_UEFI='uefi'
+    )
+
+    sys.modules['sushy'] = sushy
+    if 'ironic.drivers.modules.redfish' in sys.modules:
+        six.moves.reload_module(
+            sys.modules['ironic.drivers.modules.redfish'])
+
+xclarity_client = importutils.try_import('xclarity_client')
+if not xclarity_client:
+    xclarity_client = mock.MagicMock(spec_set=mock_specs.XCLARITY_SPEC)
+    sys.modules['xclarity_client'] = xclarity_client
+    sys.modules['xclarity_client.client'] = xclarity_client.client
+    states = mock.MagicMock(
+        spec_set=mock_specs.XCLARITY_STATES_SPEC,
+        STATE_POWER_ON="power on",
+        STATE_POWER_OFF="power off",
+        STATE_POWERING_ON="powering_on",
+        STATE_POWERING_OFF="powering_off")
+    sys.modules['xclarity_client.states'] = states
+    sys.modules['xclarity_client.exceptions'] = xclarity_client.exceptions
+    sys.modules['xclarity_client.utils'] = xclarity_client.utils
+    xclarity_client.exceptions.XClarityException = type('XClarityException',
+                                                        (Exception,), {})
+    sys.modules['xclarity_client.models'] = xclarity_client.models
